      program por
      implicit none

#ifdef MPI
      include 'mpif.h'
#endif

      include '../include/allprogs.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'
      include '../include/mem_time.inc'
      include '../include/pifhead.inc'
      
      type(pifhead_t) imghead

      logical :: confine_3D = .false.

      character(len=17)  :: subname = 'POR main routine'
      character(len=256) :: infile, new_name

      real :: zero=0.0
      real weight, mapprjbig, imgprjbig, enlargefact, center(2)

      integer :: total_count=0, new_origin=0, new_orient=0, new_hand=0
      integer :: stb_ornt_stb_orgn=0, stb_ornt_new_orgn=0
      integer :: new_ornt_stb_orgn=0, new_ornt_new_orgn=0
      integer :: flp_ornt_stb_orgn=0, flp_ornt_new_orgn=0
      integer mcw, id, local_max, local_count, buffer_size(0:255)
      integer nptles, err_code, ptle_count(256)
      integer fft_ann, i, j, k, inunit, alloc_stat, ierr
      integer first_iq, last_iq, max_iq_slab,
     +     first_z,last_z, first_hk,last_hk, max_hk_slab, num_hkl,
     +     den_min_x, den_max_x, den_min_y, den_max_y, 
     +     den_min_z, den_max_z, mem_size, 
     +     min_xyz, max_xyz, hkl_max, max_z_slab,
     +     first_rr_out, last_rr_out, max_rr_out_slab,
     +     first_u, last_u, max_u_slab,
     +     first_y, last_y, max_y_slab
      integer nrffts_1_row, nrffts_1_col, nrffts_1_omega,
     +     nrffts_2_row, nrffts_2_col, nrffts_2_omega
      
      double precision elapsed_time

      integer*8 clock1, clock2

      integer, allocatable   :: indx(:)
      real, allocatable      :: pixels(:), theta(:), phi(:), omega(:)
      real, allocatable      :: x(:), y(:), mag(:), score(:)
      real, allocatable      :: delf_maj_ptle(:), delf_min_ptle(:) !rss
      real, allocatable      :: ang_maj_ptle(:) !rss
      real, allocatable      :: ctf_val(:,:), abs_ctf(:,:), disef(:,:)
      real, allocatable      :: map3d(:,:,:), rffta_omega(:)
      real, allocatable      :: real_results_local(:,:)
      character, allocatable :: orient_flag(:), origin_flag(:)
      character, allocatable :: char_results_local(:,:)
      complex, allocatable   :: fft3d(:), fft_klh(:)

#ifdef MPI
      integer stat(mpi_status_size)
#endif

      PROGNAME = 'POR '

#ifdef MPI
      mcw = mpi_comm_world
      call MPI_INIT (mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_INIT')
      call MPI_COMM_RANK (mcw, iam, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_COMM_RANK')
      call MPI_COMM_SIZE (mcw, num_nodes, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_COMM_SIZE')
      if (num_nodes > max_numnodes) then
         write(*, 9001) num_nodes, max_numnodes
 9001    format(//' Requested number of nodes = NUM_NODES    = ',i10/
     +        ' Available number of nodes = MAX_NUMNODES = ',i10/)
         call error_stop(progname, subname,  
     +        'NUM_NODES requested exceeds MAX_NUMNODES')
      endif
#else
      iam = 0
      num_nodes = 1
#endif

      call system_clock(clock1)

      if(iam == 0) then
         write(*,*) 'PO2R: Local orientation and origin refinement'
         write(*,*) 'Parallel version run on ', num_nodes , ' CPUs'
         write(*,*) 'For doc see: http://cryoem.ucsd.edu/programs.shtm'
         write(*,*) 

         call parse_cmd_line(infile)

         if(len_trim(infile) == 0) then
            inunit = 5
            call info(inunit)
         else
            inunit=17
            open(file=infile, unit=inunit)
            call info(inunit)
            close(inunit)
         endif

C     Check the map, and get the col and row number. 3D model should
C     have the smallest pixel_size and largest box_size compared with
C     boxed images.

         call get_map_dimensions(in_3d_name, npix_col, npix_row, 
     $        angstrom_per_pix, ierr)
         if(ierr > 0) then
            call error_stop('POR', 'POR main',
     +           'Error in 3D map mode, it should be 20 or 40')
         endif

C     3D map should have larger dimension than image box size
         if(npix_row < box_out) then
            call error_stop(progname, subname, 
     $           'NPIX_ROW is less than BOX_OUT')
         endif
         
C     Check the map pixel size
         if(angstrom_per_pix < 0.01 .or. angstrom_per_pix > 100.0) then
            write(*,*) '3D map has nonsensical pixel size'
            write(*,*) 'Use pixel size from particle parameter files'
            angstrom_per_pix = PIXSIZ_OUT
         elseif ((angstrom_per_pix - PIXSIZ_OUT) > 10E-5) then
            call error_stop(progname, subname, 
     $           'Pixel size from 3D map is too large')
         else
            PIXSIZ_OUT = angstrom_per_pix
         endif
         
      endif

#ifdef MPI
C     Broadcast parameters from root to all other processes
      call bcast_parameters 
#endif

      call intlz_params(hkl_max, num_hkl, min_xyz, max_xyz, 
     +     nrffta_row, nrffta_col, nrffta_omega, nfc_2d_row, 
     +     nfc_2d_col, nrffts_1_row, nrffts_1_col, 
     +     nrffts_1_omega, nrffts_2_row, nrffts_2_col, 
     +     nrffts_2_omega, den_min_x, den_max_x, den_min_y, 
     +     den_max_y, den_min_z, den_max_z,
     +     confine_3D)

      if(iam == 0) then
         print *, ' zero_fill after  call of INTLZ_PARAMS =',zero_fill
         print *, ' nrffta_row after call of INTLZ_PARAMS =',nrffta_row
      endif

      call intlz_arrays(hkl_max, min_xyz, max_xyz, min_z, max_z,
     $     first_hk, last_hk, max_hk_slab, first_iq, last_iq,
     $     max_iq_slab, first_rr_out, last_rr_out, max_rr_out_slab,
     $     first_u, last_u, max_u_slab, first_y, last_y, max_y_slab, 
     $     first_z, last_z, max_z_slab, npix_row)

C     Calculate 3D transform of input map

      mem_size = npix_col * npix_row * max_y_slab
      allocate(map3d(npix_col, npix_row, max_y_slab), 
     +     stat = alloc_stat)
      mem_map3d = dble(4 * mem_size) 
      if (alloc_stat > 0) call error_stop (progname, subname,
     +     '3D density map memory allocation failed')
      mem_total = mem_total + mem_map3d
      mem_peak  = max(mem_peak, mem_total)
      
      call read_map(in_3d_name, map3d, npix_col, npix_row,
     +     max_ y_slab, mapprjbig)

C     3d DFT, and exchange 8 quater

      allocate(rffta_omega(nrffta_omega ), stat = alloc_stat)
      if(alloc_stat > 0) call error_stop (progname, subname,
     +     'Error in allocating memory for nrffta_omega')
      
      call vrffti( nrffta_col, rffta_omega )
      
      mem_size = (hkl_max+1)* num_hkl * (last_y - first_y + 1) 
      allocate(fft3d(mem_size), stat = alloc_stat)
      mem_fft3d = dble(8 * mem_size) 
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D density fft map memory allocation failed')
      endif
      mem_total = mem_total + mem_fft3d
      mem_peak  = max(mem_peak, mem_total)

C     FFT3D, (X,Y,Z) real -> (H,K,Z) complex(0:Hmax, Kmin:Kmax, Z_slab
C     Zmin:Zmax) because I change intlz_params.f to make hkl_max =
C     nfc_2d_row = nfc_2d_col

      do i = 0, last_y - first_y
         center(1) = (npix_row-1)/2.0
         center(2) = (npix_col-1)/2.0
         call fft_2dfft(map3d(1,1,i+1), 
     +        npix_row, npix_col, center,
     +        nrffta_omega, rffta_omega, nrffta_row, nrffta_col, 
     +        hkl_max, hkl_max, FFT3D(i*(hkl_max+1)*num_hkl+1))
      enddo

      deallocate(map3d)
      mem_total = mem_total - mem_map3d

C     Exchange data between nodes from Z_slab to H_slab

#ifdef MPI
      if (num_nodes > 1) then   
         mem_size = (hkl_max+1)*num_hkl*(last_y - first_y + 1)
         allocate(fft_klh(mem_size), stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +           '3D density fft exchange memory 1 allocation failed')
         endif
         mem_total = mem_total + 8*mem_size 
         mem_peak  = max(mem_peak, mem_total)
         
         call arrange_3d_1(FFT3D, fft_klh, hkl_max)
         
         deallocate(fft3d)
         mem_total = mem_total - mem_fft3d
         mem_fft3d = 8 * mem_size
         
         mem_size = (last_iq-first_iq+1)*num_hkl*npix_row
         allocate(fft3d(mem_size), stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +           '3D density fft exchange memory 2 allocation failed')
         endif
         mem_total = mem_total + 8*mem_size 
         mem_peak  = max(mem_peak, mem_total)
         
         call exch_3d_1(fft_klh, hkl_max, FFT3D, npix_row)

         deallocate(fft_klh)
         mem_total = mem_total - mem_fft3d
         mem_fft3d = 8 * mem_size
      endif
#endif

Cji   1D fft, (H,K,Z)-->(K,L,H) complex(Kmin:Kmax, Lmin:Lmax, H_slab 0:Hmax)

      mem_size = (last_iq-first_iq+1)* num_hkl**2
      allocate(fft_klh(mem_size), stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D density fft 1dfft allocation failed')
      endif
      mem_total = mem_total +8*mem_size 
      mem_peak  = max(mem_peak, mem_total)

      CALL cfft_1d(FFT3D, fft_klh, hkl_max, npix_row,
     +     nrffta_omega, rffta_omega)

      deallocate(fft3d)
      mem_total = mem_total - mem_fft3d
      mem_fft3d = 8 * mem_size

C     Gather 3Dfft fft_klh in to a full 3D fft map fft3d use up to
C     annulus to save memory.

      fft_ann = int(real(nrffta_row)*PIXSIZ_OUT/res_max) + 1
      mem_size = (2*fft_ann+1)*(2*fft_ann+1)*(fft_ann+1)
      allocate(fft3d(mem_size), stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D fft memory allocation failed')
      endif
      mem_total = mem_total + 8 * mem_size
      mem_peak  = max(mem_peak, mem_total)

C     gather3d has been modified to handle both serial and MPI versions
      call gather3d(fft_klh, hkl_max, fft3d, fft_ann)
      
      deallocate(fft_klh)
      mem_total = mem_total - mem_fft3d
      mem_fft3d = dble(8 * mem_size) 

#ifdef MPI
      call mpi_bcast(mapprjbig, 1, mpi_real, 0, mcw, mpi_err)
#endif

C     Initialize the CTF mem for half fft_size matrix
      
      mem_size = (nfc_2d_row +1) * (nfc_2d_col * 2 +1)
      allocate(ctf_val(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     abs_ctf(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     disef(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col), 
     +     stat = alloc_stat)
      mem_ctf_val = dble( 12 * mem_size)
      
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +      ' Memory allocation failed for ctf_val, abs_ctf, and disef')
      endif

      mem_total = mem_total + mem_ctf_val
      mem_peak  = max(mem_peak, mem_total)

      mem_cmpt_intrps = 0

C     Broadcast parameter file names and image file names

#ifdef MPI
      call mpi_bcast(param_fnames, filename_len*pfilesmx, mpi_character,
     $     0, mcw, mpi_err)
      call mpi_bcast(img_fnames, filename_len*pfilesmx, mpi_character,
     $     0, mcw, mpi_err)
#endif

C     Determine numbers of particles in parameter files
      if(iam == 0) then
         oddeven = 0
         ptle_count = 0
         call get_ptle_counts(nfiles, filename_len, param_fnames,
     $        oddeven, ptle_count)
         nptles = sum(ptle_count)
      endif

C     Broadcast numbers of particles in parameter files
#ifdef MPI
      call mpi_bcast(nfiles,     1,      mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nptles,     1,      mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(ptle_count, nfiles, mpi_integer, 0, mcw, mpi_err)
#endif

C     Calculate buffer sizes
      buffer_size = 0
      do i=0,num_nodes-1
         buffer_size(i) = nptles/num_nodes
         if(i < mod(nptles,num_nodes)) then
            buffer_size(i) = buffer_size(i) + 1
         endif
      enddo

C     Calculate the maximum number of particles assigned to a process
      local_max = maxval(buffer_size)

C     Allocate global and local arrays for particle parameters
      allocate(indx(nptles), theta(nptles), phi(nptles), omega(nptles),
     $     x(nptles), y(nptles), mag(nptles), score(nptles),
     $     orient_flag(nptles), origin_flag(nptles),
     $     real_results_local(7,local_max),
     $     char_results_local(2,local_max))
      
      if(per_ptle_ctf == 1) then !rss
         allocate(delf_maj_ptle(nptles), delf_min_ptle(nptles), !rss
     $        ang_maj_ptle(nptles)) !rss
      endif !rss

C     Get particle parameters
      if(iam == 0) then
         oddeven = 0
         if(per_ptle_ctf == 1) then  !rss
            call get_ptle_ioomctf(nfiles, filename_len, param_fnames, !rss
     $           oddeven, nptles, nbox, indx, theta, phi, omega, !rss
     $           x, y, mag, delf_maj_ptle, delf_min_ptle, ang_maj_ptle) !rss
         else                   !rss
            call get_ptle_ioom(nfiles, filename_len, param_fnames, !rss
     $           oddeven, nptles, nbox, indx, theta, phi, omega, !rss
     $           x, y, mag) !rss
         endif                  !rss
      endif
      
C     Distrubute particle information to all other processes
#ifdef MPI
      call mpi_bcast(indx,  nptles, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(theta, nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(phi,   nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(omega, nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(x,     nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(y,     nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(mag,   nptles, mpi_real,    0, mcw, mpi_err)
      if(per_ptle_ctf == 1) then !rss
         call mpi_bcast(delf_maj_ptle, nptles, mpi_real, 0,mcw,mpi_err) !rss
         call mpi_bcast(delf_min_ptle, nptles, mpi_real, 0,mcw,mpi_err) !rss
         call mpi_bcast(ang_maj_ptle,  nptles, mpi_real, 0,mcw,mpi_err) !rss
      endif !rss
#endif

C     Main loop for obtaining new origins and orientations
      id = 0
      local_count = 0
      do i=1,nfiles

         npix_row         =  nbox(i)
         npix_col         =  nbox(i)
         angstrom_per_pix = pixsizs(i)

         allocate(pixels(npix_col*npix_row))

         call pif_open(img_fnames(i), pix_file_unit, 'r')
         call get_1st_ortid(j, param_fnames(i))
         call read_1_pif(j,npix_row,npix_col, pix_file_unit, imgprjbig)
         call pif_close(pix_file_unit)
         enlargefact = mapprjbig/(imgprjbig*sqrt(real(nrffta_col)))
         
         ctf_val = 1.0
         abs_ctf = 1.0
         disef   = 1.0
         
         if (funcweight > 0.0) then 
            call disef_para(disef, nfc_2d_row, nfc_2d_col, funcweight,
     $           angstrom_per_pix, units)
         endif

         if (ctfmode > 0 .and. ctfmode <= 4)  then
            weight = enlargefact
            filter = 0 ! Required so that model multipiled by CTF
            weight = 1.0/enlargefact
      
            if(per_ptle_ctf == 0) then !rss
               call ctf_para(ctf_val, abs_ctf, 
     $              0, nfc_2d_row, -nfc_2d_col, nfc_2d_col, .true.,
     $              ctfmode, filter, tempfac, ctf_ff1, ctf_ff2,
     $              pixsizs(i), units, volts(i), 
     $              delf_majs(i), delf_mins(i), 
     $              ang_majs(i), ampfacs(i),
     $              cs_coeffs(i), weight, res_max, res_max)
            endif !rss
         endif
         
         do j=1,ptle_count(i)
            id = id + 1
            if(iam == mod(id-1,num_nodes)) then
               local_count = local_count + 1

               call pif_open(img_fnames(i), pix_file_unit, 'r')
               imghead%lunum = pix_file_unit
               call pif_read_gh(imghead)
               imghead%imgnum = indx(id)
               call pif_read_dh(imghead)
               imghead%imgnum = indx(id)
               call read_pixel_frame(imghead, npix_row*npix_col, 
     $              pixels, err_code)
               call pif_close(pix_file_unit)
               
               if(per_ptle_ctf == 1) then !rss
                  call ctf_para(ctf_val, abs_ctf, !rss
     $                 0, nfc_2d_row, -nfc_2d_col, nfc_2d_col, .true., !rss
     $                 ctfmode, filter, tempfac, ctf_ff1, ctf_ff2, !rss
     $                 pixsizs(i), units, volts(i), !rss
     $                 delf_maj_ptle(id), delf_min_ptle(id), !rss
     $                 ang_maj_ptle(id), ampfacs(i), !rss
     $                 cs_coeffs(i), weight, res_max, res_max) !rss
               endif            !rss

               call cmpt_ort(fft3d, hkl_max, fft_ann, ctf_val,
     $              abs_ctf, disef, pixels, 
     $              theta(id), phi(id), omega(id),
     $              x(id), y(id), mag(id),
     $              score(id), orient_flag(id), origin_flag(id))
               
               real_results_local(1,local_count) = theta(id)
               real_results_local(2,local_count) = phi(id)
               real_results_local(3,local_count) = omega(id)
               real_results_local(4,local_count) = x(id)
               real_results_local(5,local_count) = y(id)
               real_results_local(6,local_count) = mag(id)
               real_results_local(7,local_count) = score(id)
               char_results_local(1,local_count) = orient_flag(id)
               char_results_local(2,local_count) = origin_flag(id)
               
            endif
         enddo

         deallocate(pixels)
      enddo

C     Collect results back onto root process
#ifdef MPI
      do i=1,num_nodes-1
         if(iam == i) then
            call mpi_send(real_results_local, 7*buffer_size(i),
     $            mpi_real, 0, i, mcw, mpi_err)
            call mpi_send(char_results_local, 2*buffer_size(i),
     $            mpi_character, 0, i, mcw, mpi_err)
         endif
         if(iam == 0) then
            call mpi_recv(real_results_local, 7*buffer_size(i),
     $            mpi_real, i, i, mcw, stat, mpi_err)
            call mpi_recv(char_results_local,2* buffer_size(i),
     $            mpi_character, i, i, mcw, stat, mpi_err)

            do j = 1, buffer_size(i)
               k = i+1+(j-1)*num_nodes
               theta(k)       = real_results_local(1,j)
               phi(k)         = real_results_local(2,j)
               omega(k)       = real_results_local(3,j)
               x(k)           = real_results_local(4,j)
               y(k)           = real_results_local(5,j)
               mag(k)         = real_results_local(6,j)
               score(k)       = real_results_local(7,j)
               orient_flag(k) = char_results_local(1,j)
               origin_flag(k) = char_results_local(2,j)
            enddo
         endif
      enddo
#endif

C     Write out new particle parameter files from root process
      if(iam == 0) then
         id = 0
         do i=1,nfiles
            call increment_fname(param_fnames(i), new_name)
            open(unit=11, file=new_name)
            write(11,'(A)') trim(img_fnames(i))
            write(11,1010) pixsizs(i), units, volts(i), 
     +           ampfacs(i), delf_majs(i), delf_mins(i), 
     +           ang_majs(i), cs_coeffs(i)
            if(per_ptle_ctf == 1) then
               do j=1,ptle_count(i)
                  id = id+1
                  write(11,1235) indx(id), theta(id), phi(id), omega(id),
     $                 x(id), y(id), mag(id), score(id), zero, zero,
     $                 delf_maj_ptle(id), delf_min_ptle(id),
     $                 ang_maj_ptle(id),
     $                 orient_flag(id), origin_flag(id)
               enddo
            else
               do j=1,ptle_count(i)
                  id = id+1
                  write(11,1234) indx(id), theta(id), phi(id), omega(id),
     $                 x(id), y(id), mag(id), score(id), zero, zero,
     $                 orient_flag(id), origin_flag(id)
               enddo
            endif
            close(11)
         enddo
      endif

 1010 format(1X,f8.4,',',i2,',',f10.1,',',4(f10.4,','),f8.2) 
 1234 format(I7,',',4(F8.3,','),F7.3,',',F6.3,',',
     +     F10.3,',',F5.1,',',F5.1,',',1x,A1,',',1x,A1)
 1235 format(I7,',',4(F8.3,','),F7.3,',',F6.3,',',
     +     F10.3,',',F5.1,',',F5.1,',',F7.4,',',F7.4,',',F7.2,',',
     +     1x,A1,',',1x,A1)

      deallocate(ctf_val, abs_ctf, disef, fft3d, indx,
     $     theta, phi, omega, x, y, mag, score, orient_flag,
     $     origin_flag, real_results_local, char_results_local,
     $     rffta_omega)

      if(per_ptle_ctf == 1) then !rss
         deallocate(delf_maj_ptle, delf_min_ptle, ang_maj_ptle) !rss
      endif !rss

      call system_clock(clock2)
      time = elapsed_time(clock1, clock2)

      if(iam == 0) then
         write(*,*) 'Total run time = ', time
      endif

#ifdef MPI
      call mpi_finalize(mpi_err)
#endif

      end program por
