C  ******************************************************************
C  *                                                                *
C  *                 *****     PIF  -> MRC      *****               *
C  *                                                                *
C  *    LOGICAL UNITS: 7    = INPUT PIF FILE                        *
C  *                   11   = OUTPUT  MRC FILE                      *
C  *    MBS, Jan. 2003                                              *
C  *    Latest changes MBS, Sep. 2003 (Made output of odd dimns)    *
C  *                                                                *
C  ******************************************************************
C
      implicit none
C
      include '../include/pifhead.inc'
      type(pifhead_t) PIFhead
c
      character(len=1) readwrite(4)
      integer iargc
      integer*4 i, j, k, l, m, n, kstep
      integer*4 ncol, nrow, nsec
      integer*4 nx, ny, nz
      integer*4 imgOffset
      integer*4 img_size
      integer*4 img_plus_header
      integer*4 percent
      real*4    dtot,dval
      character MRCname*64,PIFname*64

      integer   alloc_stat
      character(len=1) , allocatable :: map_in_bt(:)
      integer*2, allocatable :: map_in_st(:)
      real*4, allocatable    :: map_in(:)
      real*4, allocatable    :: map_out(:)

      integer*4 iunit_in,iunit_out
      integer*4 mode, ilen_mrc, ilen_pif
      integer*4 nxyz(3)
      integer*4 nxyzst(3)
      data      nxyzst/3*0/
      real*4    dmin, dmax, dmean
      real*4    tmin, tmax, tmean
      common//nx,ny,nz
      equivalence (nx,nxyz)
      integer*4 htype
      integer*4 nbytes
      integer*4 labels(20,10)
      character label*80
      character dat*9,tim*8
      
      iunit_in  = 7
      iunit_out = 11

      write(*,'(a)')'********    PIF2MRC    ********'
      write(*,'(a)')
     1    '***   Will reduce images/maps to original dimensions  ****'

c
c----Read input data from user
c

      if(iargc() < 2) then
 12      write(*,'($,a)') 'PIF Filename: '
         read (*,'(a)', err=12) PIFname
 13      write(*,'($,a)') 'MRC Filename: '
         read (*,'(a)', err=13) MRCname
      else
         call getarg(1,PIFname)
         call getarg(2,MRCname)
      endif

      ilen_pif = min(len_trim(PIFname), 64)
      if(ilen_pif.gt.64) then
         write(*,*)'PIF file name too long'
         stop
      else
         do i = ilen_pif+1, 64
            PIFname(i:i) = ' '
         end do
      end if

      ilen_mrc = min(len_trim(MRCname), 64)
      if(ilen_mrc.gt.64) then
         write(*,*)'MRC file name too long'
         stop
      else
         do i = ilen_mrc+1, 64
            MRCname(i:i) = ' '
         end do
      end if
      
      readwrite(1) = 'r' 
      Pifhead%lunum = iunit_in

      call pif_open (PIFname(1:ilen_pif), Pifhead%lunum, readwrite)
c     
c   read PIF header, determine memory requirements
c
      call pif_read_gh (PIFhead)

      if(Pifhead%htype .ne. 1) then
         write(*,*)"Could not convert stack of different sizes"
         stop
      end if
      ncol = Pifhead%numcols
      nrow = Pifhead%numrows
      nsec = Pifhead%numsecs
      mode = Pifhead%mode
c      if(Pifhead%even .eq. 1) then
      if(Pifhead%mrcX .eq. 1) then
         ncol = ncol - 1
      end if
c      if(Pifhead%even .eq. 1) then
      if(Pifhead%mrcY .eq. 1) then
         nrow = nrow - 1
      end if
c      if(Pifhead%even .eq. 1) then
      if(Pifhead%mrcZ .eq. 1) then
         nsec = nsec - 1
      end if

      if((mode .ne. 20) .and.(mode .ne. 21)) nsec = Pifhead%nimg
      nx = ncol
      ny = nrow
      nz = nsec
         
      if(mode .gt. 4) then
         if(     mode .eq.  6) then
            mode = 0
         else if(mode .eq.  7) then
            mode = 1
         else if(mode .eq.  8) then
            mode = 3
         else if(mode .eq.  9) then
            mode = 2
         else if(mode .eq. 10) then
            mode = 4
         else if(mode .eq. 20) then
            mode = 2
         else if(mode .eq. 21) then
            mode = 2
         else if(mode .eq. 46) then
            mode = 2
         else
            write(*,*)"Could not convert files with that mode:", mode
            stop
         end if
      end if

C  *************************************************************
C  * Allocate core memory for maps: output map is always r*4 (MRC req);
C  * Input size depends on the PIF mode
C  *************************************************************

      allocate(map_in(Pifhead%numcols*Pifhead%numrows), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAP_IN'
         stop
      endif

      allocate(map_out(ncol*nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAP_OUT'
         stop
      endif

      if(mode .ne. 2) then
         if(mode .eq. 0) then
            nbytes = 1
ccc            allocate(map_in_bt(Pifhead%numcols*Pifhead%numrows),
ccc     $           stat=alloc_stat)
ccc            if (alloc_stat > 0) then
ccc               write(*,'(a)')'Could not allocate memory for PIF'
ccc               stop
ccc            end if
         else if (mode .eq. 1) then
            allocate(map_in_st(Pifhead%numcols*Pifhead%numrows),
     $           stat=alloc_stat)
            if (alloc_stat > 0) then
               write(*,'(a)')'Could not allocate memory for PIF'
               stop
            end if
            nbytes = 2
         else if (mode .eq. 3) then
            write(*,*)"Warning: not fully tested yet!"
            allocate(map_out((ncol+2)*nrow))
            nbytes = 4
         else if (mode .eq. 4) then
            write(*,*)"Warning: not fully tested yet!"
            allocate(map_out(2*(ncol+2)*nrow))
            nbytes = 8
         else
            write(*,*)"Mode not implemented yet:",mode
            stop
         end if
      end if

      kstep = nint(nsec/10.0)
      if(kstep.le.0) kstep = 1


C  ******************************************
C  *    Set up MRC format map output file   *
C  ******************************************
      call imopen(iunit_out,MRCname(1:ilen_mrc),'unknown')
ccc      call date(dat)
ccc      call time(tim)
      call icrhdr(iunit_out,nxyz,nxyz,mode,labels,0)
ccc      encode(80,1500,label) dat,tim
ccc 1500      format('Created with PIF2MRC',7x,a9,2x,a8)

      label = 'Created with PIF2MRC'

      dmin = 0.
      dmax = 0.
      dmean = 0.

      call iwrhdr(iunit_out,label,-1,dmin,dmax,dmean)

c      call pif_list_head (PIFhead)
C     
C     Read all sections from the input map files
C     
      write (*,*)
      write (*,*)' Read PIF in core, write MRC out...'

      dmin  =  1.e10
      dmax  = -1.e10
      dmean =  0.
      imgOffset = 1024
      img_size = Pifhead%numcols*Pifhead%numrows
      img_plus_header = nbytes*img_size + 512
      do k = 1,nsec
            Pifhead%imgNum  =  k
         if     ((Pifhead%mode .eq.0) .or.
     1         (Pifhead%mode .eq.6)) then
ccc            call pif_read_byte_image(Pifhead%luNum,map_in_bt,
ccc     1         imgOffset,Pifhead%numcols,Pifhead%numrows,1)
         else if((Pifhead%mode .eq.1) .or.
     1         (Pifhead%mode .eq.7)) then
            call pif_read_short_image(Pifhead%luNum,map_in_st,
     1         imgOffset,Pifhead%numcols,Pifhead%numrows,1)
         else if((Pifhead%mode .eq.2) .or.
     1         (Pifhead%mode .eq.9)) then
            call pif_read_imgi4 (PIFhead,map_in,
     1         Pifhead%numcols,Pifhead%numrows,k)
         else if((Pifhead%mode .eq.3) .or.
     1         (Pifhead%mode .eq.8)) then
            call pif_read_seci2(PIFhead,map_in,
     1         Pifhead%numcols+1,Pifhead%numrows,k)      !FFT      
         else if((Pifhead%mode .eq.4) .or.
     1         (Pifhead%mode .eq.10)) then
            call pif_read_seci4(PIFhead,map_in,
     1         Pifhead%numcols+1,Pifhead%numrows,k) !FFT
         else if(Pifhead%mode .eq.20) then
            call pif_read_seci2(PIFhead,map_in,
     1         Pifhead%numcols,Pifhead%numrows,k)
         else if((Pifhead%mode .eq.21) .or.
     1         (Pifhead%mode .eq.22)) then
            call pif_read_seci4(PIFhead,map_in,
     1         Pifhead%numcols,Pifhead%numrows,k)
         else if(Pifhead%mode .eq.46) then
            call pif_read_boxi4(PIFhead,map_in)
         else
            write(*,*)"Wrong mode",mode
            stop
         end if
         imgOffset = imgOffset + img_plus_header !Calculate new image offset
c         write(*,*)"k, mode:",k,mode

         if(mode .eq.0) then
ccc            do i = 1, img_size
ccc             map_in(i) = map_in_bt(i)
ccc            end do
         else if(mode .eq.1) then
            do i = 1, img_size
             map_in(i) = map_in_st(i)
            end do
         end if
c       Stuff input map into the output of possibly smaller dimensions:
            j = 0
            l = 0
            m = 0
            n = 0
            do i = 1, img_size
             m = m + 1
             if(m.gt.Pifhead%numcols) then
                m = 1
                n = n + 1
             end if
             if(m.le.ncol .and. n.le.nrow) then
                j = j + 1
                if(j.gt.ncol) then
                   j = 1
                   l = l + 1
                   if(l.ge.nrow) l=nrow-1
                end if
                map_out(j+l*ncol) = map_in(m+n*Pifhead%numcols)
             end if
            end do

         if(mode .le. 2) then
            call iclden(map_out,ncol,nrow,1,ncol,1,nrow,
     1         tmin,tmax,tmean)
         else
            call iclcdn(map_out,ncol/2+1,nrow,1,ncol/2,1,nrow,
     1         tmin,tmax,tmean)
         end if
         dmin = min(dmin,tmin)
         dmax = max(dmax,tmax)
         dmean = dmean + tmean
         call iwrsec(iunit_out,map_out)
         if(mod(k,kstep).eq.0) then
            percent = nint(k*100./nsec)
            write(*,'("+",i3,''% complete'')') percent
         end if
      end do
      dmean = dmean/nsec
      call iwrhdr(iunit_out,label,-1,dmin,dmax,dmean)

      write(*,*)'Done'

C     Free memory
      if(allocated(map_in))    deallocate(map_in)
      if(allocated(map_out))   deallocate(map_out)
      if(allocated(map_in_st)) deallocate(map_in_st)
      if(allocated(map_in_bt)) deallocate(map_in_bt)

C     Close files
      call pif_close(Pifhead%lunum)
      call imclose(iunit_out)

      return
      end
       
C ==========================================================
