

/*******************************************************************
 * FIXPIF - 
 *
 * Rob Ashmore
 * Biology Department
 * Purdue University
 * Sept 1998
 *
 *******************************************************************/
/* File fixpif.c */

// Is this a comment??

#define VERSION "2.00_20021127"

#define IS_MAIN_MODULE 1
#define IS_ROBEM_MODULE 1

#include "fixpif.h"
#include "../libR/libUtil.h"


/************************************************************************
 * OS transfer point.  The main routine does all the one-time setup and *
 * then calls XtAppMainLoop.                                            *
 ************************************************************************/
int main(argc, argv)
int argc;                           /* Command line argument count. */
char *argv[];                       /* Pointers to command line args. */
{/* MAIN */
  Arg arglst[22];
  XrmOptionDescRec options;
  Cardinal numOptions;
  int n;
  String fallbacks[] =
    { "*fontList:-dec-terminal-bold-r-normal--14-140-75-75-c-80-iso8859-1=tag",
	NULL
	};
  
  /* Show them our version number */
  /* printf ("Version[%s]\n",VERSION); */
  banner("fixpif");

  /* Init */
  currentHeader = 0;
  numImages     = 1;
  fileIsOpen    = FALSE;
  dirty         = FALSE;

  fileName.addr = (char *)calloc(2048,sizeof(char));
  fileName.length = 2048;

#ifdef sgi
  /* SGI is different Endian than VAX */
  endianSwitch= TRUE;
#endif

  /* Initialize MRM */
  MrmInitialize();                 

  XtToolkitInitialize();
  app_context = XtCreateApplicationContext();

  XtAppSetFallbackResources(app_context, fallbacks);

  /* Finally, open the display */
  dsply = XtOpenDisplay(app_context, NULL,
			"fixpif",
			"fixpif", NULL,
			0, &argc, argv);
  
  /* Add our remapping of keys to widgets */
  XtAppAddActions(app_context,rwa_action_table, 1); 

  createDefaultColormap();

  /* Explicitly call this class "fixpif" */
    if(FALSE)
    {/* DO the colormap thing */
      
      printf("Colormap here[%d]\n",color_map);

      argcnt = 0;  
      XtSetArg(args[argcnt], XmNcolormap, color_map); argcnt++;
      XtSetArg(args[argcnt], XmNvisual, visual); argcnt++; 
/*      XtSetArg(args[argcnt], XmNbackground, 0 ); argcnt++;  */ 
      XtSetArg(args[argcnt], XmNborderColor, 1); argcnt++;
      XtSetArg(args[argcnt], XmNdepth, 8); argcnt++; 

      toplevel_widget = XtVaAppCreateShell(
					   argv[0],
					   "fixpif",
					   topLevelShellWidgetClass, 
					   /* XmTopLevelShell or
					      whatever */
					   dsply,
					   XmNcolormap, color_map,
					   XmNvisual, visual,
					   XmNbackground, 0,
					   XmNborderColor, 1,
					   XmNdepth, 8,
					   NULL);
    }/* DO the colormap thing */
  else
    {/* CHILD */
      toplevel_widget = XtVaAppCreateShell(
					   argv[0],
					   "fixpif",
					   topLevelShellWidgetClass, 
					   /* XmTopLevelShell or
					      whatever */
					   dsply,
					   NULL);
    }/* CHILD */
  




  /***********************************************************************
   * Open the UID files (the output of the UIL compiler) in the hierarchy*
   ***********************************************************************/
  db_filename_vec[0] = (char *)malloc(sizeof(char)*512);
  /* Kludge this for now so the production user gets the right .UID */
  sprintf(db_filename_vec[0],"%s/fixpif.uid",getenv("EMDIR")); 
  printf("[%s]\n",db_filename_vec[0]);
  if (MrmOpenHierarchy(db_filename_num, /* Number of files. */
		       db_filename_vec,       /* Array of file names.  */
		       NULL,                  /* Default OS extenstion. */
		       &s_MrmHierarchy)       /* Pointer to returned MRM ID */
      !=MrmSUCCESS)
    s_error("can't open hierarchy");
  
  
  /* Register the items MRM needs to bind for us. */
  MrmRegisterNames(reglist, reglist_num);
  
  /* Go get the main part of the application. */
  if (MrmFetchWidget(s_MrmHierarchy, "S_MAIN_WINDOW", toplevel_widget,
		     &main_window_widget, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch main window");

  /* Go get the infoScreen part of the application. */
  if (MrmFetchWidget(s_MrmHierarchy, "infoMessageBox", toplevel_widget,
		     &infoBox, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch info window");
  XtUnmanageChild(XmMessageBoxGetChild(infoBox,XmDIALOG_HELP_BUTTON));


  /* Manage the main part and realize everything.  The interface comes up
   * on the display now.  Manage all of the screens now so we can alter
   * what we need to in XLibInitialize() - Unmanage the unneeded ones
   * thereafter.
   */
  XtManageChild(main_window_widget);
  XFlush(dsply);
  XtUnmanageChild(imageForm);

  XlibInitialize();

  /* Sit around forever waiting to process X-events.  We never leave
   * XtAppMainLoop. From here on, we only execute our callback routines. */
  XtAppMainLoop(app_context);

  return(0);

}/* MAIN */



/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/*************************************************************************
 * NON-Callback Routines - These rotuines are of the more generic C      *
 * language variety and are not called directly from X.  The callback    *
 * routines that are called by X do make heavy use of these, however.    *
 *************************************************************************/
/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/


/**************************************************************************
 * createDefaultColormap() - DO whatever is necessary to get us the desired 
 * colormap.
*/
void createDefaultColormap()
{/*  CREATEDEFAULTCOLORMAP */
  FILE *fColor;
  int num_visuals,screen;
  XVisualInfo * visual_info;
  XVisualInfo * vi;
  XVisualInfo vinfo;
  int class;
  Bool              contig;
  unsigned int      nplanes;
  int numColormaps;
  int success;
  int depth, status;
  float scaleFactor;
  XColor screen_def_rtn, exact_def_rtn;


  printf("Default colormap.\n");
/*  CREATEDefaultCOLORMAP */
  visual= DefaultVisual(dsply, DefaultScreen(dsply));
  visual_depth = (unsigned int)DefaultDepth(dsply, DefaultScreen(dsply));
  
/*  CREATEDefaultCOLORMAP */

  /* Pull the switch */
  hiColor = TRUE;

  
/*  CREATEDefaultCOLORMAP */


  /* Allocate the color cells */
  pixels_return= (unsigned long *)malloc(sizeof(unsigned long)*256);
  dummy_pixels_return= (unsigned long *)malloc(sizeof(unsigned long)*256);
  /* Allocate the color definition structs */
  if (color_defs == NULL) 
    color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  if (mirror_color_defs == NULL) 
    mirror_color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  if (gen_color_defs == NULL) 
    gen_color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
      
  if(masterProcess)
    {/* Create the colormap here is MASTER */


      printf("visual==[%d]\n",visual);
      
      color_map = XCreateColormap(dsply, DefaultRootWindow(dsply),
				  visual, AllocNone);
      printf("DEC colormap = [%d]\n",color_map);

#ifdef DOTHISHERE
      i = 0;
      XtSetArg(args[i], XtNcolormap, color_map); i++;
      XtSetArg(args[i], XmNvisual, visual); i++;
      XtSetArg(args[i], XmNdepth, visual_depth); i++;
      XtSetValues(toplevel_widget,args,i);
      XFlush(dsply);
#endif  /*ifdef DOTHISHERE */


/*  CREATEDefaultCOLORMAP */

      if (color_map == NULL) {
	fprintf (stderr, "Could not create color map for visual\n");
	exit (1);
      }
      
      /* Now let's query the DISPLAY until we get the maximum number of
       * contiguous colorcell entries in the colormap that we can .
       */
      success= FALSE;
      tryColors= 256;
      nplanes = 1;
      contig = True;
      plane_masks_return = (unsigned long *)malloc(sizeof(unsigned long)*
						   nplanes);
      dummy_plane_masks_return = (unsigned long *)malloc(sizeof(unsigned long)*
						   nplanes);
      nplanes = 0;   

/*  CREATEDefaultCOLORMAP */

      /* Init the color defs to point to each entry */
      for (i = 0; i <256 ; i++) 
	{/* Make the color map gray-scale */
	  color_defs[i].pixel= i;
	  gen_color_defs[i].pixel= i;
	  pixels_return[i] = i;
	}/* Make the color map gray-scale */

      /* Find out what's already there in the color table */
      XQueryColors(dsply, color_map, color_defs, 256);
      XQueryColors(dsply, color_map, gen_color_defs, 256);

      sprintf(outStr,"%s/cmap.robem",getenv("HOME"));
      fColor = fopen(outStr,"w");
      if(fColor)
	{/* Write it */
	  fwrite(&tryColors,sizeof(int),1,fColor);
	  for(i=0;i<tryColors;i++)
	    {/* Write it all out */
	      fwrite(&pixels_return[i],sizeof(unsigned long),1,fColor);
	    }/* Write it all out */
	}/* Write it */
      else
	{/* Trouble */
	  printf("ERROR - Cannot write colormap file.\n");
	}/* Trouble */
      fclose(fColor);

    }/* Create the colormap here is MASTER */
  else
    {/* Read from file */
      sprintf(outStr,"%s/cmap.robem",getenv("HOME"));
      fColor = fopen(outStr,"r");
      if(fColor)
	{/* Read it */
	  fread(&tryColors,sizeof(int),1,fColor);
	  for(i=0;i<tryColors;i++)
	    {/* Read it all out */
	      fread(&pixels_return[i],sizeof(unsigned long),1,fColor);
	    }/* Read it all out */
	}/* Read it */
      else
	{/* SOmethings wrong */
	  printf("ERROR - Cannot read colormap file.\n");
	}/* SOmethings wrong */
      fclose(fColor);
      
    }/* Read from file */

  
/*  CREATEDefaultCOLORMAP */

  /* Now change the colormap for drawing in pixels_return[GREEN,RED,BLUE] 
   * First calculate some fudge factors.
   */
  pixelColor= (65535.0/256.0);
  scaleFactor = 255.0 / ((float)tryColors - 7.0);
  
  /* Make these entries in the colormap GRAYSCALE */
  for(i=6;i<(tryColors);i++)
    {/* Do the colors we have been assigned */
      gen_color_defs[pixels_return[i]].pixel = pixels_return[i];
      gen_color_defs[pixels_return[i]].red   = 
	(((i-6.0) * scaleFactor) * pixelColor);
      gen_color_defs[pixels_return[i]].green =
	(((i-6.0) * scaleFactor) * pixelColor);
      gen_color_defs[pixels_return[i]].blue  = 
	(((i-6.0) * scaleFactor) * pixelColor);
      gen_color_defs[pixels_return[i]].flags = DoRed | DoGreen | DoBlue;
      color_defs[pixels_return[i]].pixel = pixels_return[i];
      color_defs[pixels_return[i]].red   = (unsigned short)
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].green = (unsigned short)
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].blue  = (unsigned short)
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].flags = DoRed | DoGreen | DoBlue;
      color_defs[pixels_return[i]].pad = 0;
      gen_color_defs[pixels_return[i]].pad = 0;
      if(masterProcess)
	success = XAllocColor (dsply, color_map, 
			       &color_defs[i]);
      pixels_return[i] = color_defs[i].pixel;
      if(hiColor)
	gen_color_defs[i].pixel =  i;
    }/* Do the colors we have been assigned */
  
  
/*  CREATEDefaultCOLORMAP */

  /* Now define our robem colors */
  color_defs[WHITE].pixel = WHITE;
  color_defs[WHITE].red   = 255 * pixelColor;
  color_defs[WHITE].green = 255 * pixelColor;
  color_defs[WHITE].blue  = 255 * pixelColor;
  color_defs[WHITE].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (dsply, color_map, &color_defs[WHITE]);
     pixels_return[WHITE] = color_defs[WHITE].pixel;}

  color_defs[YELLOW].pixel = YELLOW;
  color_defs[YELLOW].red   = 255 * pixelColor;
  color_defs[YELLOW].green = 255 * pixelColor;
  color_defs[YELLOW].blue  = 0 * pixelColor;
  color_defs[YELLOW].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (dsply, color_map, &color_defs[YELLOW]);
     pixels_return[YELLOW] = color_defs[YELLOW].pixel ;}

  color_defs[GREEN].pixel = GREEN;
  color_defs[GREEN].red   = 0;
  color_defs[GREEN].green = 255 * pixelColor;
  color_defs[GREEN].blue  = 0;
  color_defs[GREEN].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (dsply, color_map, &color_defs[GREEN]);
     pixels_return[GREEN] = color_defs[GREEN].pixel;}

  color_defs[pixels_return[RED]].pixel = RED;
  color_defs[pixels_return[RED]].red   = 255 * pixelColor;
  color_defs[pixels_return[RED]].green = 0;
  color_defs[pixels_return[RED]].blue  = 0;
  color_defs[pixels_return[RED]].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (dsply, color_map, &color_defs[pixels_return[RED]]);
     pixels_return[RED] = color_defs[RED].pixel;}

  color_defs[BLUE].pixel = BLUE;
  color_defs[BLUE].red   = 0;
  color_defs[BLUE].green = 0;
  color_defs[BLUE].blue  = 255 * pixelColor;
  color_defs[BLUE].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (dsply, color_map, &color_defs[BLUE]);
     pixels_return[BLUE] = color_defs[BLUE].pixel;}

  color_defs[PLACEHOLDER].pixel = PLACEHOLDER;
  color_defs[PLACEHOLDER].red   = 0 * pixelColor;
  color_defs[PLACEHOLDER].green = 255 * pixelColor;
  color_defs[PLACEHOLDER].blue  = 0;
  color_defs[PLACEHOLDER].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (dsply, color_map, &color_defs[PLACEHOLDER]);
     pixels_return[PLACEHOLDER] = color_defs[PLACEHOLDER].pixel;}

/*  CREATEDefaultCOLORMAP */

  color_defs[BLACK].pixel = BLACK;
  color_defs[BLACK].red   = 0 * pixelColor;
  color_defs[BLACK].green = 0 * pixelColor;
  color_defs[BLACK].blue  = 0 * pixelColor;
  color_defs[BLACK].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (dsply, color_map, &color_defs[BLACK]);
     pixels_return[BLACK] = color_defs[BLACK].pixel;}
  
 noHiColor:;

}/*  CREATEDEFAULTCOLORMAP */





/**************************************************************************
 * createColormap() - DO whatever is necessary to get us the desired 
 * colormap.
*/
void createColormap()
{/*  CREATECOLORMAP */
  FILE *fColor;
  int num_visuals,screen;
  XVisualInfo * visual_info;
  XVisualInfo * vi;
  XVisualInfo vinfo;
  int class;
  XVisualInfo * found_vi;
  Bool              contig;
  unsigned int      nplanes;
  Colormap cMaps[100];
  int numColormaps;
  int success;
  float scaleFactor;
  XColor screen_def_rtn, exact_def_rtn;



#define TRYTHIS_HDHA 1
#ifdef TRYTHIS_HDHA
  /* [1.2] Find Appropriate Visual */
  visual_info = XGetVisualInfo (dsply, VisualNoMask, NULL, &num_visuals);
  if (visual_info == NULL || num_visuals == 0) 
    {
      fprintf (stderr, "XGetVisualInfo failed\n");
      exit (1);
    }	    
  found_vi = NULL;
  
  /* [1.3] Find visual such that:
   * desired_class = 2;                    ==2 for PseudoColor 
   * desired_depth *= 3;			needed if separate colors
   * desired_depth_delta = 3;
   *
   *	1. desired_depth is as large as possible (up to true desired depth)
   *	2. visual depth is the smallest of those supported >= desired_depth
   *	3. visual class is the `most desired' for the image type
   */
  vi = visual_info;
  for(i=0;i<num_visuals;i++)
    {/* Look for one we want */
      printf("Found depth[%d] class[%d]\n",vi->depth,vi->class);
      if (vi->depth == 8 && (vi->class==3)) 
	found_vi = vi;
      if(found_vi)
	break;
      vi++;
    }/* Look for one we want */
  
  if (found_vi == NULL) 
    printf("Could not find > 8 bit visual type\n");
  else
    {/* Get screen */
      Screen *scr;

      /* Use this for our 24 bit screens */
      screen = found_vi->screen;
      root_window = RootWindow (dsply, screen); 

      scr = ScreenOfDisplay(dsply, screen);
      printf("Min Cmaps[%d]\n",XMinCmapsOfScreen(scr));
      printf("Max Cmaps[%d]\n",maxCmaps= XMaxCmapsOfScreen(scr));

    }/* Get screen */
#endif /* TRYTHIS */

  /* Get visual for default 8 bits screens */
  visual= DefaultVisual(dsply, DefaultScreen(dsply));
  visual_depth = (unsigned int)DefaultDepth(dsply, DefaultScreen(dsply));
  
  /* Allocate the color cells */
  pixels_return= (unsigned long *)malloc(sizeof(unsigned long)*256);
  dummy_pixels_return= (unsigned long *)malloc(sizeof(unsigned long)*256);
  /* Allocate the color definition structs */
  if (color_defs == NULL) 
    color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  if (mirror_color_defs == NULL) 
    mirror_color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  if (gen_color_defs == NULL) 
    gen_color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
      
  masterProcess = 1;
      
  if(masterProcess)
    {/* Create the colormap here is MASTER */
      int depth, status;


#ifdef DOTHECREATECOLORMAP


#define  USE_DEC_HELPLINE_CODE 1
#ifdef USE_DEC_HELPLINE_CODE
      class = PseudoColor;
      screen = DefaultScreen(dsply);
      depth = 8;
      status = XMatchVisualInfo(dsply, screen, depth, class, &vinfo);
      if( status == 0) 
	{
	  printf("No Matching Visuals\n");
	  exit(1);
	}
      printf("Visual ID: %x\n", vinfo.visualid);
      visual = vinfo.visual;

      printf("visual==[%d]\n",visual);
      
      color_map = XCreateColormap(dsply, DefaultRootWindow(dsply),
				  visual, 0);
      printf("DEC colormap = [%d]\n",color_map);

#else
      if(found_vi)
	{/* Get our own colormap */
	  printf("visual depth [%d]\n",found_vi->depth);
	  printf("visual       [%d]\n",found_vi->visual);
	  /* DEBUGGING STUFF
	     root_window = RootWindow (dsply, DefaultScreen(dsply));
	     color_map = XCreateColormap (dsply, root_window, visual, 
				       AllocNone ); */
	  color_map = XCreateColormap (dsply, root_window, found_vi->visual, 
				       AllocNone );
	}/* Get our own colormap */
      else
	{/* Use default */
	  /* Create the Color Map and allocate all the color cells */
	  color_map = XDefaultColormapOfScreen(
			   XDefaultScreenOfDisplay(dsply));
	}/* Use default */
#endif /* USE_DEC_HELPLINE_CODE */

      
#else
      /* Create the Color Map and allocate all the color cells */
      color_map = XDefaultColormapOfScreen(
					   XDefaultScreenOfDisplay(dsply));
#endif
      
      if(maxCmaps==1)
	{/* Use the default one */
	  color_map = XDefaultColormapOfScreen(
					       XDefaultScreenOfDisplay(dsply));
	}/* Use the default one */






      if (color_map == NULL) {
	fprintf (stderr, "Could not create color map for visual\n");
	exit (1);
      }
      
      /* Now make room for our UIL colors */
      XAllocNamedColor(dsply, color_map, "red", 
		       &screen_def_rtn, &exact_def_rtn);
      XAllocNamedColor(dsply, color_map, "yellow", 
		       &screen_def_rtn, &exact_def_rtn);
      XAllocNamedColor(dsply, color_map, "lightblue", 
		       &screen_def_rtn, &exact_def_rtn);

      /* Now let's query the DISPLAY until we get the maximum number of
       * contiguous colorcell entries in the colormap that we can .
       */
      success= FALSE;
      tryColors= 12;
      nplanes = 1;
      contig = True;
      plane_masks_return = (unsigned long *)malloc(sizeof(unsigned long)*
						   nplanes);
      dummy_plane_masks_return = (unsigned long *)malloc(sizeof(unsigned long)*
						   nplanes);
      nplanes = 0;

#ifdef TRYTHIS 
      /* Let's reserve 10 cells to be used by the OS when
       * we deallocate them below... */
      XAllocColorCells(dsply, 
		       color_map, 
		       contig, 
		       (unsigned long *)dummy_plane_masks_return, 
		       nplanes, 
		       dummy_pixels_return,
		       10);
#endif /* #ifdef TRYTHIS */

      /* Now alloacte the ones WE want */
      while(!success)
	{/* Try until we get a maximum number of color entries */
	  success= XAllocColorCells(dsply, 
				    color_map, 
				    contig, 
				    (unsigned long *)plane_masks_return, 
				    nplanes, 
				    pixels_return,
				    tryColors);
	  if(!success)
	    {/* CAn't do it */
	      tryColors--;
	      if(tryColors < 2)
		{/* CAn't go on */
		  printf("Can't XAllocColorCells() [%d]\n",tryColors);
		  exit(0);
		}/* CAn't go on */
	    }/* CAn't do it */
	}/* Try until we get a maximum number of color entries */
      /* free(plane_masks_return);
      XFree(visual_info); */

      fprintf(stdout,"Colors available [%d]\n",tryColors);
      if(tryColors < 10)
	{/* Cannot get enough colr entries in cmap */
	  printf("Cannot get enough color entries in cmap [%d]\n",tryColors);
	  exit(0);
	}/* Cannot get enough colr entries in cmap */
      
#ifdef TRYTHIS
      /* Let's free these 10 cells to the OS ... */
      XFreeColors(dsply, 
		  color_map, 
		  dummy_pixels_return,
		  10,
		  (unsigned long *)dummy_plane_masks_return);
      free(dummy_pixels_return);
#endif /* #ifdef TRYTHIS */

      /* Init the color defs to point to each entry */
      for (i = 0; i <256 ; i++) 
	{/* Make the color map gray-scale */
	  color_defs[i].pixel= i;
	  gen_color_defs[i].pixel= i;
	}/* Make the color map gray-scale */

      /* Find out what's already there in the color table */

      XQueryColors(dsply, color_map, color_defs, 256);
      XQueryColors(dsply, color_map, gen_color_defs, 256);

      sprintf(outStr,"%s/cmap.fixpif",getenv("HOME"));
      fColor = fopen(outStr,"w");
      if(fColor)
	{/* Write it */
	  fwrite(&tryColors,sizeof(int),1,fColor);
	  for(i=0;i<tryColors;i++)
	    {/* Write it all out */
	      fwrite(&pixels_return[i],sizeof(unsigned long),1,fColor);
	    }/* Write it all out */
	}/* Write it */
      else
	{/* Trouble */
	  printf("ERROR - Cannot write colormap file.\n");
	}/* Trouble */
      fclose(fColor);

    }/* Create the colormap here is MASTER */
  else
    {/* Read from file */
      sprintf(outStr,"%s/cmap.fixpif",getenv("HOME"));
      fColor = fopen(outStr,"r");
      if(fColor)
	{/* Read it */
	  fread(&tryColors,sizeof(int),1,fColor);
	  for(i=0;i<tryColors;i++)
	    {/* Read it all out */
	      fread(&pixels_return[i],sizeof(unsigned long),1,fColor);
	    }/* Read it all out */
	}/* Read it */
      else
	{/* SOmethings wrong */
	  printf("ERROR - Cannot read colormap file.\n");
	}/* SOmethings wrong */
      fclose(fColor);
      
    }/* Read from file */


  /* Now change the colormap for drawing in pixels_return[GREEN,RED,BLUE] 
   * First calculate some fudge factors.
   */
  pixelColor= (65535.0/256.0);
  scaleFactor = 255.0 / ((float)tryColors - 7.0);
  
  /* Make these entries in the colormap GRAYSCALE */
  for(i=6;i<(tryColors);i++)
    {/* Do the colors we have been assigned */
      gen_color_defs[pixels_return[i]].pixel = pixels_return[i];
      gen_color_defs[pixels_return[i]].red   = 
	(((i-6.0) * scaleFactor) * pixelColor);
      gen_color_defs[pixels_return[i]].green =
	(((i-6.0) * scaleFactor) * pixelColor);
      gen_color_defs[pixels_return[i]].blue  = 
	(((i-6.0) * scaleFactor) * pixelColor);
      gen_color_defs[pixels_return[i]].flags = DoRed | DoGreen | DoBlue;
      color_defs[pixels_return[i]].pixel = pixels_return[i];
      color_defs[pixels_return[i]].red   = (unsigned short)
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].green = (unsigned short)
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].blue  = (unsigned short)
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].flags = DoRed | DoGreen | DoBlue;
      if(masterProcess)
	XStoreColor (dsply, color_map, &color_defs[pixels_return[i]]);
    }/* Do the colors we have been assigned */
  
  /* Now define our fixpif colors */
  color_defs[pixels_return[WHITE]].pixel = pixels_return[WHITE];
  color_defs[pixels_return[WHITE]].red   = 255 * pixelColor;
  color_defs[pixels_return[WHITE]].green = 255 * pixelColor;
  color_defs[pixels_return[WHITE]].blue  = 255 * pixelColor;
  color_defs[pixels_return[WHITE]].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    XStoreColor (dsply, color_map, &color_defs[pixels_return[WHITE]]);
  color_defs[pixels_return[YELLOW]].pixel = 
    pixels_return[YELLOW];
  color_defs[pixels_return[YELLOW]].red   = 255 * pixelColor;
  color_defs[pixels_return[YELLOW]].green = 255 * pixelColor;
  color_defs[pixels_return[YELLOW]].blue  = 0 * pixelColor;
  color_defs[pixels_return[YELLOW]].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    XStoreColor (dsply, color_map, 
	       &color_defs[pixels_return[YELLOW]]);
  color_defs[pixels_return[GREEN]].pixel = pixels_return[GREEN];
  color_defs[pixels_return[GREEN]].red   = 255 * pixelColor;
  color_defs[pixels_return[GREEN]].green = 255 * pixelColor;
  color_defs[pixels_return[GREEN]].blue  = 0;
  color_defs[pixels_return[GREEN]].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    XStoreColor (dsply, color_map, &color_defs[pixels_return[GREEN]]);
  color_defs[pixels_return[RED]].pixel = pixels_return[RED];
  color_defs[pixels_return[RED]].red   = 255 * pixelColor;
  color_defs[pixels_return[RED]].green = 0;
  color_defs[pixels_return[RED]].blue  = 0;
  color_defs[pixels_return[RED]].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    XStoreColor (dsply, color_map, &color_defs[pixels_return[RED]]);
  color_defs[pixels_return[BLUE]].pixel = pixels_return[BLUE];
  color_defs[pixels_return[BLUE]].red   = 128 * pixelColor;
  color_defs[pixels_return[BLUE]].green = 128 * pixelColor;
  color_defs[pixels_return[BLUE]].blue  = 255 * pixelColor;
  color_defs[pixels_return[BLUE]].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    XStoreColor (dsply, color_map, &color_defs[pixels_return[BLUE]]);
  color_defs[pixels_return[PLACEHOLDER]].pixel = 
    pixels_return[PLACEHOLDER];
  color_defs[pixels_return[PLACEHOLDER]].red   = 0 * pixelColor;
  color_defs[pixels_return[PLACEHOLDER]].green = 255 * pixelColor;
  color_defs[pixels_return[PLACEHOLDER]].blue  = 255 * pixelColor;
  color_defs[pixels_return[PLACEHOLDER]].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    XStoreColor (dsply, color_map, 
	       &color_defs[pixels_return[PLACEHOLDER]]);
  color_defs[pixels_return[BLACK]].pixel = pixels_return[BLACK];
  color_defs[pixels_return[BLACK]].red   = 0 * pixelColor;
  color_defs[pixels_return[BLACK]].green = 0 * pixelColor;
  color_defs[pixels_return[BLACK]].blue  = 0 * pixelColor;
  color_defs[pixels_return[BLACK]].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    XStoreColor (dsply, color_map, &color_defs[pixels_return[BLACK]]);
  
  /* Make this colormap is the one that's used! */
  XListInstalledColormaps(dsply, root_window, &numColormaps);
  printf("numColormaps[%d]\n",numColormaps);
  if(masterProcess)
    XInstallColormap (dsply, color_map);/* Install the color map */

  XListInstalledColormaps(dsply, root_window, &numColormaps);
  printf("After numColormaps[%d]\n",numColormaps);

}/*  CREATECOLORMAP */



/******************************************************
 * ie3tvax() - Convert IEEE floats in Ultrix/C to 
 * VAX format float.
*/
void ie3tvax(int *inFloat)
{/* IE3TVAX */
  int i;
  int tmpFloat;

  tmpFloat= (int)*inFloat;

  *inFloat= 0;
  *inFloat |= ((tmpFloat & 0177777) << 16);
  *inFloat |= ((tmpFloat & 037777600000) >> 16);
  *inFloat |= 0400;

}/* IE3TVAX */




/*******************************************************
 * vaxtie3() - Convert VAX format floats to IEEE floats 
 * in Ultrix/C.
*/
void vaxtie3(unsigned int *inFloat)
{/* VAXTIE3 */
  unsigned int i;
  unsigned int tmpFloat;

  tmpFloat= (unsigned int)*inFloat;

  *inFloat= 0;
  *inFloat |= ((tmpFloat & 037777600000) >> 16);
  *inFloat |= ((tmpFloat & 0177777) << 16);
  *inFloat &= 037677777777;   /* Turn this bit OFF */


/*  Here's some fortran code I got off the net to do this conversion.
    Implement it when I understand it.
c
c convert single precision floating point numbers from vax representation
c to ieee while running on rs6000. originally vax version of ie3fos from
c cernlib.
c modifications indicated by    ! <<<
c
 
      subroutine ie3fvax (ms,mt,nwdo,jbad)
c
c cern proglib# m220    ie3fos          .version kernvax  2.20  861204
 
c-    convert single precision for output with copy
c-    from source in native to target in ieee data format
 
      integer      nwdo                 ! <<<
c      dimension    ms(99), mt(99)      ! <<<
      dimension    ms(nwdo), mt(nwdo)   ! <<<
 
      double precision   thdb
      dimension    this(2)
      equivalence  (thdb,this)
      equivalence  (itha,tha,this(1)), (ithb,thb,this(2))
 
      parameter    (mskb16 = '00008000'x)
      parameter    (ibadcs = '7f80fe00'x)
      parameter    (ibadcd = '7ff01fc0'x)
      parameter    (iovpcs = '7f800000'x)
      parameter    (iovpcd = '7ff00000'x)
      parameter    (iovncs = 'ff800000'x)
      parameter    (iovncd = 'fff00000'x)
 
 
      jbad = 0
      jms  = 0
      jmt  = 0
 
  301 do 349  jl=1,nwdo
      itha = ms(jms+1)
      if (itha.eq.0)         go to 347
c     jsign = itha .and. mskb16                ! <<<
      jsign = and(itha,mskb16)                 ! <<<
c     jexp  = jibits (itha,7,8)                ! <<<
      jexp  = ibits (itha,7,8)                 ! <<<
      if (jexp.le.2)         go to 331
      if (jexp.eq.255)       go to 333
c     jmant = jishft(jibits(itha,0,7), 16) .or. jibits(itha,16,16)    ! <<<
      jmant = or(ishft(ibits(itha,0,7), 16),ibits(itha,16,16))        ! <<<
c     itha  = jishft(jsign,16) .or. jishft(jexp-2,23) .or. jmant      ! <<<
      itha  = or( or(ishft(jsign,16),ishft(jexp-2,23)), jmant)        ! <<<
      go to 347
 
c--      zero / nan
  331 if (jsign.eq.0)        go to 341
      if (jexp.ne.0)         go to 341
      ithb = ibadcs
      go to 336
 
c--      overflow
  333 if (jsign.eq.0)    then
          ithb = iovpcs
        else
          ithb = iovncs
        endif
  336 jbad = jl
      itha = ithb
      go to 347
 
  341 itha = 0
  347 mt(jmt+1) = itha
      jmt = jmt + 1
  349 jms = jms + 1
 
      return
      end
*/

}/* VAXTIE3 */




/*************************************************************
 * calcScaleFactor() - Calculate the factor needed to convert
 * out floatIntegers into floats when we read this file later.
 * Just scan all of our values and pick a value that maintains 
 * out dynamic rarenge.
*/
void calcScaleFactor(float *floatScaleFactor, 
		     float *recipFloatScaleFactor, 
		     int img_min, int img_max,
		     float img_avg, float img_std,
		     int fileType)
{/* CALCSCALEFACTOR */
  int minVal,maxVal;
  char intermediateStr[256];

  minVal = -360.0;   /* Account for alp,bet,gam etc... */
  maxVal =  360.0;

  minVal= MIN(minVal,img_min);
  minVal= MIN(minVal,img_max);
  minVal= MIN(minVal,img_avg);
  minVal= MIN(minVal,img_std);

  maxVal= MAX(maxVal,img_min);
  maxVal= MAX(maxVal,img_max);
  maxVal= MAX(maxVal,img_avg);
  maxVal= MAX(maxVal,img_std);

  /* Get greatest magnitude */
  if(minVal < 0.0)
    minVal = 0.0 - minVal;
  maxVal = MAX(minVal,maxVal);
  /* Now let's give it a little extra so we're guaranteed not
   * to run over the top */
  maxVal *= 1.02;

  /* We're doing MAPs as shorts so make it smaller */
  if(fileType==64)
    *floatScaleFactor = (double)((double) MAXSHORT / (double) maxVal);
  else
    *floatScaleFactor = (double)((double) MAXINT / (double) maxVal);

  *recipFloatScaleFactor = (double)((double)1.0 / (double)*floatScaleFactor); 

}/* CALCSCALEFACTOR */



/************************************************************
 * today() - Return todays date and current time in STRING 
 * (static) form.
*/
char *today()
{/* TODAY */
  static char s1[132];
  char *sR, *ampm;
  int seconds,
      datec,
      timeofday,
      minutes,
      hours,
      hyear,
      hmonth,
      hday;
  struct tm *tim;
  time_t tSec;

  tSec= time(NULL); 
  tim= localtime((time_t *)&tSec); 

  seconds= tim->tm_sec;
  minutes= tim->tm_min;
  hours=   tim->tm_hour;
  hday=    tim->tm_mday;
  hmonth=  tim->tm_mon + 1;
  hyear=   tim->tm_year + 1900;

  if(hours > 12)
    {/* PM */
      hours -= 12;
      ampm= "PM";
    }/* PM */
  else if(hours==12)
    ampm= "PM";
  else ampm= "AM";

  sprintf(s1,"%02d/%02d/%04d,  %02d:%02d:%02d%s",
         hmonth, hday, hyear, hours, minutes, seconds, ampm);

  return(s1);
}/* TODAY */






/**************************************************************
 * createSimpleString() - Depending on which computer you're on
 * create a simple string .
*/
void createSimpleString(XmString *simpStr, char * outStr)
{/* CREATESIMPLESTRING */

  *simpStr= XmStringCreateLocalized(outStr);

}/* CREATESIMPLESTRING */









/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * XlibInitialize() - Initialize for Xlib calls.  Setup
 * the colormap we'll use and graphics contexts.  Create 
 * all of the Images and allocate memory for all of our
 * XmDrawingAreas.  Add event handlers for motion events
 * where needed and setup our cursors.
 */
void XlibInitialize()
{/* XLIBINITIALIZE */
  float scaleFactor;
  Window tWin;
  XWindowAttributes winAttrib;
  XSetWindowAttributes setWinAttrib;
  static const char dotted[8]= {4,4,4,4};


  /* Enable save unders and backing store */
  if((XDoesSaveUnders(XScreenOfDisplay(dsply, DefaultScreen(dsply)))) &&
     (XDoesBackingStore(XScreenOfDisplay(dsply, DefaultScreen(dsply)))))
    {
      attrs.save_under = TRUE;
      attrs.backing_store = WhenMapped;
      printf("Enabling 'Backing Store' and 'SaveUnder' features\n");
#ifdef NEEDED
      XChangeWindowAttributes(dsply, XtWindow(fftDrawingWidget), 
			      (CWSaveUnder | CWBackingStore), &attrs);
#endif
      XFlush(dsply);
      /* Other candidates for this:
	 image_window1= XtWindow(drawingWidget);
	 rotflt_window1= XtWindow(rotfltDrawWidget); 
	 fft_window1= XtWindow(fftDrawingWidget);
	 fftContour_window1= XtWindow(contourWidget);
	 circAvg_window1= XtWindow(circAvgWidget);
	 color_window1= XtWindow(colormapWidget);
	 test_color_window1= XtWindow(testcolormapWidget);
	 hist_color_window1= XtWindow(histWidget);
	 editBox_window1= XtWindow(editBoxWidget);
	 icosa_window1= XtWindow(icosaWidget);
      */
    }

  /* Init dimensions */
#ifdef ORIGWAY
  buffer_width= image_width = NUMCOLS;
  buffer_height= image_height = NUMROWS;
#else
  buffer_width= image_width = 
    buffer_height= image_height = defDisplayBufferSize;
#endif
  image_colors = 3;



  /* Display the image in the window created above */
  image_width = DEF_DISPLAY_SIZE;   /* This changes whenever a new image  */
  image_height = DEF_DISPLAY_SIZE;  /* is read                            */

  display_width = image_width;      /* Make sure we draw it all */
  display_height = image_height;

  XFlush (dsply);
  
  


}/* XLIBINITIALIZE */







/***************************************************************
 * s_error() - All errors are fatal.
 */
void s_error(char *problem_string)
{/* S_ERROR */
  printf("%s\n", problem_string);
  exit(0);
}/* S_ERROR */






/******************************************************************
 * clipFileName() - The VMS FORTRAN routines sometimes don't like
 * to have extensions on their filenames.  Clip those off here.
*/
void clipFileName(char *fName)
{/* CLIPFILENAME */
  int i,lngth;
  char *s2;

  s2= strrchr(fName,'.');
  *s2= '\0';
#ifdef NEDDTHIS
  lngth = strlen(fName);
  for(i=lngth;i<256;i++)
    fName[i]= ' ';
  fName[255]= '\0';
#endif
}/* CLIPFILENAME */




/******************************************************************
 * clipVersion() - Some VMS FORTRAN routines don't like to have the
 * version number on the filename.  Clip here.
*/
void clipVersion(char *fName)
{/* CLIPVersion */
  int i,lngth;
  char *s2;

  s2= strrchr(fName,';');
  if(s2==NULL)
    {/* Then look for space */
      s2= strchr(fName,' ');
    }/* Then look for space */
  if(s2!=NULL)
    {/* Last chance */
      *s2= '\0';
    }/* Last chance */
  
#ifdef USETHIS
  lngth = strlen(fName);
  for(i=lngth;i<64;i++)
    fName[i]= ' ';
  fName[63]= '\0';
#endif  

}/* CLIPVersion */




/******************************************************************
 * incrementVersion() - Increment the file version for VMS.
*/
void incrementVersion(char *fName)
{/* INCREMENTVERSION */
  int i,lngth,verNum;
  char *s2;

}/* INCREMENTVERSION */



/**************************************************************
 * raiseIt() - Do everything necessary to raise a window to the
 * foreground on the display.
*/
void raiseIt(Widget w)
{/* RAISEIT */

  XtManageChild(w);
  XtPopup(XtParent(w), XtGrabNone);

  XmProcessTraversal(w, XmTRAVERSE_CURRENT); 
  XtRealizeWidget(w);  
  XRaiseWindow(dsply, XtWindow(XtParent(w)));
  XFlush(dsply);


}/* RAISEIT */



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * handle_cr() -  This isn't really a callback
 * but it is called by a callback.  This routine changes
 * the translation table interpretations for the simple 
 * text widgets so it knows what to do when it sees a 
 * carriage return.
*/
void handle_cr(Widget widget)
{/* HANDLE_CR */
  XtTranslations parsed_t_table;

  if(widget==headNumTextW)
    {/* Do translations for study text widget */
      /* Modify this widgets translation table */
      parsed_t_table = XtParseTranslationTable(rwa_translation_table);
      XtOverrideTranslations(widget, parsed_t_table);
    }/* Do translations for study text widget */


}/* HANDLE_CR */





/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * stuffGlobal() -  
*/
void stuffGlobal()
{/* STUFFGLOBAL */
  
  XtUnmanageChild(imageForm);
  XtManageChild(globalForm);
  
  pif_read_gh_(&pifHeader);
  numImages     = pifHeader.nimg;
  currentHeader = 0;

  sprintf(outStr,"Glob");
  XmTextSetString(headNumTextW, outStr);

  sprintf(outStr,"%d",pifHeader.nimg);
  XmTextSetString(glbHeadTextW[0], outStr);
  sprintf(outStr,"%E",pifHeader.floatScaleFactor);
  XmTextSetString(glbHeadTextW[1], outStr);
  sprintf(outStr,"%E",pifHeader.recipFloatScaleFactor);
  XmTextSetString(glbHeadTextW[2], outStr);
  sprintf(outStr,"%d",pifHeader.htype);
  XmTextSetString(glbHeadTextW[3], outStr);
  sprintf(outStr,"%d",pifHeader.numCols);
  XmTextSetString(glbHeadTextW[4], outStr);
  sprintf(outStr,"%d",pifHeader.numRows);
  XmTextSetString(glbHeadTextW[5], outStr);
  sprintf(outStr,"%d",pifHeader.numSecs);
  XmTextSetString(glbHeadTextW[6], outStr);
  sprintf(outStr,"%d",pifHeader.mode);
  XmTextSetString(glbHeadTextW[7], outStr);
  sprintf(outStr,"%d",pifHeader.endianNess);
  XmTextSetString(glbHeadTextW[8], outStr);
  sprintf(outStr,"%s",pifHeader.versionStr);
  XmTextSetString(glbHeadTextW[9], outStr);

  sprintf(outStr,"%d",pifHeader.even);
  XmTextSetString(glbHeadTextW[10], outStr);
  sprintf(outStr,"%d",pifHeader.mrcX);
  XmTextSetString(glbHeadTextW[11], outStr);
  sprintf(outStr,"%d",pifHeader.mrcY);
  XmTextSetString(glbHeadTextW[12], outStr);
  sprintf(outStr,"%d",pifHeader.mrcZ);
  XmTextSetString(glbHeadTextW[13], outStr);
  sprintf(outStr,"%f",pifHeader.int4scaleF);
  XmTextSetString(glbHeadTextW[14], outStr);

  XFlush(dsply);

  dirty = FALSE;

}/* STUFFGLOBAL */





/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * stuffImage() -  
*/
void stuffImage(int imgNum)
{/* STUFFIMAGE */
  
  XtManageChild(imageForm);
  XtUnmanageChild(globalForm);
  
  pifHeader.imgNum = imgNum;
  pif_read_dh_(&pifHeader);

  sprintf(outStr,"%d",imgNum);
  XmTextSetString(headNumTextW, outStr);

  sprintf(outStr,"%d",pifHeader.imgNum);
  XmTextSetString(imgHeadTextW[0], outStr);
  sprintf(outStr,"%d",pifHeader.mode);
  XmTextSetString(imgHeadTextW[1], outStr);
  sprintf(outStr,"%f",pifHeader.img_bkg);
  XmTextSetString(imgHeadTextW[2], outStr);
  sprintf(outStr,"%d",pifHeader.datai4_min);
  XmTextSetString(imgHeadTextW[3], outStr);
  sprintf(outStr,"%d",pifHeader.datai4_max);
  XmTextSetString(imgHeadTextW[4], outStr);
  sprintf(outStr,"%f",pifHeader.datar4_min);
  XmTextSetString(imgHeadTextW[5], outStr);
  sprintf(outStr,"%f",pifHeader.datar4_max);
  XmTextSetString(imgHeadTextW[6], outStr);
  sprintf(outStr,"%f",pifHeader.datar4_avg);
  XmTextSetString(imgHeadTextW[7], outStr);
  sprintf(outStr,"%f",pifHeader.datar4_std);
  XmTextSetString(imgHeadTextW[8], outStr);
  sprintf(outStr,"%d",pifHeader.numCols);
  XmTextSetString(imgHeadTextW[9], outStr);
  sprintf(outStr,"%d",pifHeader.numRows);
  XmTextSetString(imgHeadTextW[10], outStr);
  sprintf(outStr,"%d",pifHeader.numSecs);
  XmTextSetString(imgHeadTextW[11], outStr);
  sprintf(outStr,"%s",pifHeader.descriptionStr);
  XmTextSetString(imgHeadTextW[12], outStr);
  sprintf(outStr,"%f",pifHeader.aoverb);
  XmTextSetString(imgHeadTextW[13], outStr);
  sprintf(outStr,"%f",pifHeader.map_abang);
  XmTextSetString(imgHeadTextW[14], outStr);
  sprintf(outStr,"%f",pifHeader.a_cell);
  XmTextSetString(imgHeadTextW[15], outStr);
  sprintf(outStr,"%f",pifHeader.b_cell);
  XmTextSetString(imgHeadTextW[16], outStr);
  sprintf(outStr,"%f",pifHeader.c_cell);
  XmTextSetString(imgHeadTextW[17], outStr);
  sprintf(outStr,"%f",pifHeader.alp);
  XmTextSetString(imgHeadTextW[18], outStr);
  sprintf(outStr,"%f",pifHeader.bet);
  XmTextSetString(imgHeadTextW[18], outStr);
  sprintf(outStr,"%f",pifHeader.gam);
  XmTextSetString(imgHeadTextW[20], outStr);
  sprintf(outStr,"%f",pifHeader.dela);
  XmTextSetString(imgHeadTextW[21], outStr);
  sprintf(outStr,"%f",pifHeader.delb);
  XmTextSetString(imgHeadTextW[22], outStr);
  sprintf(outStr,"%f",pifHeader.delc);
  XmTextSetString(imgHeadTextW[23], outStr);

  sprintf(outStr,"%f",pifHeader.a_star);
  XmTextSetString(imgHeadTextW[24], outStr);
  sprintf(outStr,"%f",pifHeader.b_star);
  XmTextSetString(imgHeadTextW[25], outStr);
  sprintf(outStr,"%f",pifHeader.c_star);
  XmTextSetString(imgHeadTextW[26], outStr);
  sprintf(outStr,"%f",pifHeader.alp_star);
  XmTextSetString(imgHeadTextW[27], outStr);
  sprintf(outStr,"%f",pifHeader.bet_star);
  XmTextSetString(imgHeadTextW[28], outStr);
  sprintf(outStr,"%f",pifHeader.gam_star);
  XmTextSetString(imgHeadTextW[29], outStr);
  sprintf(outStr,"%f",pifHeader.pixelSize);
  XmTextSetString(imgHeadTextW[30], outStr);

  sprintf(outStr,"%d",pifHeader.t_matrix[0]);
  XmTextSetString(imgHeadTextW[31], outStr);
  sprintf(outStr,"%d",pifHeader.t_matrix[1]);
  XmTextSetString(imgHeadTextW[32], outStr);
  sprintf(outStr,"%d",pifHeader.t_matrix[2]);
  XmTextSetString(imgHeadTextW[33], outStr);
  sprintf(outStr,"%d",pifHeader.t_matrix[3]);
  XmTextSetString(imgHeadTextW[34], outStr);
  sprintf(outStr,"%d",pifHeader.t_matrix[4]);
  XmTextSetString(imgHeadTextW[35], outStr);
  sprintf(outStr,"%d",pifHeader.t_matrix[5]);
  XmTextSetString(imgHeadTextW[36], outStr);
  sprintf(outStr,"%d",pifHeader.packRadius);
  XmTextSetString(imgHeadTextW[37], outStr);
  sprintf(outStr,"%d",pifHeader.binFactor);
  XmTextSetString(imgHeadTextW[38], outStr);
  sprintf(outStr,"%f",pifHeader.dthe);
  XmTextSetString(imgHeadTextW[39], outStr);
  sprintf(outStr,"%f",pifHeader.dphi_90);
  XmTextSetString(imgHeadTextW[40], outStr);
  sprintf(outStr,"%d",pifHeader.symmetry);
  XmTextSetString(imgHeadTextW[41], outStr);
  sprintf(outStr,"%d",pifHeader.pixSizUnits);
  XmTextSetString(imgHeadTextW[42], outStr);
  sprintf(outStr,"%f",pifHeader.res_out);
  XmTextSetString(imgHeadTextW[43], outStr);
  sprintf(outStr,"%d",pifHeader.ctfMode);
  XmTextSetString(imgHeadTextW[44], outStr);
  sprintf(outStr,"%f",pifHeader.tempFactor);
  XmTextSetString(imgHeadTextW[45], outStr);
  sprintf(outStr,"%f",pifHeader.wiener);
  XmTextSetString(imgHeadTextW[46], outStr);
  sprintf(outStr,"%d",pifHeader.borderType);
  XmTextSetString(imgHeadTextW[47], outStr);
  sprintf(outStr,"%d",pifHeader.borderWidth);
  XmTextSetString(imgHeadTextW[48], outStr);
  sprintf(outStr,"%d",pifHeader.rad_lo);
  XmTextSetString(imgHeadTextW[49], outStr);
  sprintf(outStr,"%d",pifHeader.rad_hi);
  XmTextSetString(imgHeadTextW[50], outStr);
  sprintf(outStr,"%d",pifHeader.transX);
  XmTextSetString(imgHeadTextW[51], outStr);
  sprintf(outStr,"%d",pifHeader.transY);
  XmTextSetString(imgHeadTextW[52], outStr);
  sprintf(outStr,"%d",pifHeader.transZ);
  XmTextSetString(imgHeadTextW[53], outStr);
  sprintf(outStr,"%d",pifHeader.isTrunc);
  XmTextSetString(imgHeadTextW[54], outStr);
  sprintf(outStr,"%d",pifHeader.origX);
  XmTextSetString(imgHeadTextW[55], outStr);
  sprintf(outStr,"%d",pifHeader.origY);
  XmTextSetString(imgHeadTextW[56], outStr);
  sprintf(outStr,"%d",pifHeader.origZ);
  XmTextSetString(imgHeadTextW[57], outStr);
  sprintf(outStr,"%f",pifHeader.pifVersion);
  XmTextSetString(imgHeadTextW[58], outStr);

  XFlush(dsply);
  dirty = FALSE;

}/* STUFFIMAGE */





/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * getImage() -  
*/
void getImage(int imgNum)
{/* GETIMAGE */
  char *thisStr;
  int i;

  thisStr= XmTextGetString(imgHeadTextW[1]);
  pifHeader.mode = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[2]);
  pifHeader.img_bkg = (float)strtod(thisStr,0);
  XtFree(thisStr);

  thisStr= XmTextGetString(imgHeadTextW[3]);
  pifHeader.datai4_min = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[4]);
  pifHeader.datai4_max = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[5]);
  pifHeader.datar4_min = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[6]);
  pifHeader.datar4_max = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[7]);
  pifHeader.datar4_avg = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[8]);
  pifHeader.datar4_std = (float)strtod(thisStr,0);
  XtFree(thisStr);

  thisStr= XmTextGetString(imgHeadTextW[9]);
  pifHeader.numCols = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[10]);
  pifHeader.numRows = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[11]);
  pifHeader.numSecs = (int)strtod(thisStr,0);
  XtFree(thisStr);

  thisStr= XmTextGetString(imgHeadTextW[12]);
  thisStr[79]= (char)'\0';
  sprintf(pifHeader.descriptionStr,thisStr);
  XtFree(thisStr);

  thisStr= XmTextGetString(imgHeadTextW[13]);
  pifHeader.aoverb = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[14]);
  pifHeader.map_abang = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[15]);
  pifHeader.a_cell = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[16]);
  pifHeader.b_cell = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[17]);
  pifHeader.c_cell = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[18]);
  pifHeader.alp = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[19]);
  pifHeader.bet = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[20]);
  pifHeader.gam = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[21]);
  pifHeader.dela = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[22]);
  pifHeader.delb = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[23]);
  pifHeader.delc = (float)strtod(thisStr,0);
  XtFree(thisStr);

  thisStr= XmTextGetString(imgHeadTextW[24]);
  pifHeader.a_star = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[25]);
  pifHeader.b_star = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[26]);
  pifHeader.c_star = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[27]);
  pifHeader.alp_star = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[28]);
  pifHeader.bet_star = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[29]);
  pifHeader.gam_star = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[30]);
  pifHeader.pixelSize = (float)strtod(thisStr,0);
  XtFree(thisStr);

  for(i=0;i<6;i++)
    {/* tmatrix */
      thisStr= XmTextGetString(imgHeadTextW[(31+i)]);
      pifHeader.t_matrix[i] = (int)strtod(thisStr,0);
      XtFree(thisStr);
    }/* tmatrix */

  thisStr= XmTextGetString(imgHeadTextW[37]);
  pifHeader.packRadius = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[38]);
  pifHeader.binFactor = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[39]);
  pifHeader.dthe = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[40]);
  pifHeader.dphi_90 = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[41]);
  pifHeader.symmetry = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[42]);
  pifHeader.pixSizUnits = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[43]);
  pifHeader.res_out = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[44]);
  pifHeader.ctfMode = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[45]);
  pifHeader.tempFactor = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[46]);
  pifHeader.wiener = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[47]);
  pifHeader.borderType = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[48]);
  pifHeader.borderWidth = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[49]);
  pifHeader.rad_lo = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[50]);
  pifHeader.rad_hi = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[51]);
  pifHeader.transX = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[52]);
  pifHeader.transY = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[53]);
  pifHeader.transZ = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[54]);
  pifHeader.isTrunc = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[55]);
  pifHeader.origX = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[56]);
  pifHeader.origY = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[57]);
  pifHeader.origZ = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(imgHeadTextW[58]);
  pifHeader.pifVersion = (float)strtod(thisStr,0);
  XtFree(thisStr);

  XFlush(dsply);

}/* GETIMAGE */







/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * getGlobal() -  
*/
void getGlobal(int imgNum)
{/* GETGLOBAL */
  char *thisStr;
  int i;

  thisStr= XmTextGetString(glbHeadTextW[0]);
  pifHeader.nimg = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[1]);
  pifHeader.floatScaleFactor = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[2]);
  pifHeader.recipFloatScaleFactor = (float)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[3]);
  pifHeader.htype = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[4]);
  pifHeader.numCols = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[5]);
  pifHeader.numRows = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[6]);
  pifHeader.numSecs = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[7]);
  pifHeader.mode = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[8]);
  pifHeader.endianNess = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[9]);
  thisStr[79]= '\0';
  sprintf(pifHeader.versionStr,thisStr);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[10]);
  pifHeader.even = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[11]);
  pifHeader.mrcX = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[12]);
  pifHeader.mrcY = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[13]);
  pifHeader.mrcZ = (int)strtod(thisStr,0);
  XtFree(thisStr);
  thisStr= XmTextGetString(glbHeadTextW[14]);
  pifHeader.int4scaleF = (float)strtod(thisStr,0);
  XtFree(thisStr);

  XFlush(dsply);

}/* GETGLOBAL */






/************************************************************
 * read_pif() - Read image(s) in pif format.
*/
void read_pif()
{/* READ_PIF */
  int i,j,k,l,m,n;
  fString_t rWrite,inFile;
  int htype,endianNess,imgOffset,convTsb;
  char versionStr[256],outStr[512], descriptionStr[100];
  XmString simpleString;
  fString_t descStr;
  float a_cell, b_cell,c_cell;
  float alp, bet, gam;
  int binCol,binRow,binSec;
  int strtCol,strtRow,strtSec;

  i= 4;
  if(fileIsOpen)
    pif_close_(&i); /* Close it before we go on */

  pifHeader.luNum = i;
  rWrite.addr = malloc(sizeof(char)*4);
  rWrite.length = 1;
  sprintf(rWrite.addr,"r+");
  pif_open_(fileName.addr, &i, rWrite.addr);

  fileIsOpen = TRUE;

  /* Now stuff text widgets with what you've found */
  stuffGlobal();


}/* READ_PIF */






/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * gotFilename() - Get the filename they specified in file
 * selection widget.  Branch according to what operation they
 * want performed on this filename.
*/
void gotFilename (Widget w, char *client_data,
		  XmFileSelectionBoxCallbackStruct *callback_data)
{/* GOTFILENAME */
  int i,j,k,l,m,yOff,vbYoffset; 
  int vbOffset;
  char thisFileName[1024];
  char *s1,*s2,*s3;
  XmString simpleString;
  float xcen, ycen, zcen, max_r;
  int interp, nterp;
  int *irnum;
  float *sum_rint, *sumsq_rint, *stdev;
  short outWidth,outHeight,junkShort;

  /* Pop down the file selection widget */
  XtUnmanageChild(fileSel_widget);
  XFlush(dsply);

  /* Get the filename */
  if(!XmStringGetLtoR(callback_data->value, 
		      XmSTRING_DEFAULT_CHARSET, 
		      &(fileName.addr)))
    {printf("Error extracting filename.\n");}

  /* Read the file */
  read_pif();

  /* Give feedback on main window title bar */
  sprintf(outStr,"FixPIF: [%s] - %d images total.",fileName.addr,
	  pifHeader.nimg);
  createSimpleString(&simpleString,outStr);
  XtSetArg(arglst[0], XmNdialogTitle, simpleString );
  XtSetArg(arglst[1], XmNrecomputeSize, FALSE );
  XtSetValues(main_window_widget, arglst, 2); 
  XFlush(dsply);
  XmStringFree(simpleString); 
  XFlush(dsply);

}/* GOTFILENAME */



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * cancel_fileSel() -  They want to abort the selection
 * of a file name.
*/
void cancel_fileSel(Widget widget, char *tag,
		    XmFileSelectionBoxCallbackStruct *callback_data)
{/* CANCEL_FILESEL */

  XtUnmanageChild(fileSel_widget);

}/* CANCEL_FILESEL */



/* ************************************************************
 * imgNumActivate() - When the "ImgNum" button is pressed 
 * redisplay the window using the new imgNum.
 */
void imgNumActivate(Widget widget, int *tag,
		    XmAnyCallbackStruct *callback_data)
{/* IMGNUMACTIVATE */
  char *thisStr;

  if(fileIsOpen)
    {/* OK */

      if(dirty)
	{/* Check to see if we need to write first */
	  if(askThem("Do you want to save your changes to this header?",
		  FALSE,XmDIALOG_QUESTION))
	    {/* Write it */
	      write_activate(widget, tag, callback_data);
	    }/* Write it */
	}/* Check to see if we need to write first */

      thisStr= XmTextGetString(headNumTextW);
      currentHeader = (int)strtod(thisStr,0);
      XtFree(thisStr);
      
      /* Clip it accordingly */
      if(currentHeader > (numImages))
	currentHeader= (numImages);
      if(currentHeader < 0)
	currentHeader= 0;

      sprintf(outStr,"%d",currentHeader);
      XmTextSetString(headNumTextW, outStr);
      
      if(currentHeader==0)
	stuffGlobal();
      else stuffImage(currentHeader);

    }/* OK */
  else
    {/* Error */
      askThem("You must open a file first.",FALSE,XmDIALOG_INFORMATION);
    }/* Error */

}/* IMGNUMACTIVATE */




/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/***********************************************************************
 * CALLBACK routines - Rotuines called directly from X whenever the    *
 * appropriate event occurs.                                           *
 ***********************************************************************/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/*************************************************************
 * text_changed() -
*/

void text_changed(Widget widget, int *tag,
		  XmAnyCallbackStruct *callback_data)
{/* TEXT_CHANGED */

  dirty = TRUE;

}/* TEXT_CHANGED */





/*************************************************************
 * text_create() - Create all of the text widgets needed for
 * each PIF header attribute.
 *
*/
void text_create(Widget widget, int *tag,
		 XmAnyCallbackStruct *callback_data)
{/* TEXT_CREATE */
  Arg arglist[9];
  int i,j,totX;
  char numStr[20];
  XFontStruct *font;
  XmFontList font_list;
  XmString tcs;
  XtCallbackRec callBck[5];
  int bColor;
  Widget widL;
  XmStringCharSet cs= "ISOLatin1";

  /* Load all of these guys with proportional "fixed" font */
  font = XLoadQueryFont(dsply, "fixed");
  font_list = XmStringCreateFontList(font, cs); 

  totX= 0;

  if(*tag==HEADNUM)
    {/* Header number text widget */
      headNumTextW = widget;
      handle_cr(widget);
    }/* Header number text widget */
  else if(*tag==GLOBAL)
    {/* global form */
      globalForm = widget;

      for(i=0;i<MAXGLBFIELDS;i++)
	{/* Do all fields */
	  

	  XtSetArg(arglist[0], XmNx, (130) );
	  XtSetArg(arglist[1], XmNy, (50 + i*30));
	  XtSetArg(arglist[2], XmNcolumns, 50);
	  XtSetArg(arglist[3], XmNfontList, font_list);      
	  XtSetArg(arglist[4], XmNeditable, FALSE);      
	  sprintf(numStr,"Number%d",i);

	  glbHeadTextW[i] = XmCreateText(widget, numStr, arglist, 3);
	  XtAddCallback(glbHeadTextW[i], XmNvalueChangedCallback, 
			(XtCallbackProc) text_changed, NULL);
	  XtManageChild(glbHeadTextW[i]);

	  XtSetArg(arglist[0], XmNx, 5);
	  XtSetArg(arglist[1], XmNy, (50 + (i*30)));
	  XtSetArg(arglist[2], XmNalignment, XmALIGNMENT_BEGINNING);
	  XtSetArg(arglist[3], XmNlabelString, 
			    XmStringLtoRCreate(glbFields[i], cs));
      
	  XtManageChild(XmCreateLabel(widget, "", arglist, 4));

	}/* Do all fields */

    }/* global form */
  else if (*tag==IMAGE)
    {/* Image header form */
      Arg arglst[32];
      XmString tcs;
      XtCallbackRec callBck[5];    /* Our callbacks */

      /* Setup the file selection widget - do this only once */
      fileSel_widget = (Widget)XmCreateFileSelectionDialog(main_window_widget,
							   "Get file",NULL,0);
      createSimpleString(&tcs,"*.*");
      XtSetArg(arglst[0], XmNx, 10);
      XtSetArg(arglst[1], XmNy, 10 );
      XtSetArg(arglst[2], XmNdirMask, tcs);  
      XtSetArg(arglst[3], XmNdefaultPosition, FALSE); 
      XtSetArg(arglst[4], XmNfontList, fixpifFontList); 
      XtSetArg(arglst[5], XmNlabelFontList,  fixpifFontList);
      XtSetArg(arglst[6], XmNbuttonFontList,  fixpifFontList);
      XtSetValues(fileSel_widget, arglst, 7);
      
      /* Add callbacks to file selection widget */
      callBck[0].callback= (XtCallbackProc)gotFilename;
      callBck[0].closure= 0;
      callBck[1].callback= 0;
      callBck[1].closure= 0;
      XtAddCallbacks(fileSel_widget, XmNokCallback, 
		     callBck);  
      callBck[0].callback= (XtCallbackProc)cancel_fileSel;
      callBck[0].closure= 0;
      callBck[1].callback= 0;
      callBck[1].closure= 0;
      XtAddCallbacks(fileSel_widget, XmNcancelCallback, 
		     callBck);  
      XFlush(dsply);
      XmStringFree(tcs);
      
      imageForm = widget;

      totX= 0;
      for(i=0;i<MAXIMGFIELDS;i++)
	{/* Do all fields */

	  if(i < 23)
	    XtSetArg(arglist[1], XmNy, (10 + i*30));
	  else
	    {/* DO next column */
	      if(i < 47)
		{/* second row */
		  totX = 380;
		  XtSetArg(arglist[1], XmNy, (10 + ((i-22)*30)));
		}/* second row */
	      else 
		{/* third row */
		  totX = 650;
		  XtSetArg(arglist[1], XmNy, (10 + ((i-46)*30)));
		}/* third row */
	    }/* DO next column */
	  XtSetArg(arglist[0], XmNx, (totX + 130) );
	  XtSetArg(arglist[2], XmNcolumns, 20);
	  XtSetArg(arglist[3], XmNfontList, font_list);      
	  XtSetArg(arglist[4], XmNeditable, FALSE);      
	  sprintf(numStr,"Number%d",i);

	  imgHeadTextW[i] = XmCreateText(widget, numStr, arglist, 3);
	  XtAddCallback(imgHeadTextW[i], XmNvalueChangedCallback, 
			(XtCallbackProc) text_changed, NULL);
	  XtManageChild(imgHeadTextW[i]);

	  if(i < 23)
	    XtSetArg(arglist[1], XmNy, (10 + i*30));
	  else
	    {/* DO next column */
	      if(i < 47)
		{/* second row */
		  totX = 380;
		  XtSetArg(arglist[1], XmNy, (10 + ((i-22)*30)));
		}/* second row */
	      else 
		{/* third row */
		  totX = 700;
		  XtSetArg(arglist[1], XmNy, (10 + ((i-46)*30)));
		}/* third row */
	    }/* DO next column */
	  XtSetArg(arglist[0], XmNx, (totX + 5) );
	  XtSetArg(arglist[2], XmNalignment, XmALIGNMENT_BEGINNING);
	  XtSetArg(arglist[3], XmNlabelString, 
			    XmStringLtoRCreate(imgFields[i], cs));
      
	  XtManageChild(XmCreateLabel(widget, "", arglist, 4));

	}/* Do all fields */


    }/* Image header form */

}/* TEXT_CREATE */




/*-----------------------------------*/
/*************************************
 * INFO screen callbacks             *
 *************************************/
/*-----------------------------------*/
/**************************************************************
 * askThem()- This is the generic display a message and get
 * them to respond screen.  We get a return value from globAsk
 * which is set by infoOK() or infoCancel() (see below).
*/
int askThem(char *infoStr, int systemModal, int dialogType)
{/* ASKTHEM */
  XmString simpleString;

  /* Init and try to position the window */
  globAsk = -99;
  XFlush(dsply);
  createSimpleString(&simpleString,infoStr);
  XtSetArg(arglst[0], XmNmessageString, simpleString);
  XtSetArg(arglst[1], XmNdialogType, dialogType);
  XtSetArg(arglst[2], XmNx, 150);
  XtSetArg(arglst[3], XmNy, 150);
  XtSetValues(infoBox, arglst, 4);
  XFlush(dsply);
  XmStringFree(simpleString); 
 
#ifdef IMPLEMENT_THIS
  /* Freeze the display until they give us an answer!!!  This is
   * important and we need an answer!!!!  */
  if(systemModal)
    {/* Make them respond to this!!! */
      XtSetArg(arglst[0], XmNdialogStyle, XmDIALOG_SYSTEM_MODAL);
      XtSetValues(infoBox, arglst, 1);
    }/* Make them respond to this!!! */
#endif /* #ifdef IMPLEMENT_THIS */

  /* Init it and pop it up sit here and wait until we get 
   * something back */
  gFinished= FALSE;
  XtManageChild(infoBox);
  XtPopup(XtParent(infoBox), XtGrabNone);

  XmProcessTraversal(infoBox, XmTRAVERSE_CURRENT); 
  XtRealizeWidget(infoBox);  
  XFlush(dsply);

  /* gFinished is set in the infoOK() or infoCancel() callback */
  while(!gFinished)
    XtAppProcessEvent(app_context, XtIMAll);

  /* Pop it down and return their response */
  XtUnmanageChild(infoBox);
  XtPopdown(XtParent(infoBox));
  XtSetArg(arglst[0], XmNdialogStyle, XmDIALOG_MODELESS);
  XtSetValues(XtParent(infoBox), arglst, 1);
  XFlush(dsply);

  return(globAsk);

}/* ASKTHEM */




/************************************************************
 * infoOK() - If the OK button is pressed from the infoscreen
 * return here.  Set gFinished to cause the value of globAsk
 * be return to the starting point (see askThem() above).
*/
void infoOK(Widget widget, int *tag, XmAnyCallbackStruct *callback_data)
{/* infoOK */

  globAsk = TRUE;
  XtUnmanageChild(infoBox);
  gFinished= TRUE;

}/* infoOK */


/************************************************************
 * infoCancel() - If the CNACEL button is pressed from the 
 * infoscreen return here.  Set gFinished to cause the value 
 * of globAsk be return to the starting point (see askThem() 
 * above).
*/
void infoCancel(Widget widget, int *tag,
		XmAnyCallbackStruct *callback_data)
{/* infoCancel */

  globAsk = FALSE;
  XtUnmanageChild(infoBox);
  gFinished= TRUE;

}/* infoCancel */


/*-----------------------------------*/
/*************************************
 * MAIN screen Generic I/O           *
 *************************************/
/*-----------------------------------*/


/*************************************************************
 * read_activate() - They selected the "Read" submenu.  Branch
 * on the appropriate.
 */
void read_activate(Widget  widget, int *tag,
		   XmAnyCallbackStruct *callback_data)
{/* READ_ACTIVATE */

  raiseIt(fileSel_widget); 

}/* READ_ACTIVATE */



/*************************************************************
 * write_all_activate() - 
 */
void write_all_activate(Widget widget, int *tag,
			XmAnyCallbackStruct *callback_data)
{/* WRITE_ALL_ACTIVATE */
  int i,saveCurrentHeader;

  saveCurrentHeader = currentHeader;
  for(i=0;i<=numImages;i++)
    {/* Do all */

      currentHeader = i;
      sprintf(outStr,"%d",currentHeader);
      XmTextSetString(headNumTextW, outStr);
      
      if(currentHeader==0)
	stuffGlobal();
      else stuffImage(currentHeader);

      if(currentHeader > 0)
	{/* image */
	  pifHeader.imgNum = currentHeader;
	  getImage(currentHeader);
	  pif_write_dh(&pifHeader);
	}/* image */
      else
	{/* global */
	  getGlobal(currentHeader);
	  pif_write_gh(&pifHeader);
	}/* global */

    }/* Do all */

  currentHeader = saveCurrentHeader;
  sprintf(outStr,"%d",currentHeader);
  XmTextSetString(headNumTextW, outStr);

  if(currentHeader==0)
    stuffGlobal();
  else stuffImage(currentHeader);

  dirty = FALSE;

}/* WRITE_ALL_ACTIVATE */



/*************************************************************
 * write_activate() - 
 */
void write_activate(Widget widget, int *tag,
		    XmAnyCallbackStruct *callback_data)
{/* WRITE_ACTIVATE */

  if(currentHeader > 0)
    {/* image */
      pifHeader.imgNum = currentHeader;
      getImage(currentHeader);
      pif_write_dh(&pifHeader);
    }/* image */
  else
    {/* global */
      getGlobal(currentHeader);
      pif_write_gh(&pifHeader);
    }/* global */

  dirty = FALSE;

}/* WRITE_ACTIVATE */



/* ************************************************************
 * exit_activate() - They said they wanted to exit the program
 * ask again to make sure.
 */
void exit_activate(Widget widget, int *tag,
		   XmAnyCallbackStruct *callback_data)
{/* exit_ACTIVATE */
  
#ifdef REALLYNEEDTHIS
  if(fileIsOpen)
    pif_close_(&i); /* Close it before we go on */
#endif
  exit(0);
  
}/* exit_ACTIVATE */





/*************************************************************
 * headNumArrow_activate() - They want to expand the histogram
 * on the Alter Colormap screen.
*/
void headNumArrow_activate(Widget widget, int *tag,
			   XmAnyCallbackStruct *callback_data)
{/* HEADNUMARROW_ACTIVATE */
  char *thisStr;

  if(fileIsOpen)
    {/* OK to do */

      if(dirty)
	{/* Check to see if we need to write first */
	  if(askThem("Do you want to save your changes to this header?",
		  FALSE,XmDIALOG_QUESTION))
	    {/* Write it */
	      write_activate(widget, tag, callback_data);
	    }/* Write it */
	}/* Check to see if we need to write first */

      thisStr= XmTextGetString(headNumTextW);
      currentHeader = (int)strtod(thisStr,0);
      XtFree(thisStr);
      
      /* is it up or down ? */
      if(*tag==XmARROW_UP)
	{/* Up */
	  currentHeader++;
	}/* Up */
      else if(*tag==XmARROW_DOWN)
	{/* Dopwn */
	  currentHeader--;
	}/* Down */
      else printf("Current Header unchanged.\n");
      
      /* Clip it accordingly */
      if(currentHeader > (numImages))
	currentHeader= (numImages);
      if(currentHeader < 0)
	currentHeader= 0;
      
      sprintf(outStr,"%d",currentHeader);
      XmTextSetString(headNumTextW, outStr);
      
      if(currentHeader==0)
	stuffGlobal();
      else stuffImage(currentHeader);
    }/* OK to do */
  else
    {/* Error */
      askThem("You must open a file first.",FALSE,XmDIALOG_INFORMATION);
    }/* Error */

}/* HEADNUMARROW_ACTIVATE */
