      subroutine get_ptle_counts(nfiles, filename_len, param_fnames,
     $     oddeven, ptle_count)

C     Determine the number of particle images in a set of particle
C     parameter files
C     
C     nfiles         number of parameter files
C     filename_len   maximum number of characters in file name
C     param_fnames   array of file names
C     oddeven        flag indicating whether 'odd', 'even', or all images
C                    should be counted. Note that designation of even or
C                    odd is continuous across all files
C     ptle_count     array containing number of images in files

      integer, intent(in) :: nfiles, filename_len, oddeven
      character(len=filename_len), intent(in) :: param_fnames(nfiles)
      integer, intent(out) :: ptle_count(nfiles)

      integer i, indx_temp, ios
      integer, save :: skipone = 1
      real theta_temp, phi_temp, omega_temp, x_temp, y_temp, mag_temp
      character(len=1) :: heading

      do i=1,nfiles
         ptle_count(i) = 0
         open(unit=17, file=param_fnames(i))
         read(17,'(A)') heading
         read(17,'(A)') heading
         
         do
            read(17,*,iostat=ios) indx_temp, theta_temp, phi_temp,
     $           omega_temp, x_temp, y_temp, mag_temp
            
            if(ios .ne. 0) exit ! Break out of loop
            
            if(  (oddeven .ne. 1 .and. oddeven .ne. 2) .or.
     $           (oddeven == 1 .and. skipone == 1) .or.
     $           (oddeven == 2 .and. skipone == 0) ) then
               ptle_count(i) = ptle_count(i) + 1
            endif
            
            skipone = mod(skipone+1, 2) 
         enddo
         close(17)
      enddo
      
      return
      end subroutine get_ptle_counts
