C  ****************************************************************************
C  *                                                                          *
C  *                     *****     MISCLIB.SUBS     *****                     *
C  *                                                                          *
C  *    LIBRARY OF MISCELLANEOUS SUBROUTINES/FUNCTIONS:                       *
C  *                                                                          *
C  *      BELL          - RING TERMINAL BELL N TIMES                          *
C  *      BLANKL        - SEND BLANK LINES TO SYS$OUTPUT                      *
C  *                      IF #LINES < 1, BLANK CURRENT LINE W/O LINEFEED      *
C  *      BSL           - COMPUTE BESSEL FUNCTIONS                            *
C  *      CONTUR_*      - A SERIES OF SUBROUTINES USED FOR CONTOURING "MAPS"  *
C  *      CONVERSION FUNCTIONS - FUNCTION ROUTINES TO CONVERT BETWEEN         *
C  *                              VARIABLE TYPES                              *
C  *        I2I4        - CONVERT FROM INTEGER*2 TO INTEGER*4                 *
C  *        I4I2        - CONVERT FROM INTEGER*4 TO INTEGER*2                 *
C  *        I2R4        - CONVERT FROM INTEGER*2 TO REAL*4                    *
C  *        I4R4        - CONVERT FROM INTEGER*4 TO REAL*4                    *
C  *        R4I2        - CONVERT FROM REAL*4 TO INTEGER*2                    *
C  *        R4I4        - CONVERT FROM REAL*4 TO INTEGER*4                    *
C  *      CHK_PRIME     - FACTOR "N"; FIND NUMBER "N_OUT" WITH NO PRIME >19   *
C  *      COPY_I2       - COPY INTEGER*2 ARRAY1 INTO INTEGER*2 ARRAY2.        *
C  *      COPY_I4       - COPY INTEGER*4 ARRAY1 INTO INTEGER*4 ARRAY2.        *
C  *      COPY_R4       - COPY REAL*4 ARRAY1 INTO REAL*4 ARRAY2.              *
C  *      DECTOOCT      - RETURN OCTAL VALUE OF DECIMAL NUMBER                *
C  *      FILE_CHECK    - CHECK FOR STANDARD FILE TYPES.                      *
C  *      GET_YESNO     - RETURN LOGICAL IN RESPONSE TO Y/N QUESTION.         *
C  *      GNOI_R4_SIZE  - DETERMINE SIZE FOR GAUSS ARRAY.                     *
C  *      GNOI_R4_FILL  - FILL GAUSS ARRAY WITH NORMAL DISTRIBUTION.          *
C  *      HITCR         - HIT <CR> TO CONTINUE PROGRAM.                       *
C  *      HKLDSTAR      - COMPUTE DSTAR FOR REFLECTION H,K,L                  *
C  *      HKL_TO_HK     - TRANSFORM FROM HKL INDICIES TO H'K' USING THE       *
C  *                       INVERSE OF THE TRANSFORMATION MATRIX "T_MATRIX"    *
C  *      HK_TO_HKL     - TRANSFORM FROM H'K' INDICIES TO HKL                 *
C  *      I2_MEDIAN_3   - RETURN MEDIAN OF THREE I*2 NUMBERS                  *
C  *      LATCEN        - LATTICE CENTERING: FROM JOACHIM'S SPIDER ROUTINES   *
C  *      LATCEN2       - MODIFIED VERSION OF LATCEN                          *
C  *      NUMCHARI2     - DETERMINES THE NUMBER OF CHARACTERS IN AN INTEGER*2 *
C  *                       NUMBER.                                            *
C  *      NUMCHARI4     - DETERMINES THE NUMBER OF CHARACTERS IN AN INTEGER*4 *
C  *                       NUMBER.                                            *
C  *        NUM_DIGIT_R4  - FOR REAL*4 NUMBER DETERMINES NUMBER OF DIGITS TO    *
C  *                     LEFT OF DECIMAL POINT AND TOTAL DIGITS IN NUMBER   *
C  *      PIRADDEG      - GENERATE PI, RADIAN AND DEGREE TO MAXIMUM SINGLE    *
C  *                       PRECISION ACCURACY.                                *
C  *      PIRADDEG_D    - GENERATE PI, RADIAN AND DEGREE TO MAXIMUM DOUBLE    *
C  *                       PRECISION ACCURACY.                                *
C  *      REAL_TO_REC   - CONVERT REAL SPACE PARAMETERS TO RECIPROCAL         *
C  *                       LATTICE PARAMETERS.                                *
C  *      REC_TO_REAL   - CONVERT RECIPROCAL LATTICE PARAMETERS TO REAL       *
C  *                       SPACE PARAMETERS.                                  *
C  *      SEED          - GENERATES THE SEED VALUE FOR THE RANDOM NUMBER      *
C  *                       GENERATOR.                                         *
C  *      SOLV2D        - FROM JOACHIM'S SPIDER SOFTWARE.                     *
C  *      SWAP_INT2     - SWAP TWO INTEGER*2 NUMBERS.                         *
C  *      SWAP_INT4     - SWAP TWO INTEGER*4 NUMBERS.                         *
C  *      SWAP_REAL4    - SWAP TWO REAL*4 NUMBERS.                            *
C  *      SWAP_REAL8    - SWAP TWO REAL*8 NUMBERS.                            *
C  *      SWBYTE        - SWAP BYTES IN INTEGERS.                             *
C  *      TSBEND        - IDENTIFIES PROGRAM TERMINATION WITH A STRING OF *'s *
C  *      TSBEND_CLEAR  - TERMINATES PROGRAM AFTER CLEARING THE SCREEN.       *
C  *      WAVEL         - COMPUTE ELECTRON WAVELENGTH FROM VOLTS.             *
C  *        VMS            - Allows user to "exit" to operating system within a  *
C  *                    program                                          *
C  *      SDF routines:                                                        *
C  ****************************************************************************
C
C

C  **************************************************************
C  *                                                            *
C  *             *****     SUBROUTINE BELL     *****            *
C  *                                                            *
C  *    RING TERMINAL BELL N TIMES                              *
C  *                                                            *
C  *    JANUARY 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER    *
C  **************************************************************
C
      SUBROUTINE BELL (N)
C
      IMPLICIT NONE
C
      INTEGER*2 N, I
      character(len=1) A
C  ----------------------------------------------------------------------------
C
      A = achar(7)
      IF(N.LE.0) RETURN
      DO I = 1,N
        WRITE(*,'(A,A1)') '+', A
      ENDDO
C
      RETURN
      END

C  *****************************************************************
C  *                                                               *
C  *              *****     SUBROUTINE BLANKL     *****            *
C  *                                                               *
C  *    ROUTINE TO INSERT NLINES BLANK LINES INTO THE OUTPUT       *
C  *     IF NLINES < 1, BLANK OUT THE CURRENT LINE                 *
C  *                                                               *
C  *    MARCH 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER         *
C  *****************************************************************
C
      SUBROUTINE BLANKL (NLINES)
C
      IMPLICIT NONE
C
      INTEGER*2 I, NLINES
C  ----------------------------------------------------------------------------
C
      IF(NLINES.GT.0) THEN
         DO I = 1,NLINES
           WRITE(*,'(1X)')
         ENDDO
       ELSE
         WRITE(*,1000)
      ENDIF
C
      RETURN
 1000      FORMAT('+',79(' '))
      END

C  *****************************************************************
C  *                                                               *
C  *              *****     SUBROUTINE BSL     *****               *
C  *                                                               *
C  *    FUNCTION ROUTINES TO COMPUTE BESSEL FUNCTIONS              *
C  *                                                               *
C  *    JULY 1986  PURDUE UNIVERSITY VAX 11/750, T. BAKER          *
C  *    ORIGINAL ORIGIN UNKNOWN                                    *
C  *    USED IN PROGRAMS SUCH AS EMROTFLT                          *
C  *****************************************************************
C
C  ***********************
C  *    FUNCTION BSL0    *
C  ***********************
C
      FUNCTION BSL0 (X0)
C
      IMPLICIT NONE
C
      REAL*4 BSL0, X0, X1, X2, P, Q
C  ----------------------------------------------------------------------------
C
      IF(X0-4.0) 100, 200, 200
  100      X2   = (X0/4.0)**2
      BSL0 = ((((((-0.00050144*X2+0.00767719)*X2-0.07092536)*X2+
     1  0.44435843)*X2-1.7777561)*X2+3.9999973)*X2-3.9999999)*X2+1.0
      RETURN
C
  200      X1 = 4.0/X0
      X2 = X1**2
      P  = ((((-0.00000370*X2+0.0001736)*X2-.48761E-04)*X2+
     1    0.00017343)*X2-0.00175306)*X2+0.39894228
      Q  = (X1)*(((((0.00000323*X2-0.00001421)*X2+0.00003425)*X2-
     1    0.00008698)*X2+0.00045643)*X2-0.01246694)
      BSL0 = SQRT(X1)*(P*COS(X0-.7853981E+00)-Q*SIN(X0-.7853981E+00))
      RETURN
C
      END
C  ***********************
C  *    FUNCTION BSL1    *
C  ***********************
C
      FUNCTION BSL1 (X0)
C
      IMPLICIT NONE
C
      REAL*4 BSL1, X0, X1, X2, P, Q
C  ----------------------------------------------------------------------------
C
      IF(X0-4.0) 100, 200, 200
  100      X1   = X0/4.0
      X2   = X1**2
      BSL1 = X1*(((((((-0.0001290*X2+0.0022069)*X2-0.02366168)*X2+
     1      0.1777583)*X2-0.8888840)*X2+2.6666661)*X2-4.0)*X2+2.0)
      RETURN
C
  200      X1 = 4.0/X0
      X2 = X1**2
      P  = ((((0.0000042*X2-0.0000201)*X2+0.0000581)*X2-0.0002232)*
     1    X2+0.0029218 )*X2+0.3989423
      Q  = (X1)*(((((-0.0000037*X2+0.0000162)*X2-.398708E-04)*X2+
     1    0.0001065)*X2-0.0006390)*X2+0.0374008)
      BSL1 = SQRT(X1)*(P*SIN(X0-.7853981E+00)+Q*COS(X0-.7853981E+00))
      RETURN
C
      END

C  *********************************************************************
C  *                                                                   *
C  *                 *****     FUNCTION I2I4     *****                 *
C  *                                                                   *
C  *    CONVERT AN INTEGER*2 NUMBER TO AN INTEGER*4 NUMBER.            *
C  *                                                                   *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                 *
C  *********************************************************************
C
      FUNCTION I2I4 (I2_NUM)
C
      IMPLICIT NONE
C
      INTEGER*2 I2_NUM
      INTEGER*4 I2I4
C  ----------------------------------------------------------------------------
C
      I2I4 = I2_NUM
C
      RETURN
      END

C  *********************************************************************
C  *                                                                   *
C  *                 *****     FUNCTION I4I2     *****                 *
C  *                                                                   *
C  *    CONVERT AN INTEGER*4 NUMBER TO AN INTEGER*2 NUMBER.            *
C  *                                                                   *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                 *
C  *********************************************************************
C
      FUNCTION I4I2 (I4_NUM)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      INTEGER*4 I4I2, I4_NUM
C  ----------------------------------------------------------------------------
C
      IF(I4_NUM.LE.I2_MAX .AND. I4_NUM.GE.I2_MIN) THEN
         I4I2 = I4_NUM
       ELSE
         PRINT *, '******************************************'
         PRINT *, '*    ERROR IN FUNCTION ROUTINE I4I2      *'
         PRINT *, '*                                        *'
         PRINT *, '* I*4 number too large to convert to I*2 *'
         PRINT *, '******************************************'
      ENDIF
C
      RETURN
      END

C  *********************************************************************
C  *                                                                   *
C  *                 *****     FUNCTION I2R4     *****                 *
C  *                                                                   *
C  *    CONVERT AN INTEGER*2 NUMBER TO A REAL*4 NUMBER.                *
C  *                                                                   *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                 *
C  *********************************************************************
C
      FUNCTION I2R4 (I2_NUM)
C
      IMPLICIT NONE
C
      INTEGER*2 I2_NUM
      REAL*4    I2R4
C  ----------------------------------------------------------------------------
C
      I2R4 = FLOAT(I2_NUM)
C
      RETURN
      END

C  *********************************************************************
C  *                                                                   *
C  *                 *****     FUNCTION I4R4     *****                 *
C  *                                                                   *
C  *    CONVERT AN INTEGER*4 NUMBER TO A REAL*4 NUMBER.                *
C  *                                                                   *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                 *
C  *********************************************************************
C
      FUNCTION I4R4 (I4_NUM)
C
      IMPLICIT NONE
C
      INTEGER*4 I4_NUM
      REAL*4    I4R4
C  ----------------------------------------------------------------------------
C
      I4R4 = FLOAT(I4_NUM)
C
      RETURN
      END

C  *********************************************************************
C  *                                                                   *
C  *                 *****     FUNCTION R4I2     *****                 *
C  *                                                                   *
C  *    CONVERT REAL*4 NUMBER TO AN INTEGER*2 NUMBER.                  *
C  *                                                                   *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                 *
C  *********************************************************************
C
      FUNCTION R4I2 (R4_NUM)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      REAL*4    R4I2, R4_NUM
C  ----------------------------------------------------------------------------
C
      IF(NINT(R4_NUM).LE.I2_MAX .AND. NINT(R4_NUM).GE.I2_MIN) THEN
         R4I2 = NINT(R4_NUM)
       ELSE
         PRINT *, '******************************************'
         PRINT *, '*    ERROR IN FUNCTION ROUTINE R4I2      *'
         PRINT *, '*                                        *'
         PRINT *, '* R*4 number too large to convert to I*2 *'
         PRINT *, '******************************************'
      ENDIF
C
      RETURN
      END

C  *********************************************************************
C  *                                                                   *
C  *                 *****     FUNCTION R4I4     *****                 *
C  *                                                                   *
C  *    CONVERT REAL*4 NUMBER TO AN INTEGER*4 NUMBER.                  *
C  *                                                                   *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                 *
C  *********************************************************************
C
      FUNCTION R4I4 (R4_NUM)
C
      IMPLICIT NONE
C
      INTEGER*4 R4I4
      REAL*4    R4_NUM
C  ----------------------------------------------------------------------------
C
      R4I4 = NINT(R4_NUM)
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE CHK_PRIME     *****                *
C  *                                                                          *
C  *    CHECKS THE NUMBER "N" FOR THE LARGEST PRIME FACTOR.  IT THEN RETURNS  *
C  *     A NUMBER "N_OUT" WHICH IS THE NEXT SMALLEST NUMBER THAT DOESN'T HAVE *
C  *     A PRIME FACTOR GREATER THAN 19.                                      *
C  *                                                                          *
C  *    IF "EVEN" = .TRUE. THEN ONLY RETURN "N_OUT" THAT IS EVEN.             *
C  *                                                                          *
C  *    AUGUST 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  ****************************************************************************
C
      SUBROUTINE CHK_PRIME (N, N_OUT, EVEN)
C
      IMPLICIT NONE
C
      INTEGER*4 PRIMES(8), I, J, N, N_OUT, N_TEST
      LOGICAL   EVEN
      DATA PRIMES/2,3,5,7,11,13,17,19/
C  ----------------------------------------------------------------------------
C
      N_TEST = N
      IF(EVEN.AND.MOD(N_TEST,2).NE.0) N_TEST = N_TEST - 1
      J = N_TEST
C
   10      DO I=1,8
        IF(MOD(J,PRIMES(I)).EQ.0) THEN
           J = J/PRIMES(I)
           IF(J.EQ.1) THEN                ! NUMBER PASSES TESTS AND IS OK NOW
             N_OUT = N_TEST
             RETURN
           ENDIF
           GO TO 10
         ELSE
        ENDIF
      ENDDO
C  ************************************************
C  *    HERE IF NUMBER IS NOT COMPATIBLE          *
C  *    TRY REDUCING SIZE TO NEXT LOWER NUMBER    *
C  ************************************************
      N_TEST = N_TEST - 1
      IF(EVEN.AND.MOD(N_TEST,2).NE.0) N_TEST = N_TEST - 1
      J = N_TEST
      GO TO 10
C
      END

C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE COPY_I2     *****                *
C  *                                                                          *
C  *    COPY INTEGER*2 ARRAY1 INTO INTEGER*2 ARRAY2.                          *
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                        *
C  ****************************************************************************
C
      SUBROUTINE COPY_I2 (ARRAY1, ARRAY2, ARRAY_SIZE)
C
      IMPLICIT NONE
C
      INTEGER*4 I, ARRAY_SIZE
      INTEGER*2 ARRAY1(ARRAY_SIZE), ARRAY2(ARRAY_SIZE)
C  ----------------------------------------------------------------------------
C
      DO I = 1,ARRAY_SIZE
        ARRAY2(I) = ARRAY1(I)
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE COPY_I4     *****                *
C  *                                                                          *
C  *    COPY INTEGER*4 ARRAY1 INTO INTEGER*4 ARRAY2.                          *
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                        *
C  ****************************************************************************
C
      SUBROUTINE COPY_I4 (ARRAY1, ARRAY2, ARRAY_SIZE)
C
      IMPLICIT NONE
C
      INTEGER*4 I, ARRAY_SIZE, ARRAY1(ARRAY_SIZE), ARRAY2(ARRAY_SIZE)
C  ----------------------------------------------------------------------------
C
      DO I = 1,ARRAY_SIZE
        ARRAY2(I) = ARRAY1(I)
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE COPY_R4     *****                *
C  *                                                                          *
C  *    COPY REAL*4 ARRAY1 INTO REAL*4 ARRAY2.                                *
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                        *
C  ****************************************************************************
C
      SUBROUTINE COPY_R4 (ARRAY1, ARRAY2, ARRAY_SIZE)
C
      IMPLICIT NONE
C
      INTEGER*4 I, ARRAY_SIZE
      REAL*4    ARRAY1(ARRAY_SIZE), ARRAY2(ARRAY_SIZE)
C  ----------------------------------------------------------------------------
C
      DO I = 1,ARRAY_SIZE
        ARRAY2(I) = ARRAY1(I)
      ENDDO
C
      RETURN
      END

C  *******************************************************************
C  *                                                                 *
C  *               *****     FUNCTION DECTOOCT     *****             *
C  *                                                                 *
C  *    FUNCTION TO RETURN OCTAL EQUIVALENT OF DECIMAL INTEGER       *
C  *                                                                 *
C  *    JUNE 1984 T. BAKER, PURDUE UNIVERSITY                        *
C  *******************************************************************
C
      FUNCTION DECTOOCT(NUM)
C
      IMPLICIT NONE
C
      INTEGER*4 NUM, OCTNUM, I, DECTOOCT
C  ----------------------------------------------------------------------------
C
      OCTNUM = 0
      DO I = 1,8
        OCTNUM = OCTNUM +(10**(I-1))*MOD(NUM,8)
        NUM    = NUM/8
      ENDDO
      DECTOOCT = OCTNUM
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE FILE_CHECK     *****               *
C  *                                                                          *
C  *    ROUTINE TO CHECK FOR STANDARD FILE TYPES                              *
C  *                                                                          *
C  *    THE MAIN PURPOSE OF THIS IS TO ENABLE IMG_OPEN, FFT_OPEN, SF_OPEN AND *
C  *     MAP_OPEN TO CHECK FOR INCOMPATIBLE FILE STRUCTURES WITHOUT CRASHING  *
C  *     THE APPLICATION PROGRAM.                                             *
C  *                                                                          *
C  *    JULY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                     *
C  *      Modified 30 Aug 1994 by D. Belnap                              *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (RID OF EM.CMM).                *
C  ****************************************************************************
C
      SUBROUTINE FILE_CHECK (LUN, FILENAME, FILE_ID, IMG_BKG)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, LUN, LRECL, IREC, FILE_ID
      real*4    IMG_BKG
      INTEGER*2 ITMP(2048), I, NCIMG, NRIMG, NBKG
      LOGICAL   FOUND
      CHARACTER FILENAME*64
C  ----------------------------------------------------------------------------
C
C  ******************************
C  *    CHECK IF FILE EXISTS    *
C  ******************************
      INQUIRE (FILE=FILENAME, EXIST=FOUND)
      IF(.NOT.FOUND) THEN
        PRINT *, '*************************'
        PRINT *, '* File can not be found *'
        PRINT *, '*************************'
        FILE_ID = -999                        ! SIGNIFY UNKNOWN FILE_ID
        RETURN
      ENDIF
C  **********************************************************
C  *    CHECK FOR PURDUE FORMAT IMAGE FILE: FILE_ID = -1    *
C  **********************************************************
      CLOSE (UNIT=LUN)
      OPEN (UNIT=LUN, FILE=FILENAME, ERR=100, STATUS='OLD', 
     1     ACCESS='SEQUENTIAL', ACTION='READ', FORM='UNFORMATTED')
      READ(LUN, ERR=100) FILE_ID
      IF(FILE_ID.EQ.-1) THEN
        PRINT *, '**************************************'
        PRINT *, '* This is a PURDUE format IMAGE file *'
        PRINT *, '**************************************'
        RETURN
      ENDIF
C  ************************************************************************
C  *    CHECK FOR BRANDEIS FORMAT IMAGE FILE: THERE IS NO FILE_ID WITH    *
C  *     THIS OLDER TYPE FILE, THEREFORE USE FILE_ID = 0 TO IDENTIFY IT   *
C  ************************************************************************
 100  CLOSE (UNIT=LUN)
      OPEN (UNIT=LUN, FILE=FILENAME, ERR=200, STATUS='OLD', 
     1     ACCESS='SEQUENTIAL', ACTION='READ', FORM='UNFORMATTED')
      READ(LUN, ERR=200) NBKG, NCIMG, NRIMG
      NCOL    = NCIMG
      NROW    = NRIMG
      IMG_BKG = NBKG
C       ****************************************************
C  -----*    CHECK FOR SUSPICIOUS IMG_BKG, NCOL OR NROW    *
C       ****************************************************
      IF(IMG_BKG.LT.0.OR.IMG_BKG.GT.255) GO TO 200
      IF(NCOL.LE.0.OR.NROW.LE.0) GO TO 200
C       *********************************************************
C  -----*    AS ACID TEST, MAKE SURE FIRST DATA RECORD IS OK    *
C       *********************************************************
      READ(LUN,ERR=200) (ITMP(I),I=1,NROW)
      FILE_ID = 0
      PRINT *, '****************************************'
      PRINT *, '* This is a BRANDEIS format IMAGE file *'
      PRINT *, '****************************************'
      REWIND LUN                     ! RESET POINTER TO FIRST DATA RECORD
      READ(LUN) NBKG, NCIMG, NRIMG
      NCOL    = NCIMG
      NROW    = NRIMG
      IMG_BKG = NBKG
      RETURN
C  *******************************************
C  *    CHECK FOR "FFT" FILE: FILE_ID = 1    *
C  *******************************************
 200  CLOSE (UNIT=LUN)
      LRECL = 4096
  300      LRECL = LRECL/2
      IF(LRECL.LT.2) GO TO 400
      OPEN (UNIT=LUN, FILE=FILENAME, ERR=300, STATUS='OLD',
     1     ACCESS='DIRECT', RECL=LRECL) !RSS, ASSOCIATEVARIABLE=IREC)
      READ (LUN,REC=1,ERR=400) FILE_ID
      IF(FILE_ID.EQ.1) THEN
        PRINT *, '************************************'
        PRINT *, '* This is a FOURIER TRANSFORM file *'
        PRINT *, '************************************'
        RETURN
      ENDIF
C  ******************************************
C  *    CHECK FOR "SF" FILE: FILE_ID = 2    *
C  ******************************************
 400  CLOSE (UNIT=LUN)
      OPEN (UNIT=LUN, FILE=FILENAME, ERR=450, STATUS='OLD',
     1     ACCESS='SEQUENTIAL', ACTION='READ', FORM='FORMATTED')
      READ (LUN,*,ERR=450) FILE_ID
      IF(FILE_ID.EQ.2) THEN
        PRINT *, '******************************************'
        PRINT *, '* This is an ASCII STRUCTURE FACTOR file *'
        PRINT *, '******************************************'
        RETURN
      ENDIF
C  *******************************************
C  *    CHECK FOR "SF" FILE: FILE_ID = -2    *
C  *******************************************
 450  CLOSE (UNIT=LUN)
      OPEN (UNIT=LUN, FILE=FILENAME, ERR=500, STATUS='OLD',
     1     ACCESS='SEQUENTIAL', ACTION='READ', FORM='FORMATTED')
      READ (LUN,*,ERR=500) FILE_ID
      IF(FILE_ID.EQ.-2) THEN
        PRINT *, '******************************************'
        PRINT *, '* This is a binary STRUCTURE FACTOR file *'
        PRINT *, '******************************************'
        RETURN
      ENDIF
C  *******************************************
C  *    CHECK FOR "MAP" FILE: FILE_ID = 3    *
C  *******************************************
 500  CLOSE (UNIT=LUN)
      OPEN (UNIT=LUN, FILE=FILENAME, ERR=600, STATUS='OLD',
     1     ACCESS='SEQUENTIAL', ACTION='READ', FORM='UNFORMATTED')
      READ (LUN,ERR=600) FILE_ID
      IF(FILE_ID.EQ.3 .OR. FILE_ID.EQ.-3) THEN
        PRINT *, '**********************'
        PRINT *, '* This is a MAP file *'
        PRINT *, '**********************'
        RETURN
      ENDIF
C
  600      FILE_ID = -999                              ! SIGNIFY UNKNOWN FILE_ID
      PRINT *, '*************************************'
      PRINT *, '* File type not IMG, FFT, SF or MAP *'
      PRINT *, '*************************************'
      RETURN
C
      END

C  **********************************************************************
C  *                                                                    *
C  *               *****     SUBROUTINE GET_YESNO     *****             *
C  *                                                                    *
C  *    ACCEPT TOUCH-KEY RESPONSE TO YES/NO QUESTION.                   *
C  *    RETURN LOGICAL ANSWER.  DEFAULT (<CR>) ANSWER PASSED THROUGH    *
C  *     ARGUMENT LIST.                                                 *
C  *                                                                    *
C  *    NOVEMBER 1989   PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    LAST MODIFIED 28-DEC-1999 by TSB (VT_READCHAR not working).     *
C  **********************************************************************
C
      SUBROUTINE GET_YESNO (ANSWER, DEFAULT)
C
      IMPLICIT NONE
C
      INTEGER*2   ICHR
      LOGICAL     ANSWER, DEFAULT
      CHARACTER*1 IKEY
C  ----------------------------------------------------------------------------
C
  100      READ (*,'(A1)') IKEY
      IF(IKEY.EQ.'Y' .OR. IKEY.EQ.'y') THEN 
         ANSWER = .TRUE.
       ELSE IF(IKEY.EQ.'N' .OR. IKEY.EQ.'n') THEN
         ANSWER = .FALSE.
       ELSE
         ICHR = ICHAR(IKEY)
ctsb         IF(ICHR.EQ.13 .OR. ICHR.EQ.221) THEN         ! Doesn't work 28-Dec-99 ???
         IF(ICHR.EQ.32) THEN
            ANSWER = DEFAULT
          ELSE
            PRINT *, '**********************************'
            PRINT *, '* Must respond with Y, N or <CR> *'
            PRINT *, '**********************************'
            WRITE(*,'(A)') '$Enter choice:'
            GO TO 100
         ENDIF
      ENDIF
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE GNOI_R4_SIZE     *****               *
C  *                                                                          *
C  *      Determines the size of the array needed based on the increment (INC)  *
C  *     (INC) used to sample the Normal distribution between -3 and 3        *
C  *     standard deviations.                                                 *
C  *                                                                          *
C  *    Based on code written by L. Mattern (1987) and N. Dilley (1990).      *
C  *                                                                          *
C  *    DECEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
      SUBROUTINE GNOI_R4_SIZE (INC, NUM)
C
      IMPLICIT NONE
C
      INTEGER*4 I, J, K, NUM
      REAL*4        X, FX, INC, PI, RADIAN, DEGREE, SIG_TWOPI
C  ----------------------------------------------------------------------------
C
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      SIG_TWOPI = 1.0/SQRT(2.0*PI)
      NUM       = 0                                 ! ARRAY SIZE COUNTER
C  ******************************************************************
C  *    THE 500 IN THE EQUATION IS AN ARBITRARY SCALE FACTOR USED   *
C  *     TO ASSURE THAT A LARGE ENOUGH ARRAY WILL BE GENERATED.     *
C  ******************************************************************

C     Replace real do-loop variable with integer. Original code
C     (commented out) follows

      do k = 1, nint(6.0/INC)+1
         X = -3.0 + (k-1)*INC
         FX = 500.0*(SIG_TWOPI * EXP(-0.5*X*X))
         J  = NINT(FX)
         IF(NINT(FX).GT.0) NUM = NUM + NINT(FX)
      ENDDO

C      DO X = -3.0, 3.0, INC
C         FX = 500.0*(SIG_TWOPI * EXP(-0.5*X*X))
C         J  = NINT(FX)
C         IF(NINT(FX).GT.0) NUM = NUM + NINT(FX)
C      ENDDO

      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE GNO_R4_FILL     *****               *
C  *                                                                          *
C  *    Loads array gauss(num) with a gaussian distribution between -3 and 3  *
C  *     standard deviations.                                                 *
C  *                                                                          *
C  *      Uses GNOI_R4_SIZE to determines the size of the array GAUSS needed,   *
C  *     based on the increment (INC) used to sample the normal distribution  *
C  *     between -3 and 3 standard deviations.                                *
C  *                                                                          *
C  *                             1.0                                          *
C  *    Formula used: f(x) = ------------ exp(-x**2/2.0)                      *
C  *                         sqrt(2.0*pi)                                     *
C  *                                                                          *
C  *    Based on code written by L. Mattern (1987) and N. Dilley (1990).      *
C  *                                                                          *
C  *    DECEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
      SUBROUTINE GNOI_R4_FILL (GAUSS, NUM, INC)
C
      IMPLICIT NONE
C
      INTEGER*4 NUM, COUNT, I, J, K
      REAL*4        X, FX, INC, PI, RADIAN, DEGREE, SIG_TWOPI, GAUSS(NUM)
C  ----------------------------------------------------------------------------
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      SIG_TWOPI = 1.0/SQRT(2.0*PI)
      COUNT     = 0

C     Replace real do-loop variable with integer. Original code
C     (commented out) follows

      do k = 1, nint(6.0/INC)+1
         X = -3.0 + (k-1)*INC
         FX = 500.0*(SIG_TWOPI * EXP(-0.5*X*X))
         J  = NINT(FX)
         IF(J.GT.0) THEN
            DO I = 1,J
               COUNT = COUNT + 1
               GAUSS(COUNT) = X
            ENDDO
         ENDIF
      ENDDO

c      DO X = -3.0, 3.0, INC
c        FX = 500.0*(SIG_TWOPI * EXP(-0.5*X*X))
cCCC        WRITE(*,'(A,F10.2,A,F10.2)') ' X:', X, ' F(X):', FX
c        J  = NINT(FX)
c        IF(J.GT.0) THEN
c          DO I = 1,J
c            COUNT = COUNT + 1
c            GAUSS(COUNT) = X
cCCC            WRITE(*,'(I8,F10.2)') COUNT, GAUSS(COUNT)
c          ENDDO
c        ENDIF
c      ENDDO

      RETURN
      END

C  *******************************************************************
C  *                                                                 *
C  *               *****     SUBROUTINE HITCR     *****              *
C  *                                                                 *
C  *    RESUME PROGRAM AFTER ENTERING A <CR>                         *
C  *                                                                 *
C  *    JUNE 1986 T. BAKER, PURDUE UNIVERSITY                        *
C  *******************************************************************
C
      SUBROUTINE HITCR
C
      IMPLICIT NONE
C
      INTEGER*4 I
C  ----------------------------------------------------------------------------
C
      PRINT *, '********************************'
      PRINT *, '* Hit <CR> to continue program *'
      PRINT *, '********************************'
      READ (*,*,ERR=100) I
C
  100      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE HKL_DSTAR     *****                *
C  *                                                                          *
C  *    COMPUTE RECIPROCAL LATTICE DISTANCE TO HKL REFLECTION                 *
C  *                                                                          *
C  *    APRIL 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                    *
C  *    LAST MODIFIED  2-NOV-85 BY T. BAKER                                   *
C  ****************************************************************************
C
      SUBROUTINE HKL_DSTAR (H, K, L, D_STAR, A_STAR, B_STAR, C_STAR,
     1     ALP_STAR, BET_STAR, GAM_STAR)
C
      IMPLICIT NONE
C
      REAL*4    D_STAR, A_STAR, B_STAR, C_STAR
      REAL*4    ALP_STAR, BET_STAR, GAM_STAR
      INTEGER*2 H, K, L, IH, IK, IL
      real*8    deg_to_rad
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      IH = ABS(H)
      IK = ABS(K)
      IL = ABS(L)
      D_STAR = SQRT( (IH*A_STAR)**2 + (IK*B_STAR)**2 + (IL*C_STAR)**2
     1             + 2*IH*IK*A_STAR*B_STAR*COS(deg_to_rad*GAM_STAR)
     2             + 2*IH*IL*A_STAR*C_STAR*COS(deg_to_rad*BET_STAR)
     3             + 2*IK*IL*B_STAR*C_STAR*COS(deg_to_rad*ALP_STAR))
C
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *                  *****     SUBROUTINE HKL_TO_HK     *****              *
C  *                                                                        *
C  *    TRANSFORM FROM HKL INDICIES TO H'K' USING THE INVERSE OF THE        *
C  *       TRANSFORMATION MATRIX "T_MATRIX"                                 *
C  *                                                                        *
C  *     T (1,1),(1,2),(1,3) = INDICIES OF FIRST PRINCIPAL LATTICE VECTOR   *
C  *                              (NEAR HORIZONTAL)                         *
C  *     T (2,1),(2,2),(2,3) = INDICIES OF SECOND PRINCIPAL LATTICE VECTOR  *
C  *                              (NEAR VERTICAL)                           *
C  *                                                                        *
C  *    H, K, L = INPUT 3D MILLER INDICIES                                  *
C  *    IH, IK  = OUTPUT 2D MILLER INDICIES                                 *
C  *                                                                        *
C  *    JULY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                   *
C  **************************************************************************
C
      SUBROUTINE HKL_TO_HK (H, K, L, IH, IK, T_MATRIX)
C
      IMPLICIT NONE
C
      INTEGER*4 T_MATRIX(2,3)
      INTEGER*2 H, K, L, IH, IK
C  ----------------------------------------------------------------------------
C
      IH = H*T_MATRIX(1,1) + K*T_MATRIX(1,2) + L*T_MATRIX(1,3)
      IK = H*T_MATRIX(2,1) + K*T_MATRIX(2,2) + L*T_MATRIX(2,3)
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE HK_TO_HKL     *****                *
C  *                                                                          *
C  *    TRANSFORM FROM H'K' INDICIES TO HKL                                   *
C  *                                                                          *
C  *     T (1,1),(1,2),(1,3) = INDICIES OF FIRST PRINCIPAL LATTICE VECTOR     *
C  *                              (NEAR HORIZONTAL)                           *
C  *     T (2,1),(2,2),(2,3) = INDICIES OF SECOND PRINCIPAL LATTICE VECTOR    *
C  *                              (NEAR VERTICAL)                             *
C  *                                                                          *
C  *    IH, IK  = INPUT 2D MILLER INDICIES                                    *
C  *    H, K, L = OUTPUT 3D MILLER INDICIES                                   *
C  *                                                                          *
C  *    JULY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                     *
C  ****************************************************************************
C
      SUBROUTINE HK_TO_HKL (IH, IK, H, K, L, T_MATRIX)
C
      IMPLICIT NONE
C
      INTEGER*2 H, K, L, IH, IK
      INTEGER*4 T_MATRIX (2,3)
C  ----------------------------------------------------------------------------
C
      H = IH*T_MATRIX(1,1) + IK*T_MATRIX(2,1)
      K = IH*T_MATRIX(1,2) + IK*T_MATRIX(2,2)
      L = IH*T_MATRIX(1,3) + IK*T_MATRIX(2,3)
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     FUNCTION I2_MEDIAN_3     *****               *
C  *                                                                          *
C  *    DETERMINES THE MEDIAN OF THREE INTEGER*2 NUMBERS.                     *
C  *                                                                          *
C  *    ADAPTED FROM FUNCTION MED WRITTEN BY W. GROCHULSKI.                   *
C  *                                                                          *
C  *    FEBRUARY 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER                   *
C  *    LAST MODIFIED 27-FEB-1990 BY T. BAKER                                 *
C  ****************************************************************************
C
      FUNCTION I2_MEDIAN_3 (A, B, C)
C
      IMPLICIT NONE
C
      INTEGER*2 A, B, C, D, I2_MEDIAN_3
C -----------------------------------------------------------------------------
C
      IF(A.LE.C) THEN
         D           = MAX(A,B)
         I2_MEDIAN_3 = MIN(D,C)
       ELSE
         D           = MIN(A,B)
         I2_MEDIAN_3 = MAX(D,C)
      ENDIF
C
      RETURN
      END

C  ****************************************************
C  *                                                  *
C  *      *****     SUBROUTINE LATCEN     *****       *
C  *                                                  *
C  *    ADOPTED FROM JOACHIM FRANK'S SPIDER SYSTEM    *
C  ****************************************************
C
      SUBROUTINE LATCEN (INTRY,IH,IK,X,Y,AX,AY,BX,BY,XCEN,YCEN)
C
      IMPLICIT NONE
C
      INTEGER*2 I, J, IPT
      INTEGER*4 INTRY, IH, IK
      REAL*4    XM(2,3), YM(2,3), X, Y, AX, AY, BX, BY, TST
      REAL*4        XCEN, YCEN
C  ----------------------------------------------------------------------------
C
      IF(INTRY.EQ.1) THEN
        IPT = IPT+1
        XM(1,3) = XM(1,3)+IH*(X-XCEN)
        XM(2,3) = XM(2,3)+IK*(X-XCEN)
        XM(1,1) = XM(1,1)+IH*IH
        XM(1,2) = XM(1,2)+IH*IK
        XM(2,1) = XM(1,2)
        XM(2,2) = XM(2,2)+IK*IK
        YM(1,3) = YM(1,3)+IH*(Y-YCEN)
        YM(2,3) = YM(2,3)+IK*(Y-YCEN)
        YM(1,1) = XM(1,1)
        YM(1,2) = XM(1,2)
        YM(2,1) = XM(2,1)
        YM(2,2) = XM(2,2)
        RETURN
      ENDIF
      IF(INTRY.EQ.2) THEN
        IF(IPT.LT.2) THEN
          PRINT *, '***************************************'
          PRINT *, '* LATCEN requires more than one point *'
          PRINT *, '***************************************'
          RETURN
        ENDIF
        CALL SOLV2D (XM, TST)
        CALL SOLV2D (YM, TST)
        IF(TST.GE.0.5) THEN
          PRINT *, '**********************************'
          PRINT *, '* SINGULARITY IN EQUATIONS FOUND *'
          PRINT *, '**********************************'
          RETURN
        ENDIF
        AX = XM(1,3)
        BX = XM(2,3)
        AY = YM(1,3)
        BY = YM(2,3)
        RETURN
      ENDIF
C  **********************************************
C  *    HERE TO INIITIALIZE XM, YM MATRICIES    *
C  **********************************************
      DO I = 1,3
        DO J = 1,2
          XM(J,I) = 0.0
          YM(J,I) = 0.0
        ENDDO
      ENDDO
      IPT = 0
      RETURN
      END

C  *****************************************************
C  *                                                   *
C  *      *****     SUBROUTINE LATCEN2     *****       *
C  *                                                   *
C  *    ADOPTED FROM JOACHIM FRANK'S SPIDER SYSTEM     *
C  *    MODIFIED FEB 27, 1997, T. BAKER                *
C  *****************************************************
C
      SUBROUTINE LATCEN2 (IH1, IK1, X1, Y1, IH2, IK2, X2, Y2, AX, AY, 
     1 BX, BY, XCEN, YCEN)
C
      IMPLICIT NONE
C
      INTEGER*2 I, J
      INTEGER*4 IH1, IK1, IH2, IK2
      REAL*4    XM(2,3), YM(2,3), X1, Y1, X2, Y2, AX, AY, BX, BY, TST
      REAL*4        XCEN, YCEN
C  ----------------------------------------------------------------------------
C
C  *************************************
C  *    INITIALIZE XM, YM MATRICIES    *
C  *************************************
      DO I = 1,3
        DO J = 1,2
          XM(J,I) = 0.0
          YM(J,I) = 0.0
        ENDDO
      ENDDO
C
      XM(1,3) = IH1*(X1-XCEN) + IH2*(X2-XCEN) 
      XM(2,3) = IK1*(X1-XCEN) + IK2*(X2-XCEN) 
      XM(1,1) = IH1*IH1 + IH2*IH2
      XM(1,2) = IH1*IK1 + IH2*IK2 
      XM(2,1) = XM(1,2) 
      XM(2,2) = IK1*IK1 + IK2*IK2
C
      YM(1,3) = IH1*(Y1-YCEN) + IH2*(Y2-YCEN) 
      YM(2,3) = IK1*(Y1-YCEN) + IK2*(Y2-YCEN) 
      YM(1,1) = XM(1,1)
      YM(1,2) = XM(1,2)
      YM(2,1) = XM(2,1)
      YM(2,2) = XM(2,2)
C
      CALL SOLV2D (XM, TST)
      CALL SOLV2D (YM, TST)
      IF(TST.GE.0.5) THEN
        PRINT *, '**********************************'
        PRINT *, '* SINGULARITY IN EQUATIONS FOUND *'
        PRINT *, '**********************************'
        RETURN
      ENDIF
      AX = XM(1,3)
      BX = XM(2,3)
      AY = YM(1,3)
      BY = YM(2,3)
C
      RETURN
      END

C  ************************************************************************
C  *                                                                      *
C  *                 *****     FUNCTION NUMCHARI2     *****               *
C  *                                                                      *
C  *    DETERMINE THE NUMBER OF CHARACTERS IN THE INTEGER*2 NUMBER "N"    *
C  *                                                                      *
C  *    SEPTEMBER 1986  PURDUE UNIVERSITY VAX 11/750, T. BAKER            *
C  ************************************************************************
C
      FUNCTION NUMCHARI2 (N)
C
      IMPLICIT NONE
C
      REAL*4    REALNUM
      INTEGER*2 N
      INTEGER*4 NUMCHARI2, NCHR
C  ----------------------------------------------------------------------------
C
      NCHR    = 0
      REALNUM = ABS(FLOAT(N))
      DO WHILE (REALNUM.GE.1.0)
        NCHR    = NCHR+1
        REALNUM = REALNUM/10.0
      ENDDO
      IF(N.LE.0) NCHR = NCHR+1
      NUMCHARI2 = NCHR
C
       RETURN
      END

C  ************************************************************************
C  *                                                                      *
C  *                 *****     FUNCTION NUMCHARI4     *****               *
C  *                                                                      *
C  *    DETERMINE THE NUMBER OF CHARACTERS IN THE INTEGER*4 NUMBER "N"    *
C  *                                                                      *
C  *    SEPTEMBER 1986  PURDUE UNIVERSITY VAX 11/750, T. BAKER            *
C  ************************************************************************
C
      FUNCTION NUMCHARI4 (N)
C
      IMPLICIT NONE
C
      REAL*4    REALNUM
      INTEGER*4 N, NUMCHARI4, NCHR
C  ----------------------------------------------------------------------------
C
      NCHR    = 0
      REALNUM = ABS(FLOAT(N))
      DO WHILE (REALNUM.GE.1.0)
        NCHR    = NCHR+1
        REALNUM = REALNUM/10.0
      ENDDO
      IF(N.LE.0) NCHR = NCHR+1
      NUMCHARI4 = NCHR
C
       RETURN
      END

C  ************************************************************************
C  *                                                                      *
C  *               *****    FUNCTION NUM_DIGIT_R4    *****                *
C  *                                                                      *
C  *    For a real number (REAL*4), determines the number of digits to    *
C  *      the left of the decimal point.  It then adds the number of        *
C  *      digits to the right (given) to give total digits in number.       *
C  *      Adapted from NUMCHARI2.                                      *
C  *                                                                      *
C  *    APRIL 1995  Purdue University,  D. Belnap                    *
C  *    LAST MODIFIED  3-DEC-1998 BY TSB (Fixed bug in count).            *
C  ************************************************************************
C
      FUNCTION NUM_DIGIT_R4 (REALNUM, RIGHT_DIGITS)
C
      IMPLICIT NONE
C
      REAL*4    DUMMY, REALNUM
      INTEGER*4 NUM_DIGIT_R4, N_DIGIT, RIGHT_DIGITS
C  ----------------------------------------------------------------------------
C
      N_DIGIT = 2                        ! DECIMAL POINT PLUS 1 LEFT DIGIT
      IF(REALNUM.LT.0.0) N_DIGIT = 3            ! MINUS SIGN
      DUMMY   = ABS(REALNUM)
C
      DO WHILE (DUMMY.GE.10.0)
        N_DIGIT = N_DIGIT + 1
        DUMMY   = DUMMY/10.0
      ENDDO
C
      IF (RIGHT_DIGITS.GT.0)  N_DIGIT = N_DIGIT + RIGHT_DIGITS
      NUM_DIGIT_R4 = N_DIGIT
C
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE PIRADDEG     *****                 *
C  *                                                                          *
C  *    ROUTINE TO COMPUTE PI, RADIAN AND DEGREE TO MAXIMUM SINGLE PRECISION  *
C  *      ACCURACY                                                            *
C  *                                                                          *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER                 *
C  ****************************************************************************
C
      SUBROUTINE PIRADDEG (PI, RADIAN, DEGREE)
C
      IMPLICIT NONE
C
      REAL*4 PI, RADIAN, DEGREE
C  ----------------------------------------------------------------------------
C
      PI     = 4.0*ATAN2(1.0,1.0)
      RADIAN = PI/180.0
      DEGREE = 1.0/RADIAN
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE PIRADDEG_D     *****               *
C  *                                                                          *
C  *    ROUTINE TO COMPUTE PI, RADIAN AND DEGREE TO MAXIMUM DOUBLE PRECISION  *
C  *      ACCURACY                                                            *
C  *                                                                          *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                      *
C  ****************************************************************************
C
      SUBROUTINE PIRADDEG_D (DPI, DRADIAN, DDEGREE)
C
      IMPLICIT NONE
C
      REAL*8 DPI, DRADIAN, DDEGREE
C  ----------------------------------------------------------------------------
C
      DPI     = 4.0*ATAN2(1.0,1.0)
      DRADIAN = DPI/180.0
      DDEGREE = 1.0/DRADIAN
C
      RETURN
      END

C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE REAL_TO_REC     *****             *
C  *                                                                     *
C  *    COMPUTE REAL SPACE PARAMETERS KNOWING THE RECIPROCAL LATTICE     *
C  *     PARAMETERS                                                      *
C  *                                                                     *
C  *    APRIL 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER               *
C  ***********************************************************************
C
      SUBROUTINE REAL_TO_REC (A_CELL,B_CELL,C_CELL,ALP,BET,GAM,VOL,
     1 A_STAR,B_STAR,C_STAR,VOL_STAR,ALP_STAR,BET_STAR,GAM_STAR)
C
      IMPLICIT NONE
C
      INTEGER*2 I, J
      REAL*4    TEST(3,2), A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL
      REAL*4    A_STAR, B_STAR, C_STAR, VOL_STAR
      REAL*4    ALP_STAR, BET_STAR, GAM_STAR
      real*4    cosalp, cosbet, cosgam, sinalp, sinbet, singam
      real*8    deg_to_rad, rad_to_deg
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      rad_to_deg = 1.0d0/deg_to_rad

      TEST(1,1) = A_CELL
      TEST(2,1) = B_CELL
      TEST(3,1) = C_CELL
      TEST(1,2) = ALP
      TEST(2,2) = BET
      TEST(3,2) = GAM
      IF(A_CELL.LE.0.0) A_CELL = -1.0
      IF(B_CELL.LE.0.0) B_CELL = -1.0
      IF(C_CELL.LE.0.0) C_CELL = -1.0
      IF(ALP.LE.0.0)       ALP = 90.0
      IF(BET.LE.0.0)       BET = 90.0
      IF(GAM.LE.0.0)       GAM = 90.0

      cosalp = cos(alp*deg_to_rad)
      cosbet = cos(bet*deg_to_rad)
      cosgam = cos(gam*deg_to_rad)
      sinalp = sin(alp*deg_to_rad)
      sinbet = sin(bet*deg_to_rad)
      singam = sin(gam*deg_to_rad)

      VOL = A_CELL*B_CELL*C_CELL * SQRT (1.0 + 
     1     2*COSALP*COSBET*COSGAM-COSALP**2
     2     -COSBET**2-COSGAM**2)
      A_STAR = B_CELL*C_CELL*SINALP/VOL
      B_STAR = C_CELL*A_CELL*SINBET/VOL
      C_STAR = A_CELL*B_CELL*SINGAM/VOL
      VOL    = ABS(VOL)
      IF(A_STAR.LT.0.0) A_STAR = 0.0
      IF(B_STAR.LT.0.0) B_STAR = 0.0
      IF(C_STAR.LT.0.0) C_STAR = 0.0
      ALP_STAR = rad_to_deg * ACOS((COSBET*COSGAM-COSALP) / 
     1                     (SINBET*SINGAM))
      BET_STAR = rad_to_deg * ACOS((COSGAM*COSALP-COSBET) / 
     1                     (SINGAM*SINALP))
      GAM_STAR = rad_to_deg * ACOS((COSALP*COSBET-COSGAM) / 
     1                     (SINALP*SINBET))
      VOL_STAR = 1.0/VOL

      A_CELL = TEST(1,1)
      B_CELL = TEST(2,1)
      C_CELL = TEST(3,1)
      ALP    = TEST(1,2)
      BET    = TEST(2,2)
      GAM    = TEST(3,2)
C
      RETURN
      END

C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE REC_TO_REAL     *****             *
C  *                                                                     *
C  *    COMPUTE RECIPROCAL LATTICE PARAMETERS KNOWING THE REAL SPACE     *
C  *     PARAMETERS                                                      *
C  *                                                                     *
C  *    JULY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                *
C  ***********************************************************************
C
      SUBROUTINE REC_TO_REAL (A_CELL,B_CELL,C_CELL,ALP,BET,GAM,VOL,
     1 A_STAR,B_STAR,C_STAR,VOL_STAR,ALP_STAR,BET_STAR,GAM_STAR)
C
      IMPLICIT NONE
C
      REAL*4    TEST(3,2), A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL
      REAL*4    A_STAR, B_STAR, C_STAR, VOL_STAR
      REAL*4    ALP_STAR, BET_STAR, GAM_STAR
      real*8    deg_to_rad, rad_to_deg
      real*4    cosalp_star, cosbet_star, cosgam_star
      real*4    sinalp_star, sinbet_star, singam_star
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      rad_to_deg = 1.0d0/deg_to_rad

      TEST(1,1) = A_STAR
      TEST(2,1) = B_STAR
      TEST(3,1) = C_STAR
      TEST(1,2) = ALP_STAR
      TEST(2,2) = BET_STAR
      TEST(3,2) = GAM_STAR
      IF(A_STAR.LE.0.0)   A_STAR   = -1.0
      IF(B_STAR.LE.0.0)   B_STAR   = -1.0
      IF(C_STAR.LE.0.0)   C_STAR   = -1.0
      IF(ALP_STAR.LE.0.0) ALP_STAR = 90.0
      IF(BET_STAR.LE.0.0) BET_STAR = 90.0
      IF(GAM_STAR.LE.0.0) GAM_STAR = 90.0

      cosalp_star = cos(alp_star*deg_to_rad)
      cosbet_star = cos(bet_star*deg_to_rad)
      cosgam_star = cos(gam_star*deg_to_rad)
      sinalp_star = sin(alp_star*deg_to_rad)
      sinbet_star = sin(bet_star*deg_to_rad)
      singam_star = sin(gam_star*deg_to_rad)

      VOL_STAR = A_STAR*B_STAR*C_STAR * SQRT( 1.0 + 
     1     2*COSALP_STAR*COSBET_STAR*COSGAM_STAR
     1      -COSALP_STAR**2 
     1      -COSBET_STAR**2
     1      -COSGAM_STAR**2)
      A_CELL = B_STAR * C_STAR * SINALP_STAR/VOL_STAR
      B_CELL = C_STAR * A_STAR * SINBET_STAR/VOL_STAR
      C_CELL = A_STAR * B_STAR * SINGAM_STAR/VOL_STAR
      VOL_STAR = ABS(VOL_STAR)
      IF(A_CELL.LT.0.0) A_CELL = 0.0
      IF(B_CELL.LT.0.0) B_CELL = 0.0
      IF(C_CELL.LT.0.0) C_CELL = 0.0
      ALP = rad_to_deg * ACOS((COSBET_STAR*COSGAM_STAR-COSALP_STAR) / 
     1            (SINBET_STAR*SINGAM_STAR))
      BET = rad_to_deg * ACOS((COSGAM_STAR*COSALP_STAR-COSBET_STAR) / 
     1            (SINGAM_STAR*SINALP_STAR))
      GAM = rad_to_deg * ACOS((COSALP_STAR*COSBET_STAR-COSGAM_STAR) / 
     1            (SINALP_STAR*SINBET_STAR))
      VOL = 1.0/VOL_STAR
C
      A_STAR   = TEST(1,1)
      B_STAR   = TEST(2,1)
      C_STAR   = TEST(3,1)
      ALP_STAR = TEST(1,2)
      BET_STAR = TEST(2,2)
      GAM_STAR = TEST(3,2)
C
      RETURN
      END

C  ***************************************************************
C  *                                                             *
C  *            *****     SUBROUTINE SEED     *****              *
C  *                                                             *
C  *    GENERATES THE SEED FOR RANDOM NUMBER GENERATOR           *
C  *                                                             *
C  *    THE LARGEST INTEGER*4 NUMBER IN THE VAX IS 2147483647    *
C  *     2147483647/(24*60*60 = seconds in a day) = 24855        *
C  *                                                             *
C  *    LAST MODIFIED 27-JUL-86 BY T. BAKER                      *
C  ***************************************************************
C
      SUBROUTINE SEED (ISEED)
C
      IMPLICIT NONE
C
      REAL*4    SECNDS
      INTEGER*4 ISEED
C  ----------------------------------------------------------------------------
C
      ISEED = 24855 * INT(SECNDS(0.0))
C
      RETURN
      END

C  ****************************************************
C  *                                                  *
C  *      *****     SUBROUTINE SOLV2D     *****       *
C  *                                                  *
C  *    ADOPTED FROM JOACHIM FRANK'S SPIDER SYSTEM    *
C  ****************************************************
C
      SUBROUTINE SOLV2D (A, TST)
C
      IMPLICIT NONE
C
      REAL*4 A(2,3), TST, D, P, Q
C -----------------------------------------------------------------------------
C
      TST = 0.0
      D   = A(1,1)*A(2,2) - A(1,2)*A(2,1)
      P   = A(1,3)*A(2,2) - A(2,3)*A(1,2)
      Q   = A(2,3)*A(1,1) - A(1,3)*A(2,1)
      IF(D.NE.0.0) GO TO 100
      IF(P.EQ.0.0.AND.Q.EQ.0.0) GO TO 300
      GO TO 200
  100      IF(ABS(D).GE.(1.E-6)*ABS(P).AND.ABS(D).GE.(1.E-6)*ABS(Q))
     1 GO TO 400
  200      CONTINUE
      TST    = 1.0
  300      A(1,3) = 0.0
      A(2,3) = 0.0
      RETURN
  400      A(1,3) = P/D
      A(2,3) = Q/D
      RETURN
      END

C  ******************************************************************
C  *                                                                *
C  *            *****     SUBROUTINE SWAP_INT2     *****            *
C  *                                                                *
C  *    SWAPS TWO INTEGER*2 NUMBERS.                                *
C  *                                                                *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER            *
C  ******************************************************************
C
      SUBROUTINE SWAP_INT2 (I, J)
C
      IMPLICIT NONE
C
      INTEGER*2 I, J, ITMP
C  ----------------------------------------------------------------------------
C
      ITMP = I
      I    = J
      J    = ITMP
C
      RETURN
      END

C  ******************************************************************
C  *                                                                *
C  *            *****     SUBROUTINE SWAP_INT4     *****            *
C  *                                                                *
C  *    SWAPS TWO INTEGER*4 NUMBERS.                                *
C  *                                                                *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER            *
C  ******************************************************************
C
      SUBROUTINE SWAP_INT4 (I, J)
C
      IMPLICIT NONE
C
      INTEGER*4 I, J, ITMP
C  ----------------------------------------------------------------------------
C
      ITMP = I
      I    = J
      J    = ITMP
C
      RETURN
      END

C  ******************************************************************
C  *                                                                *
C  *           *****     SUBROUTINE SWAP_REAL4     *****            *
C  *                                                                *
C  *    SWAPS TWO REAL*4 NUMBERS.                                   *
C  *                                                                *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER            *
C  ******************************************************************
C
      SUBROUTINE SWAP_REAL4 (A, B)
C
      IMPLICIT NONE
C
      REAL*4 A, B, TMP
C  ----------------------------------------------------------------------------
C
      TMP = A
      A   = B
      B   = TMP
C
      RETURN
      END

C  ******************************************************************
C  *                                                                *
C  *           *****     SUBROUTINE SWAP_REAL8     *****            *
C  *                                                                *
C  *    SWAPS TWO REAL*8 NUMBERS.                                   *
C  *                                                                *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER            *
C  ******************************************************************
C
      SUBROUTINE SWAP_REAL8 (A, B)
C
      IMPLICIT NONE
C
      REAL*8 A, B, TMP
C  ----------------------------------------------------------------------------
C
      TMP = A
      A   = B
      B   = TMP
C
      RETURN
      END

C  ******************************************************************
C  *                                                                *
C  *             *****     SUBROUTINE SWBYTE     *****              *
C  *                                                                *
C  *    SWAP BYTES IN INTEGERS.                                     *
C  *                                                                *
C  *    ADAPTED FROM CODE WRITTEN BY M. UNSER (NIH) 6/88            *
C  *    NOVEMBER 1994  PURDUE UNIVERSITY AXP 3000/400, T. BAKER     *
C  ******************************************************************
C
      SUBROUTINE SWBYTE (BYTE, N)
C
      INTEGER*4 N
      LOGICAL*1 BYTE(N), TMP
C  ----------------------------------------------------------------------------
C
      DO I = 1,N/2
        TMP         = BYTE(I)
        BYTE(I)     = BYTE(N+1-I)
        BYTE(N+1-I) = TMP
      ENDDO
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *                *****     SUBROUTINE TSBEND     *****               *
C  *                                                                    *
C  *    MARKS THE END OF A PROGRAM BY TYPING 80 *'s ON SYS$OUTPUT       *
C  **********************************************************************
C
      SUBROUTINE TSBEND
C
C  ----------------------------------------------------------------------------
      WRITE(*,1000)
 1000      FORMAT(1X,80('*'))
      CALL EXIT
      END

C ### contains: box_menu, gauss_load, sys$dochelp
C  **********************************************************************
C  *                                                      *
C  *            *****      subroutine BOX_MENU        *****                  *
C  *                                                      *
C  *            Routine to make menu box on screen providing [num]      *
C  *      options to user.                                     *
C  *                                                      *
C  *      Created:  July 10th, 1990  by nrd@bragg.bio.purdue.edu            *
C  **********************************************************************
C
      SUBROUTINE BOX_MENU (COMMANDS, NUM)
C
      IMPLICIT NONE
C
      INTEGER*4    NUM, I, STRING_LEN, MAX_LENGTH, ENDROW
      CHARACTER*80 COMMANDS(NUM), BORDER, DASHES
C  -----------------------------------------------------------------------
      BORDER='**************************************************'//
     1 '******************************'
      DASHES='--------------------------------------------------'//
     1 '------------------------------'
C
      MAX_LENGTH=0
C
      DO I=1,NUM
        IF (STRING_LEN(COMMANDS(I)).GT.MAX_LENGTH) MAX_LENGTH = 
     1 STRING_LEN(COMMANDS(I))
      ENDDO
C
      ENDROW = MAX_LENGTH+6
      BORDER = BORDER(1:ENDROW)
      DASHES = '*'//DASHES(1:ENDROW-2)//'*'
      PRINT *, BORDER
      PRINT *, '*  '//COMMANDS(1)(1:MAX_LENGTH+2)//'*'
      PRINT *, DASHES 
      DO I=2,NUM
        PRINT *, '*  '//COMMANDS(I)(1:MAX_LENGTH+2)//'*'      
      ENDDO
      PRINT *, BORDER
C      
      RETURN
C
      END      

C ***********************************************************************
C *                                                      *
C *            *****  SUBROUTINE GAUSS_LOAD *****                  *
C *                                                      *
C *            Loads array gauss(num_elements) with a gaussian dis-    *
C *      tribution between -3 and 3 (3*stan. dev.) in increments of inc. *
C *                                                      *
C *      CREATED: July, 1990  by nrd@bragg.bio.purdue.edu            *
C *      MODIFIED: Sept. 18, 1990  by nrd@bragg                        *
C ***********************************************************************
C
      SUBROUTINE GAUSS_LOAD (GAUSS, NUM_ELEMENTS)
C
      IMPLICIT NONE
C
      INTEGER*4 NUM_ELEMENTS, COUNT, HEIGHT, I, K
      REAL*4        X, INC, GAUSS(NUM_ELEMENTS)
C  --------------------------------------------------------------------
      COUNT = 1
      INC   = 21.556/FLOAT(NUM_ELEMENTS)

C     Replace real do-loop variable with integer. Original code
C     (commented out) follows

      do k = 1, nint(6.0/INC)+1
         X = -3.0 + (k-1)*INC
         HEIGHT = INT(10*EXP(-X*X))
         DO I = 0,HEIGHT
            GAUSS(COUNT) = X
            IF (COUNT.LT.NUM_ELEMENTS) COUNT = COUNT + 1
         ENDDO
      ENDDO 

c      DO X = -3.0, 3.0, INC                   
c         HEIGHT = INT(10*EXP(-X*X))
c         DO I = 0,HEIGHT
c            GAUSS(COUNT) = X
c            IF (COUNT.LT.NUM_ELEMENTS) COUNT = COUNT + 1
c         ENDDO
c      ENDDO 

       RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                    *****     FUNCTION WAVEL     *****                    *
C  *                                                                          *
C  *    COMPUTE ELECTRON WAVELENGTH IN NANOMETERS GIVEN THE VOLTAGE.          *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED 13-AUG-1999 BY T. BAKER                                 *
C  ****************************************************************************
C
      FUNCTION WAVEL (VOLTS)
C
      IMPLICIT NONE
C
      REAL*4 WAVEL, VOLTS
C  ----------------------------------------------------------------------------
C
      WAVEL = 1.23 / SQRT (VOLTS + (1.0E-6 * VOLTS**2))             ! IN NM
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *           *****     SUBROUTINE CALCULATET_EQUIVALENTS     *****          *
C  *                                                                          *
C  *    CALCULATE THE POSTIONS OF THE N_SYM EQUIVALENT POINTS.                *
C  *                                                                          *
C  *    ALL POSITIONS ARE STORED IN A FORMAT IN WHICH THE CENTER HAS BEEN     *
C  *     BEEN REMOVED SO THEY ARE POSITIONS AWAY FROM THE ORIGIN.             *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 CALCULATE_EQUIVALENTS SUBROUTINE.*
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
      SUBROUTINE CALCULATE_EQUIVALENTS (ROT, N_SYM, OVERLAPS,
     1 VALUE, VEC1, VEC2)
C
      IMPLICIT NONE
C
      REAL*4    ROT(3,3,60), VALUE(60), VEC1(3,60), VEC2(3)
      INTEGER*4 N_SYM, I_SYM
      LOGICAL*1 OVERLAPS(60)
C  ----------------------------------------------------------------------------
C
      DO I_SYM = 1,N_SYM
        VALUE(I_SYM)    = 0.0
        OVERLAPS(I_SYM) = .FALSE.
        CALL VECMUL (VEC1(1,I_SYM), ROT(1,1,I_SYM), VEC2)
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE REMOVE_OVERLAPS     *****             *
C  *                                                                          *
C  *    SET THE OVERLAP KEY ON EACH EQUIVALENT POSITION SO THAT ONLY ONE      *
C  *     MEMBER OF A DEGENERATE SET OF POSITIONS IS TAKEN.                    *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 REMOVE_OVERLAPS SUBROUTINE.      *
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
      SUBROUTINE REMOVE_OVERLAPS (VEC1, N_SYM, OVERLAPS)
C
      IMPLICIT NONE
C
      INTEGER*4 N_SYM, I, J
      REAL*4    VEC1(3,60)
      LOGICAL*1 OVERLAPS(60)
C  ----------------------------------------------------------------------------
C
      DO I = 1,N_SYM-1
        DO J = I+1,N_SYM
          IF( ((VEC1(1,I) - VEC1(1,J))**2 +
     1        (VEC1(2,I) - VEC1(2,J))**2 +
     2        (VEC1(3,I) - VEC1(3,J))**2 )
     3        .LT. 1.0) THEN
            OVERLAPS(J) = .TRUE.
          ENDIF
        ENDDO
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *           *****     SUBROUTINE GET_EQUIVALENT_VALUES     *****           *
C  *                                                                          *
C  *    GET THE VALUES OF THE EQUIVALENT POSITIONS.                           *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 GET_EQUIVALENT_VALUES SUBROUTINE.*
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
      SUBROUTINE GET_EQUIVALENT_VALUES (AVG, MAP_OUT, NCOL2, NROW2, 
     1 ICOL, IROW, N_SYM, MAP_IN, NCOL, NROW, NSEC, CENX, CENY, 
     2 CENZ, OVERLAPS, VALUE, VEC1)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, NCOL2, NROW2
      INTEGER*4 I_SYM, N_SYM, NSUM
      INTEGER*4 ICOL, IROW
      REAL*4    MAP_IN(NCOL,NROW,NSEC), CENX, CENY, CENZ
      REAL*4    MAP_OUT(NCOL2,NROW2), SUM, AVG, VALUE(60), VEC1(3,60)
      LOGICAL*1 OVERLAPS(60)
C  ----------------------------------------------------------------------------
C
      NSUM  = 0
      SUM   = 0.0
      DO I_SYM = 1, N_SYM
          IF(.NOT.OVERLAPS(I_SYM)) THEN
            CALL INTERP3D (VALUE(I_SYM), VEC1(1,I_SYM), VEC1(2,I_SYM), 
     1    VEC1(3,I_SYM), MAP_IN, NCOL, NROW, NSEC, CENX, CENY, CENZ)
          NSUM = NSUM + 1
          SUM  = VALUE(I_SYM) + SUM
          ENDIF
      ENDDO
      IF(NSUM.LT.1) THEN
         AVG   = 0.0
       ELSE
         AVG = SUM/float(NSUM)
      ENDIF
      MAP_OUT(ICOL,IROW) = AVG
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE FILL_PERP2     *****               *
C  *                                                                          *
C  *    FILL THE SYMMETRY RELATED POINT ICOL,IROW ASSUMING A TWOFOLD PERPEN-  *
C  *     DICULAR TO THE SECTION THROUGH THE CENTRAL POSITION.                 *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 FILL_PERP2 SUBROUTINE.           *
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
      SUBROUTINE FILL_PERP2 (MAP_OUT, NCOL2, NROW2, ICOL, IROW, 
     1 CENX2, CENY2)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL2, NROW2, ICOL, IROW, JCOL, JROW
      REAL*4    MAP_OUT(NCOL2,NROW2), CENX2, CENY2
C  ----------------------------------------------------------------------------
C
      JCOL = NINT(2.0*CENX2 - ICOL)
      JROW = NINT(2.0*CENY2 - IROW)
C
      IF(JCOL.LT.1 .OR. JROW.LT.1) THEN
         MAP_OUT(ICOL,IROW)   = 0.0
       ELSE
         MAP_OUT(ICOL,IROW)   = MAP_OUT(JCOL,JROW)
      ENDIF
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE INTERP3D     *****                 *
C  *                                                                          *
C  *    PERFORM 3D INTERPOLATION IN THE MAP.  ALSO ZERO'S VALUES IF THEY FALL *
C  *     OFF THE EDGE.                                                        *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 INTERP3D SUBROUTINE.             *
C  *                                                                          *
C  *    SEPTEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  ****************************************************************************
C
      SUBROUTINE INTERP3D (VALUE, X, Y, Z, MAP, NCOL, NROW, NSEC, 
     1 CENX, CENY, CENZ)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, IX, IX1, IY, IY1, IZ, IZ1
      REAL*4    MAP(NCOL,NROW,NSEC), CENX, CENY, CENZ, DX, DY, DZ
      REAL*4    VALUE, X, Y, Z, TMPX, TMPY, TMPZ
C  ----------------------------------------------------------------------------
C
        VALUE = 0.0
C
        TMPX   = X + CENX
      DX     = TMPX - AINT(TMPX)
      IX     = INT(TMPX)
        IF(IX.LT.1  .OR. IX.GE.NCOL) RETURN
      IX1    = IX + 1
C
        TMPY   = Y + CENY
      DY     = TMPY - AINT(TMPY)
      IY     = INT(TMPY)
        IF(IY.LT.1  .OR. IY.GE.NROW) RETURN          
      IY1    = IY + 1
C
        TMPZ   = Z + CENZ
      DZ     = TMPZ - AINT(TMPZ)
      IZ     = INT(TMPZ)
        IF(IZ.LT.1  .OR. IZ.GE.NSEC) RETURN          
      IZ1    = IZ + 1
C
      VALUE = (1.-DX)*(1.-DY)*(1.-DZ)*MAP(IX ,IY ,IZ )
     1      +    DX *(1.-DY)*(1.-DZ)*MAP(IX1,IY ,IZ )
     2      +(1.-DX)*    DY *(1.-DZ)*MAP(IX ,IY1,IZ )
     3      +    DX *    DY *(1.-DZ)*MAP(IX1,IY1,IZ )
     4      +(1.-DX)*(1.-DY)*    DZ *MAP(IX ,IY ,IZ1)
     5      +    DX *(1.-DY)*    DZ *MAP(IX1,IY ,IZ1)
     6      +(1.-DX)*    DY *    DZ *MAP(IX ,IY1,IZ1)
     7      +    DX *    DY *    DZ *MAP(IX1,IY1,IZ1)
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *              *****     SUBROUTINE GET_3F_MATRIX     *****                *
C  *                                                                          *
C  *    GENERATE MISSING 3-FOLD ICOSAHEDRAL SYMMETRY MATRIX FOR AVERAGING.    *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S GET_SYMMETRY_MATRICES SUBROUTINE.            *
C  *                                                                          *
C  *    SEPTEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  ****************************************************************************
C
      SUBROUTINE GET_3F_MATRIX (ROT, N_SYM)
C
        INTEGER*4 N_SYM, I, J
      INTEGER*4 THREE_FOLDS(20)/2,3,8,11,16,18,22,23,27,30,38,39,40,
     1                         42,48,53,56,57,58,59/
        REAL*4    ROT(3,3,60)
C  ----------------------------------------------------------------------------
C
      CALL GENROT (ROT)
      DO I = 1,3
        DO J = 1,3
          ROT(I,J,2) = ROT(I,J,THREE_FOLDS(3))
        ENDDO
      ENDDO
      DO I = 1,3
        DO J = 1,3
          ROT(I,J,3) = ROT(I,J,THREE_FOLDS(15))
        ENDDO
      ENDDO
C
      N_SYM = 3
      WRITE(6,*) ' Icosahedral three folds applied '
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE VECMUL     *****                  *
C  *                                                                          *
C  *    MULTIPLY A COLUMN VECTOR A BY A MATRIX X AND GENERATE THE PRODUCT     *
C  *     COLUMN VECTOR Y.                                                     *
C  *                                                                          *
C  *        Y = aX WHERE X,Y ARE 3 VECTORS AND a IS A 3 x 3 MATRIX            *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 VECMUL SUBROUTINE.               *
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
      SUBROUTINE VECMUL (Y, A, X)
C
      IMPLICIT NONE
C
      REAL*4    Y(3), A(3,3), X(3), X_COPY(3)
      INTEGER*4 I
C  ----------------------------------------------------------------------------
C
      DO I = 1,3
        X_COPY(I) = X(I)
      ENDDO
C
      DO I = 1,3
        Y(I) = X_COPY(1)*A(I,1) + X_COPY(2)*A(I,2) + X_COPY(3)*A(I,3)
      ENDDO
C
      RETURN
      END
