/*  File: libUtil.c 
 *
 * Generic utility routines.
 *
 * Rob Ashmore
 * Purdue University
 * October 2000
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../include/common.h"
#include "libUtil.h"


static BYTE *reserveMemory;


/*************************************************************
 * Code from Misha to determine the max amount of memory
 * available to be allocated
 *************************************************************/
/* Global variables */

unsigned long     half_gb = 512*1024000;		/* minimal memory 512 Mbytes	*/


/* Function prototypes */

void get_max_mem(unsigned long *maxmem);
void get_max_mem_(unsigned long *maxmem);

/* Find max memory available on the machine */

void get_max_mem(unsigned long *maxmem)
{
  get_max_mem_(maxmem);
}

void get_max_mem_(unsigned long *maxmem)
{
 char *p;
  *maxmem = half_gb;
    while (p = (char *)malloc(*maxmem)) 
      {
	if(p)
	  free(p);
	/*Increment size by 10Mb*/
	*maxmem += 10240000;
      }
    *maxmem -= 10240000;
}


/*************************************************************
 * memory_reserve() - Allocate some memory and hold onto it 
 * until memory_release() frees it.  This is used when we 
 * are trying to allocate as large a block of memory as we 
 * can this reserved block insures we have enough memory left 
 * for other things.
*/
void memory_reserve(int *numMbytes)
{/* MEMORY_RESERVE */

  reserveMemory = (BYTE *)malloc((*numMbytes)*1000000);

}/* MEMORY_RESERVE */


/*************************************************************
 * memory_release() - See memory_reserve()
*/
void memory_release()
{/* MEMORY_RELEASE */

  free(reserveMemory);

}/* MEMORY_RELEASE */
void memory_reserve_(int *numMbytes)
{
  memory_reserve(numMbytes);
}
void memory_release_()
{
  memory_release();
}





/*********************************************************
 * get_str() - Get a string of ascii characters.  Allocate
 * any storage necessary here locally.  Trash any leading
 * white space found.  Also trash any trailing WS.
*/
void get_str(FILE *flptr, char brkchr, char *string)
{/* DBGET_STR */
  char ans;
  char *s1;
  
  ans= ' ';
  while(!feof(flptr) && ans==' ')
    ans= fgetc(flptr);         /* Eat the white space */
  s1= string;
  while(!feof(flptr) && (ans!='\n' && ans!=brkchr))
    {/* Get the string */
      *(s1++)= ans;
      ans= fgetc(flptr);
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

}/* GET_STR */



/************************************************************
 * stripWhite() - Strip off the whitespace from given string 
 */
void stripWhite(char *string)
{/* STRIPWHITE */
  int i,len;
  char *s1,*s2;
  
  s1 = string;
  while(*s1==' ' || *s1=='\t')
    s1++;

  len = strlen(s1); /* it's smaller now maybe */
  for(i=0;i<len;i++)
    string[i] = s1[i];

  string[len]= '\0';                   /* Terminate the string */

}/* STRIPWHITE */







/************************************************************
 * stripWhiteCR() - Strip off the whitespace from given string 
 */
void stripWhiteCR(char *string)
{/* STRIPWHITECR */
  int i,len;
  char *s1,*s2;
  
  s1 = string;
  while(*s1==' ' || *s1=='\t' || *s1=='\n')
    s1++;

  len = strlen(s1); /* it's smaller now maybe */
  for(i=0;i<len;i++)
    string[i] = s1[i];

  string[len]= '\0';                   /* Terminate the string */

}/* STRIPWHITECR */




/* *************************************************************
 * cvupper() - Convert input string to upper case.
*/
char *cvupper(instr)
  char *instr;
{/* CVUPPER */
  register char *p= instr;
  register char ch;
  register int l2uConst= ('A'-'a');

  while(ch= *p)
    {/* Convert to upper case */
      if(ch>='a' && ch<='z')
        *p= (ch+l2uConst);
      p++;
    }/* Convert to upper case */

  return(instr);
}/* CVUPPER */





/* *************************************************************
 * rmequal() - Remove extra equal signs.
*/
char *rmequal(instr)
  char *instr;
{/* RMEQUAL */
  register char *p= instr;
  register char ch;
  register int l2uConst= ('A'-'a');
  int alreadyFound;

  alreadyFound = FALSE;
  while(ch= *p)
    {/* Convert to upper case */
      if(ch=='=')
	{/* FOund */
	  if(alreadyFound)
	    *p= ' '; /* Replace with white */
	  alreadyFound = TRUE;
	}/* FOund */
      p++;
    }/* Convert to upper case */

  return(instr);
}/* RMEQUAL */





/* ********************************************************
 * scanStatic() - Get a string of ascii characters.  Return
 * static string kept here locally.  DO NOT USE FREE_STR()
 * ON STRINGS RETURNED FROM THIS FUNCTION!  Advance the pointer
 * in case subsequent calls will be made.
*/
char *scanStatic(char **instr, char brkchr)
{/* SCANSTATIC */
  char ans;
  char *s1;
  static char string[2048];
  int i, foundOne= FALSE;
  
  ans= ' ';
  while((**instr!='\0')&&
        ((**instr)==' ' || (**instr)=='\t' || (**instr)=='\n')  )
    (*instr)++;                          /* Eat the white space */
  s1= string;
  ans = *((*instr)++);
  while((ans != '\0' && ans!='\n' && ans!=brkchr))
    {/* Get the string */
      *(s1++)= ans;
      ans= *((*instr)++);
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

  return(string);              /* Remember we're static here ! */

}/* SCANSTATIC */




/* **********************************************************
 * substStatic() - Get the sub-string from start for length
 * characters.  Return a static string.  DO NOT TRY TO FREE
 * THIS MEMORY WITH FREE_STR()!
*/
static char *substStatic(instr, start, length)
  char *instr;
  int start, length;
{/* SUBSTSTATIC */
  static char tmpString[1024];
  char *p;
  int i,inLngth;

  tmpString[0]= '\0';         /* Start out terminated */
  inLngth= strlen(instr);
  start= MIN(inLngth,start);  /* Clip as necessary */
  length= MIN((inLngth-start),length);
  length= MAX(length,0);
  p= (instr + start);
  for (i=0;i<length;i++)
    {/* GET the sub-string */
      tmpString[i]= *(p++);
    }/* GET the sub-string */
  tmpString[length]= '\0';    /* End up terminated */

  return(tmpString);
}/* SUBSTSTATIC */




/*********************************************************/
void add_map_padded_(float *map1,
		     int *ncol1, int *nrow1, int *nsec1,
		     float *map2,
		     int *ncol2, int *nrow2, int *nsec2)
{/* ADD_MAP_PADDED_ */
  add_map_padded_(map1, ncol1, nrow1, nsec1,
		  map2, ncol2, nrow2, nsec2);
}/* ADD_MAP_PADDED_ */


void add_map_padded(float *map1,
		    int *ncol1, int *nrow1, int *nsec1,
		    float *map2,
		    int *ncol2, int *nrow2, int *nsec2)
{/* ADD_MAP_PADDED */
  int i,j,k,rowOut1,secOut1,colOff,rowOff,secOff;
  int nc1,nc2,nr1,nr2,ns1,ns2;
  int rowOut2,secOut2;
  
  nc2 = (*ncol2);
  nr2 = (*nrow2);
  ns2 = (*nsec2);
  nc1 = (*ncol1);
  nr1 = (*nrow1);
  ns1 = (*nsec1);
  colOff = (int)((nc2 - nc1)/2);
  rowOff = (int)((nr2 - nr1)/2);
  secOff = (int)((ns2 - ns1)/2);
  
  /* Init */
  for(i=0;i<ns2;i++)
    {/* secs */
      secOut2 = (i * (nc2*nr2));
      for(j=0;j<nr2;j++)
	{/* row */
	  rowOut2 = secOut2  + (j * nc2);
	  for(k=0;k<nc2;k++)
	    {/* col */
	      map2[rowOut2+k]= 0.0;
	    }/* col */
	}/* row */
    }/* secs */

  /* Now transfer it */
  for(i=0;i<ns1;i++)
    {/* secs */
      secOut2= (secOff+i) * (nc2*nr2);
      secOut1= (i) * (nc1*nr1);
      for(j=0;j<nr1;j++)
	{/* row */
	  rowOut2= secOut2 + ((rowOff+j) * (nc2));
	  rowOut1= secOut1 + (j * (nc1));
	  for(k=0;k<nc1;k++)
	    {/* col */
	      map2[rowOut2+colOff+k]= map1[rowOut1+k];
	    }/* col */
	}/* row */
    }/* secs */

}/* ADD_MAP_PADDED */


