/* emUnix.c
 * 
 * Holding area for code migrating from FORTRAN/VMS to UNIX/C.
 * 
 * Rob Ashmore
 * Purdue Univ.
 * rwa@bragg.bio.purdue.edu
*/

#define IS_ROBEM_MODULE 1

#include "emUnix.h"



#define THRESHHOLD 1.0
#define SLIDE 0.4

static float threshold = THRESHHOLD;
static float slide = SLIDE;

int callXdy(double x, double y, double z, double minRad, double maxRad)
{
  return(0);
}


//**************************************************
// Used for icosahedron drawing 
double disDotPlane(double x,double y,double z,
		double x1,double y1,double z1,
		double x2,double y2,double z2,
		double x3,double y3,double z3)
{

	double A,B,C,D,sum;
	double d;		
	A = (y2-y1)*(z3-z1)+(y3-y1)*(z1-z2);	
	B = (x3-x1)*(z2-z1)+(x2-x1)*(z1-z3);	
	C = (x2-x1)*(y3-y1)+(x3-x1)*(y1-y2);	
	D = 	 x1*(y1-y2)*(z3-z1)
		+z1*(x1-x2)*(y3-y1)
		+y1*(x1-x3)*(z2-z1)
		+x1*(y3-y1)*(z2-z1)
		+y1*(x2-x1)*(z3-z1)
		+z1*(x3-x1)*(y2-y1);
	sum = A*x+B*y+C*z+D;
	if (sum < 0.0)
		sum = -sum;
	d = sum/(sqrt(A*A+B*B+C*C));	
	return d;
}


//**************************************************
// Used for icosahedron drawing 
int isOnPlane(double x,double y,double z,
		double x1,double y1,double z1,
		double x2,double y2,double z2,
		double x3,double y3,double z3)
{
	double dst = disDotPlane(x,y,z,x1,y1,z1,x2,y2,z2,x3,y3,z3);
	if (dst < threshold)
	return 1;
	else
	return 0;
	
}


//**************************************************
// Used for icosahedron drawing 
void crossPoint (double xa,double ya,double za,
		double xb,double yb,double zb,
		double x1,double y1,double z1,
		double x2,double y2,double z2,
		double x3,double y3,double z3, 
		double *Xx, double *Xy, double *Xz)
{
	//to obtain the cross point of a line and a plane
	//plane function: AX+BY+CZ+D = 0
	//line function: x/x2=y/y2=z/z2 = t; because x1=y1=z1 =0;
	double A, B, C, D;
	double t;
	
	A = (y2-y1)*(z3-z1) - (y3-y1)*(z2-z1);
	B = (x3-x1)*(z2-z1) - (x2-x1)*(z3-z1);
	C = (x2-x1)*(y3-y1) - (x3-x1)*(y2-y1);
	D = -x1*(y2-y1)*(z3-z1) - z1*(x2-x1)*(y3-y1) - 
	  y1*(x3-x1)*(z2-z1) + x1*(y3-y1)*(z2-z1) + 
	  y1*(x2-x1)*(z3-z1) + z1*(x3-x1)*(y2-y1);
	
	t = (-D - A*xa- B*ya- C*za)/(A*(xb-xa) + B*(yb-ya) + C*(zb-za));
	*Xx = (xb-xa)*t + xa;
	*Xy = (yb-ya)*t + ya;
	*Xz = (zb-za)*t + za;
	
}		

//**************************************************
// Used for icosahedron drawing 
int isOutPlane(	double x,double y,double z,
		double x1,double y1,double z1,
		double x2,double y2,double z2, int PointHigherOut)
{
	//this function will check if this point is outside the plane 
	//will check 3 planes which form the central cone in 1/8 space
	//consider the line parallel to z axis, will check the relationship
	//between the original point and the cross point
	//the plane is decided by 3 points x1y1z1, x2y2z2, 000
	//if POintHigherOut is 1, this means if the original point 
        //is higher than cross point then 
	//original point is cosidered as outside the plane
	double Xx, Xy, Xz;
	crossPoint(x,y,0,x,y,z,0,0,0,x1,y1,z1,x2,y2,z2, &Xx, &Xy, &Xz);
	if(PointHigherOut && z > Xz)
		return 1;
	else if(!PointHigherOut && z < Xz)
		return 1;
	else
		return 0;
		
	
}	
	

//**************************************************
// Used for icosahedron drawing 	
int isOnBulge(double sphererad, double x,double y,double z,
		double x1,double y1,double z1,
		double x2,double y2,double z2,
		double x3,double y3,double z3)
{
	double a, b, c;
	double dstcenter;
	double dstplane;
	double dstsphere;
	double dstXcenter;
	double dstIcosPoint;
	double dstXsphere;
	double diff;
	
	crossPoint(x,y,z,0,0,0, x1,y1,z1,x2,y2,z2,x3,y3,z3, &a, &b, &c);
	dstcenter = sqrt(x*x + y*y + z*z);
	//distance between the cross point(of icosahedral and the line)
	//and the center
	dstXcenter = sqrt(a*a + b*b + c*c);	
	//distance between sphere to the point
	dstsphere = sphererad - dstcenter;	
	//distance between X point and the sphere;
	dstXsphere = sphererad - dstXcenter;	
	
	if(dstXcenter > dstcenter) 
		return 0;
	//if(dstXsphere < threshold)
	//	return 1;
	//the distance between the point and the cross point
	dstIcosPoint = dstcenter - dstXcenter; 	
	
	diff = dstXsphere * slide - dstIcosPoint;
	if( diff < threshold && diff > -threshold)
		return 1;
	return 0;
}


/*************************************************************/
//cut out an icosahedral face
int cutIcosFace(double x, double y, double z, double rad) 
{
     
	double x2,y2,z2;	
	double x4,y4,z4;
	double x6,y6,z6;
	double rad85;
	double rad52;
	double distance;
	distance = sqrt(x*x + y*y + z*z);
	//x,y,z is the cartessian position of the considered voxel
	//rad is the radius from the origin to the five fold vertex
	
	//first we need to convert this voxel to the first 1/8 space	
	if (x < 0 || y < 0 || z < 0)	
		return 0;
		
	distance = sqrt(x*x + y*y + z*z);
	if( distance > rad)
		return 0;	
	
	
	//these are the cartessian values of six points enclosing 
	//the icosahedral shell
	rad85 = 0.85065*rad;
	rad52 = 0.52573*rad;
		
	x2 = 0.0;y2 = rad85;z2 = rad52;//5 fold point on yz plane		
	x4 = rad85;y4 = rad52;z4 = 0.0;//5-fold point which is on xy plane	
	x6 = rad52;y6 = 0.0;z6 = rad85;//5-fold point on xz plane
	if(isOutPlane(x,y,z,x2,y2,z2,x6,y6,z6, 0) && 
	   isOutPlane(x,y,z,x2,y2,z2,x4,y4,z4, 1) && 
	   isOutPlane(x,y,z,x6,y6,z6,x4,y4,z4, 1))
	{
		return 1;
	
	}
	
	return 0;

}

//************************************************************
//cut out an assymetric unit
int cutAsymUnit(double x, double y, double z, double rad)	
{
	double x4, y4, z4;
	double x5, y5, z5;
	double x6, y6, z6;
	double x46m, y46m, z46m;	//the middle point of the point 6 
	                                //and 2-fold correspoinding point 
	                                //of 4 around 5
	double rad85;
	double rad52;
	double distance;
	if (x < 0 || y < 0 || z < 0)	
		return 0;
	distance = sqrt(x*x + y*y + z*z);
	if( distance > rad )
		return 0;  //the voxel is too close to center, 
	                   //speed up the process
	
	rad85 = 0.85065*rad;
	rad52 = 0.52573*rad;
	x4 = rad85;y4 = rad52;z4 = 0.0;//5-fold point which is on xy plane	
	x5 = rad85;y5 = 0.0;z5 = 0.0;//midege point on x axis	
	x6 = rad52;y6 = 0.0;z6 = rad85;//5-fold point on xz plane
	x46m = (x6+x4)/2.0; y46m = (y6-y4)/2.0; z46m=(z6+z4)/2.0;
	 
	if(isOutPlane(x,y,z,x4,y4,z4,x6,y6,z6, 0) && 
	   isOutPlane(x,y,z,x4,y4,z4,x46m,y46m,z46m, 1))
	{
		return 1;
	
	}
	return 0;
	
}



//**************************************************
// Used for icosahedron drawing 
int bulge(double x, double y, double z, double rad)
{

	
	double x1,y1,z1;
	double x2,y2,z2;
	double x3,y3,z3;
	double x4,y4,z4;
	double x5,y5,z5;
	double x6,y6,z6;
	double rad85;
	double rad52;
	double distance;
	distance = sqrt(x*x + y*y + z*z);
	//x,y,z is the cartessian position of the considered voxel
	//rad is the radius from the origin to the five fold vertex
	
	//first we need to convert this voxel to the first 1/8 space	
	
	
	if (x < 0)	
		x = -x;
	if (y < 0)	
		y = -y;
	if (z < 0)	
		z = -z;
		
		
	//we need to get rid of the points whose position is out of range
	if( distance > (rad+threshold) || 
	    distance < (0.79465 * rad - threshold)
	)
		return 0;	//the voxel is too close to center, 
	                        //speed up the process
	
	//these are the cartessian values of six points enclosing 
	//the icosahedral shell
	rad85 = 0.85065*rad;
	rad52 = 0.52573*rad;
	x1 = 0.0;y1 = 0.0;z1 = rad85;//midedge point on Z axis		
	x2 = 0.0;y2 = rad85;z2 = rad52;//5 fold point on yz plane	
	x3 = 0.0;y3 = rad85;z3 = 0.0;//midege point on y axis	
	x4 = rad85;y4 = rad52;z4 = 0.0;//5-fold point which is on xy plane
	x5 = rad85;y5 = 0.0;z5 = 0.0;//midege point on x axis	
	x6 = rad52;y6 = 0.0;z6 = rad85;//5-fold point on xz plane
		
	if(isOutPlane(x,y,z,x2,y2,z2,x6,y6,z6, 1))
	{
		if(isOnBulge(rad,x,y,z,x1,y1,z1,x2,y2,z2,x6,y6,z6))
			return 1;
		else
			return 0;
	}
	
	else if(isOutPlane(x,y,z,x4,y4,z4,x6,y6,z6, 0))
	{
		
		if(isOnBulge(rad,x,y,z,x4,y4,z4,x5,y5,z5,x6,y6,z6))
			return 1;
		else
			return 0;
	}
	else if (isOutPlane(x,y,z,x4,y4,z4,x2,y2,z2, 0))
	{
		if(isOnBulge(rad,x,y,z,x2,y2,z2,x4,y4,z4,x3,y3,z3))
			return 1;
		else
			return 0;
	}	
	else if(isOnBulge(rad,x,y,z,x2,y2,z2,x6,y6,z6,x4,y4,z4))		
	{
		return 1;
	}
	else
		return 0;	
}


//**************************************************
// Used for icosahedron drawing 
int doIcosshell(double x, double y, double z, double rad)
{

	double ka, kb, kc,kd;
	
	double x1,y1,z1;
	double x2,y2,z2;
	double x3,y3,z3;
	double x4,y4,z4;
	double x5,y5,z5;
	double x6,y6,z6;

	if (x < 0)	
	x = -x;
	if (y < 0)	
	y = -y;
	if (z < 0)	
	z = -z;
	//we need to get rid of the points whose position is out of range
	if(x > (rad +threshold) || y > (rad +threshold) || 
	   z > (rad +threshold))
	
	{
		return 0;
	}
	if(sqrt(x*x + y*y + z*z) < (0.79465 * rad - threshold))	
	{
		return 0;	//the voxel is too close to center, 
		                //speed up the process
	}
	
	//these are the cartessian values of six points enclosing 
	//the icosahedral shell
	x1 = 0.0;
	y1 = 0.0;
	z1 = 0.85065*rad;//center to midedge
	
	x2 = 0.0;
	y2 = 0.85065*rad;
	z2 = 0.52573*rad;//half of the edge
	
	x3 = 0.0;
	y3 = 0.85065*rad;
	z3 = 0.0;
	
	x4 = 0.85065*rad;//center to midege
	y4 = 0.52573*rad;
	z4 = 0.0;
	
	x5 = 0.85065*rad;
	y5 = 0.0;
	z5 = 0.0;
	
	x6 = 0.52573*rad;
	y6 = 0.0;
	z6 = 0.85065*rad;
	
		
	//x,y,z is the cartessian position of the considered voxel
	//rad is the radius from the origin to the five fold vertex
	
	//first we need to convert this voxel to the first 1/8 space
	
	
	//we need to consider the projection of the voxel on xy plane
	//so we can decide which icosahedral plane is more close to the 
	//the voxel.
	
	
	ka = -1.61804;
	kb = 1.61804;
	kc = -0.38211;
	kd = 0.61803;
	if((x <= 0.525729*rad)
		&&(y <= (ka * x + 0.85084*rad)) 
		&& (isOnPlane(x,y,z,x1,y1,z1,x2,y2,z2,x6,y6,z6) == 1))
	{
		return 1;
	}
	
	else if((x >= 0.525729*rad 
		&& x <= 0.85065*rad)
		&&(y <= (kb*x-0.85065*rad)) 
		&& (isOnPlane(x,y,z,x4,y4,z4,x5,y5,z5,x6,y6,z6) == 1))
	{
		return 1;
	}
	else if ((	(x <= 0.85065*rad)
		&&(y <= 0.85065*rad)
		&&(z <= 0.85065*rad)
		&&(y >= (ka * x + 0.85084*rad))
		&&(y >= (kb*x - 0.85065*rad))
		&&(y <= (kc * x + 0.85065*rad)) 
		&& (isOnPlane(x,y,z,x2,y2,z2,x4,y4,z4,x6,y6,z6) == 1)))
	{
		return 1;
	}
	
	
	else if( z <= (-kd*x + 0.52573*rad)
		&& (isOnPlane(x,y,z,x2,y2,z2,x3,y3,z3,x4,y4,z4) == 1))		
	{
		//printf("\n your projection is in area 4!");
		return 1;
	}
	
	//else
	return 0;
	
	
	
}


//***************************************************************
// Display the appropriate icosahedron
int icosshell(double x, double y, double z, double rad, int tag)
{
  if(tag==CUTFACE)
    cutIcosFace(x,y,z,rad);
  else if(tag==CUTASYM)
    cutAsymUnit(x,y,z,rad);
  else if(tag==BULGE)
    bulge(x,y,z,rad);
  else if(tag==ICOSNORMAL)
    doIcosshell(x,y,z,rad);
}


/***********************************************************************
 * checkMemory() - Check to make sure malloc did indeed give us the 
 * memory we wanted.
*/
int checkMemory(void *status, int line, char *file)
{/* CHECKMEMORY */
  int rtnVal;
  char outStr[512];

  if(status==NULL)
    {/* error */
      rtnVal= 0;
      printf("Error allocating memory [%d][%s]\n",line,file);
      sprintf(outStr,"Error allocating memory [%d][%s]\n\
You may want to restart the program\n",line,file);
      askThem(outStr,FALSE,XmDIALOG_INFORMATION);
    }/* error */
  else rtnVal = 1;

  return(rtnVal);
  
}/* CHECKMEMORY */



/***********************************************************************
 * deg2rad() - convert Degrees to radians.
*/
double deg2rad(double degrees)
{/* DEG2RAD */
  double rtnVal;

  rtnVal = ((double)((double)2.0 * (double)PI) * 
	    (double)((double)degrees / (double)360.0));
  return(rtnVal);

}/* DEG2RAD */




/***********************************************************************
 * rad2deg() - convert Radians to degrees.
*/
double rad2deg(double rads)
{/* RAD2DEG */
  double rtnVal;

  rtnVal = ((rads / THISTWOPI) * 360.0);
  return(rtnVal);

}/* RAD2DEG */




/***********************************************************************
 * fsign() - Float transfer of sign function to mimic the FORTRAN SIGN
 * intrinsic function.  Transfer the sign of the second arg to the first.
*/
float fsign(float arg1, float arg2)
{/* FSIGN */
  int oneNeg,twoNeg;

  oneNeg=twoNeg= FALSE;
  if(arg1<0.0)
    oneNeg= TRUE;
  if(arg2<0.0)
    twoNeg= TRUE;

  if(oneNeg == twoNeg) 
    return(arg1);           /* Already the same */
  else
    return((0.0 - arg1));   /* Flip the sign */

}/* FSIGN */





/******************************************************************************
C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE EMMAP_ROB_DPHSHD     *****            *
C  *                                                                          *
C  *    THIS IS ADAPTED FROM THE PROGRAM SURFACE WHICH PRODUCES DEPTH-CUE     *
C  *     MAPS AND A SHADED SURFACE REPRESENTATION OF A THREE-DIMENSIONAL MAP. *
C  *                                                                          *
C  *    SUB-MENU TOUCH-KEY OPTIONS:                                           *
C  *                                                                          *
C  *        A   COMPUTE DEPTH-CUE MAP FROM 3D MAP                             *
C  *        B   COMPUTE SHADED MAP FROM DEPTH-CUE DATA                        *
C  *        D   DISPLAY DEPTH-CUE OR SHADED DATA ON GRAPHICS                  *
C  *        E   EXIT SUBROUTINE                                               *
C  *        W   WRITE DEPTH-CUE OR SHADED DATA TO "MAP" DISK FILE             *
C  *                                                                          *
C  *    UNITS: 3 = TEMPORARY DIRECT ACCESS SCRATCH FILE OF DEPTH-CUE IMAGES.  *
C  *                                                                          *
C  *    MAY 1988 PURDUE UNIVERSITY VAX 8550, T. BAKER.                        *
C  *    MODIFIED  9-MAY-1991 BY T. BAKER.                                     *
C  *	Modified 20-JAN-1994 by D. Belnap				      *
C  ****************************************************************************
C
C	SUBROUTINE EMMAP_ROB_DPHSHD (LUN,MAP,NCOL,NROW,NSEC,TMP,DPHSHD_MAP)
*
* Ported to C May 1996  RWA
*
* Main loop of the emmap cueing/shading routines.
*/
void emmap_rob_dphshd_C (float *map,
			 float *final_map,
			 float *radial_map,
			 float *shad_map,
			 float *origMap,
			 int   useForMinMax,
			 float surfaceDensity,
			 int ncol, int nrow, 
			 int firstSec, int nsec,
			 int col1, int col2,
			 int row1, int row2,
			 char ikey, 
			 double thetaX, double thetaY,
			 double minRadius, double maxRadius,
			 double xCen, double yCen, double zCen,
			 float radialXcen, float radialYcen, float radialZcen,
			 int lowRes, int topRadialSec)
{/* emmap_rob_dphshd */
  int   num_views;
  int   sec1, sec2;
  double     theta;
  float      surf_dens;
  BYTE     surf;
  BYTE     shift;

  /* Initialize */
  i2_min= -32767;
  i2_max=  32767;
  i4_min=  -2147483647;
  i4_max=   2147483647;

  /* Some data inits */
  map_abang = 90.0;
  alp = 90.0;
  bet = 90.0;
  gam = 90.0;
  aoverb = 1.0;
  dela = 1.0;
  delb = 1.0;
  delc = 1.0;
  

  /*
    C  ***************************************
    C  *    INITIALIZE PARAMETER DEFAULTS    *
    C  ***************************************
    */
  sec1           = firstSec;
  sec2           = nsec;

  surf_dens      = surfaceDensity;

  /* Why do we set this so low???*/
  num_views    = 1;
  
  theta = globTheta;     /* Passed from XWindows */
  
  if(surf_dens < 0)
    surf_dens = 0.0 - surf_dens;
  
  surf    = FALSE;
  shift   = FALSE;
  
  /* we select 'd' for density from the dph_options() */
  /* this causes surf=TRUE to happen */
  surf = TRUE;
  
  /* DO depth cue ON_THE_FLY ... */
  if(radialMapShade)
    dph_nomem_tilt_fast (map, origMap, ncol, nrow, 
			 radial_map, 
			 shift, surf, sec1, sec2, 
			 col1, col2,
			 row1, row2,
			 num_views, 
			 irev, surf_dens, useForMinMax, shad_map,
			 thetaX, thetaY, minRadius, maxRadius,
			 xCen, yCen, zCen,
			 radialXcen, radialYcen, radialZcen,
			 lowRes, topRadialSec);
  else 
    dph_nomem_tilt_fast (map, origMap, ncol, nrow, 
			 final_map, 
			 shift, surf, sec1, sec2, 
			 col1, col2,
			 row1, row2,
			 num_views, 
			 irev, surf_dens, useForMinMax, shad_map,
			 thetaX, thetaY, minRadius, maxRadius,
			 xCen, yCen, zCen,
			 radialXcen, radialYcen, radialZcen,
			 lowRes, topRadialSec);
  

}/* emmap_rob_dphshd */



/*****************************************************************************
C  ***************************************************************************
C  *                                                                         *
C  *                *****     SUBROUTINE EMMAP_SHD     *****                *
C  *                                                                         *
C  *    ADAPTED FROM MRREFL.FOR BY M. RADERMACHER OF ALBANY - STRAIGHT ON    *
C  *      ILLUMINATION                                                       *
C  *                                                                         *
C  *    UNIT: 3 = TEMPORARY DIRECT ACCESS SCRATCH FILE OF DEPTH-CUE MAP      *
C  *              (OPEN IF DPH_SCRATCH = .TRUE.)                             *
C  *                                                                         *
C  *    IN EMMAP_SHD1, THE DEPTH_CUE MAP ALREADY EXISTS IN CORE MEMORY       *
C  *     (DPH_SCRATCH = .FALSE.) IN THE ARRAY "DPHSHD_MAP", OR IN A SCRATCH  *
C  *     FILE (DPH_SCRATCH = .TRUE.).                                        *
C  *                                                                         *
C  *    PROGRAMMERS AND USERS PLEASE NOTE:  THE ALGORITHM FOR SHADING AT     *
C  *     ANGLES AWAY FROM THE VIEW DIRECTION IS NOT STRICTLY CORRECT IN THE  *
C  *     SENSE THAT NO ACCOUNT IS MADE FOR ELIMINATING LIGHT FROM FALLING    *
C  *     IN AREAS THAT WOULD OTHERWISE BE SHADED DUE TO OBSTRUCTIONS TO THE  *
C  *     PATH OF THE LIGHT SOURCE                                            *
C  *                                                                         *
C  *    LAST MODIFIED 22-APR-1991 BY T. BAKER.                               *
C  ***************************************************************************
C
* Ported to C May 1996 RWA
* 
* Routine to shade depth/radial cued views.
*/
void emmap_rob_shd1 (cue_t *cuedMap,
		     double *tmp2d, double *tmp,
		     int ncol, int nrow,
		     int sec1, int sec2, 
		     float fract, 
		     int useForMinMax, 
		     float glbAngx, float stepVal,
		     float glbAngy, int whichStep,
		     float passMin, int angx, int angy, int thisFract)
{/* EMMAP_ROB_SHD1 */
  int       i, ii, j, jj, k;
  double    scaleIt, old_min;
  double    depth, depth1, depth2, depth3, shad, xx, yy, cs;
  double    thisMin,thisMax;
  float     *thisMap,*radMap,*shadMap;
  int       shadStep;
  double    sinx, cosx, siny, cosy;

  map_min  = passMin;
  old_min  = (double)map_min;
  scaleIt    = (double)(map_max - map_min);   /*! to convert to 0-255 scale */

  /*
    c
    c  *******************************************
    c  *    determine the shading for the map    *
    c  *******************************************
    */
  thisMin = r4_max;
  thisMax = r4_min;
  thisMin = 99999999.9;
  thisMax = -99999999.9;
  thisMin = HUGE;   /* dgm HUGE instead of 99999999.9 */
  thisMax = -HUGE;  /* dgm -HUGE instead of -99999999.9 */

  /* Do all sections */
  for(k = (sec1-1);k<sec2;k++)
    {/* For k */

      if(whichStep==angx)
	glbAngx += stepVal;
      else if(whichStep==angy)
	glbAngy -= stepVal;
      else if(whichStep==thisFract)
	fract += stepVal;

      sinx           = sin(deg2rad((double)glbAngx));
      cosx           = cos(deg2rad((double)glbAngx));
      siny           = sin(deg2rad((double)glbAngy));
      cosy           = cos(deg2rad((double)glbAngy));

      thisMap = cuedMap[k].depth;
      radMap  = cuedMap[k].radial;
      shadMap  = cuedMap[k].shad;

      for(j = 0;j<nrow;j++)
	for(i = 0;i<ncol;i++)
	  tmp2d[(i+(j * ncol))] = (double)thisMap[(i+(j * ncol))];

      for(j = 0;j<(nrow);j++)
	{/* For j */
#ifdef ORIG_WAY_WORKS
	  if(j == nrow)
	    jj = 1;	     /*duplicate first value for wrap around*/
	  else
	    jj = j + 1;
#else
	  if(j == (nrow-1))
	    jj = 0;	     /*duplicate first value for wrap around*/
	  else
	    jj = j + 1;
#endif
	  
	  for(i = 0;i<(ncol);i++)
	    {/* For i */
	      tmp[i] = 0.0;		/* reinitialize tmp array*/
	      tmp[i+1] = 0.0;		/* reinitialize tmp array*/
	      if(i == (ncol-1))
	        ii = 0;		    /* duplicate first value for wrap around*/
	      else
	        ii = i + 1;

	      depth1 = tmp2d[(i+(j * ncol))]  - old_min;
	      depth2 = tmp2d[(ii+(j * ncol))] - old_min;
	      depth3 = tmp2d[(i+(jj * ncol))] - old_min;

	      if(depth1 > 0.0 && depth2 > 0.0 && depth3 > 0.0) 
		{/* then */

		  /* OK here's where our "striping" problem happens when
		   * we try and rotate maps.  Suppose we are trying to 
		   * rotate a MAP 35 degrees in Y and we have some surfaces
		   * that have been "cut".  These cut surfaces are depth
		   * cued such that they have very small differences in
		   * their depth in the local area of depth1,depth2,depth3
		   * that we are looking at here.  Because small numbers
		   * cause HUGE differences in the calculation of shad
		   * below we get striping.  Here's some example points from
		   * a MAP I tried:
		   *
		   * j[145]i[0]
		   * depth1[72.000000]depth2[74.000000]depth3[72.000000]
		   * xx[-2.000000]yy[0.000000]cs[0.447214]shad[42.318746]
		   * (xx * sinx)[-0.347296] (yy * siny)[0.000000]  
		   * (cosx * cosy)[0.969846]
		   * j[145]i[1]
		   * depth1[74.000000]depth2[75.000000]depth3[74.000000]
		   * xx[-1.000000]yy[0.000000]cs[0.707107]shad[85.575559]
		   * (xx * sinx)[-0.173648] (yy * siny)[0.000000]  
		   * (cosx * cosy)[0.969846]
		   *
                   * Notice that the first point has a difference of -2 for
		   * xx and the second has a difference of -1.  These cause
		   * a big difference in the shad values of 42 and 85 
		   * respectively.
		   *
		   * RWA 8/6/97 - I'm wondering why the depths aren't
		   * showing up in fractional floating point - they are 
		   * always integer...  I'm going to check this out
		   * in dph_nomem_tilt_fast().  
		   *
		   * I'm now thinking it's hard to believe that getting any
		   * better depths out of dph_nomem_tilt_fast() will
		   * help this problem.  It's already giving us differences
		   * in depth that are very small.  I'm wondering if
		   * we need an entirely new algorithm for shading as
		   * in the four lines below.
		   */

		  xx   = (depth1-depth2);
		  yy   = (depth1-depth3); 
		  cs   = 1.0/sqrt((xx * xx) + (yy * yy) + 1.0);
		  shad = scaleIt * cs * ((xx * sinx) + (yy * siny) + 
				       (cosx * cosy));

		  if(shad < 0.0) 
		    shad = 0.0;
		  if(!radialMapShade)
		    shad  = shad*fract;  /* Fract has no meaning for radial */
		  if(radialMapShade)
		    {/* We are shading radially here */
		      int steps,foundIt;

		      depth = (double)radMap[(i+(j * ncol))];

		      foundIt = FALSE;
		      shadStep = (int)(160.0/(double)expandStep);
		      for(steps=0;steps<expandStep;steps++)
			{/* Calc amount of shade */
			  int cutOff;
			  cutOff = (int)(160 - (steps * shadStep));
			  if(!foundIt)
			    if(shad > cutOff)
			      {/* found it */
				shad = (double)((double)0.0 - (double)steps);
				foundIt = TRUE;
			      }/* found it */
			}/* Calc amount of shade */
		      if(!foundIt)
			shad = (double)((double)0.0 - (double)(expandStep-1));

		      if(depth == 0.0)
			shad = 0.0;
		      if((depth + shad)<0.0)
			shad = 0.0;

		    }/* We are shading radially here */
		  else depth = depth1 * (1.0 - fract);

		  tmp[i] = (double)(shad + depth);

		  if(thisMin > tmp[i])
		    thisMin = (double)tmp[i];
		  if(thisMax < tmp[i])
		    thisMax = (double)tmp[i];
		  
		}/* then */
	      
	    }/* For i */
	  
	  for(i = 0;i<ncol;i++)
	    shadMap[(i+(j * ncol))] = (float)tmp[i];
	  
	}/* For j */
      
      
    }/* For k */
  if(thisMax == thisMin)
    thisMin -= 0.01;

  if(useForMinMax)
    {/* Then update globals */
      map_min = (float)thisMin;
      map_max = (float)thisMax;
    }/* Then update globals */

}/* EMMAP_ROB_SHD1 */






/***********************************************************************
C  *********************************************************************
C  *                                                                   *
C  *            *****     SUBROUTINE DPH_NOMEM_TILT_FAST     *****     *
C  *                                                                   *
C  * Modified from dph_notilt_fast()
C  *********************************************************************
C
* RW AShmore July 29, 1997 - I'm trying to incorporate the concept that
* as the volume they want to look at gets smaller this routine should
* get faster.  Suppose they cut the map down so that they want to look
* at:
*               
*         col   33   to   88   (col1 and col2)
*         row   33   to   88   (row1 and row2)
*         sec   0    to   55   (sec1 and sec2)
*
* So if we just alter our for(col=0;col<ncol;col++) loops below to
* be for(col=col1,col<col2,col++) we will "clip off" the sections
* below 33 until they've roatated into the col and row constraints.
* We need to choose col1, col2, row1, row2, sec1 and sec2 intelligently
* such that when we rotate we see all we can.  I've left it opened up
* for now so that we don't miss anything (e.g. 0 to ncols etc...) but
* with a little thought we might be able to optimize this for speed.
*   Suppose we are just rotating in Y - then we don't need to worry to
* worry about the rows - they will always fall in the row1 row2 region.
* Same for X - cols will be within region.  I've added code below to
* stuff the startRow endRow startCol endCol appropriately.
*
* This rtn has some problems with striping on the display of any
* map that is cut and then rotated.   Not sure why it is happening.
*/
void dph_nomem_tilt_fast (float *coremap,
			  float *origMap,
			  int ncol, int nrow, 
			  float *dphshd_map, 
			  BYTE shift, BYTE surf, 
			  int sec1, int sec2, 
			  int col1, int col2,
			  int row1, int row2,
			  int num_views,
			  int thisIrev, float surf_dens,
			  int useForMinMax,
			  float *shad_map,
			  double thetaX, double thetaY,
			  double minRadius, double maxRadius,
			  double xCen, double yCen, double zCen,
			  float radialXcen, float radialYcen, float radialZcen,
			  int lowRes, int topRadialSec)
{/* dph_nomem_tilt_fast */
  int      col, row;
  int      startCol,endCol,startRow,endRow;
  double    thisSecn,thisRow,thisCol;
  int      secn, topSec, bottomSec;
  int      colOffset,rowOffset,secOffset;
  double    origNumSecsM1,nrowM1,ncolM1,sec2M1,row2M1,col2M1;
  double    topSecMtmp1;
  double    tmp1;
  double   tmp_fract;
/*  float    dens1, dens2, dens_diff, absDensDiff, depth; */
  double   dens1, dens2, dens_diff, absDensDiff;
  double   q, qSq, mapRadiusSq, minRadSq,maxRadSq,minRadSqT1p1,maxRadSqTp95;
  double   xSq,ySq,zSq,mapRadius;
  double   thisSecnF,thisRowF,thisColF;
  double   cthetaX,sthetaX,cthetaY,sthetaY;
  double   NUsec1,NUsec2M1;
#define COREMAPREF(x,y,z) coremap[((((int)z)*ncol*nrow)+(((int)y)*ncol)+((int)x))]
  

  origNumSecsM1= (double)(origNumSecs - 1.0);
  nrowM1       = (double)(nrow - 1.0);
  ncolM1       = (double)(ncol - 1.0);
  sec2M1 = sec2 - 1.0;
  row2M1 = row2 - 1.0;
  col2M1 = col2 - 1.0;

  minRadSq= minRadius * minRadius;
  /* Compute 10% past the lowest radius to see if we need to 
   * flip the sign or not (See below). */
  minRadSqT1p1 = minRadSq * 1.1;
  maxRadSq= maxRadius * maxRadius;
  maxRadSqTp95 = maxRadSq * 0.95;

  cthetaX = (double)cos((double)deg2rad((double)thetaX));
  sthetaX = (double)sin((double)deg2rad((double)thetaX));
  cthetaY = (double)cos((double)deg2rad((double)thetaY));
  sthetaY = (double)sin((double)deg2rad((double)thetaY));

  /* How much do we have to offset for asymmetric maps */
  colOffset = (largestDim - ncol) / 2;
  rowOffset = (largestDim - nrow) / 2;
  secOffset = (largestDim - origNumSecs) / 2;
  xCen += colOffset;
  yCen += rowOffset;

  radialXcen += colOffset;
  radialYcen += rowOffset;

  /* Now try and optimize for speed */
  startCol= 0;
  endCol  = largestDim;
  startRow= 0;
  endRow  = largestDim;

  if(thetaX==0.0)
    {/* We don't have to worry about rows */
      startRow = row1;
      endRow   = row2;
      endRow   += rowOffset; 
    }/* We don't have to worry about rows */
  if(thetaY==0.0)
    {/* We don't have to worry about cols */
      startCol = col1;
      endCol   = col2;
      endCol   += colOffset;
    }/* We don't have to worry about cols */

  topSec = origNumSecs;
  if(!lowRes)
    topSec = largestDim;  /* Does this give us a better rotation??? */
  else topSec = origNumSecs;

  /* Try this */
  topSec = largestDim;

  bottomSec = 0;
  startCol= 0;
  endCol  = largestDim;
  startRow= 0;
  endRow  = largestDim;


  if(!lowRes && (radialMapShade || saveRadialMapShade))
    {/* Do for Highres */
      /* topSec *= 2.0; */     /* In case we have to rotate it */
      xCen = radialXcen;
      yCen = radialYcen;
      zCen = radialZcen; 
    }/* Do for Highres */


  for(row = startRow;row<endRow;row++)
    {/*! process data row by row*/
      
      ySq = (double)(((double)row - yCen)*
		     ((double)row - yCen));
      
      for(col = startCol;col <endCol;col++)
	{/* for col */
	  tmp1 = 0.0;  /*	! initialize tmp1 */
	  
	  xSq = (double)(((double)col - xCen)*
			 ((double)col - xCen));
	  
	  qSq = (xSq+ySq);
	  
	  for(secn = (topSec-1);secn>=(bottomSec);secn--)
	    {/* for secn */
	      
	      /* CHeck to see if we're in radius */
	      zSq = (double)(((double)secn - zCen)*
			     ((double)secn - zCen));
	      mapRadiusSq = (qSq+zSq);
	      if(mapRadiusSq < minRadSq ||
		 mapRadiusSq > maxRadSq)
		{/* OK to use this */
		  goto nextSection;
		}/* OK to use this */
	      
	      /* Init it */
	      dens1= surf_dens - 1.0;
	      
	      /* Get dens1 based on the rotation of thetaX
	       * and thetaY */
	      /* First do rotation in Y */
	      if(thetaY != 0.0)
		{/* rotate */
		  thisSecnF = (double)(
				       ((double)((double)col-xCen)*
					(double)(0.0-sthetaY))+
				       ((double)((double)secn-zCen) * 
					cthetaY)
				       );
		  thisRowF  = (double)row;
		  thisColF  = (double)(((double)((double)col-xCen)*
					cthetaY)+
				       ((double)((double)secn-zCen)*
					sthetaY));
		  thisSecnF  += (double)zCen;
		  thisColF  += (double)xCen;
		}/* rotate */
	      else
		{/* No rotate */
		  thisSecnF = (double)secn;
		  thisRowF  = (double)row;
		  thisColF  = (double)col;
		}/* No rotate */
	      /* Now do it in X */
	      if(thetaX != 0.0)
		{/* rotate */
		  thisSecn = (double)(
				      ((thisRowF-(double)yCen) * 
				       sthetaX)+
				      ((thisSecnF-(double)zCen) * 
				       cthetaX)
				      );
		  thisRow  = (double)(
				      ((thisRowF-(double)yCen) * 
				       cthetaX)+
				      ((thisSecnF-(double)zCen) * 
				       (double)(0.0-sthetaX))
				      );
		  thisCol  = (double)thisColF;
		  thisRow  += (double)yCen;
		  thisSecn += (double)zCen;
		}/* rotate */
	      else
		{/* Don't rotate */
		  thisSecn = (double)thisSecnF;
		  thisRow  = (double)thisRowF;
		  thisCol  = (double)thisColF;
		}/* Don't rotate */
	      
	      /* Center this thing */
	      thisCol -= colOffset;
	      thisRow -= rowOffset;
	      
	      
#ifdef OLDWAY_WORKS_here_983284
	      if(
		 (thisSecn >= 0.0 && thisSecn < (double)(origNumSecsM1)) &&
		 (thisRow  >= 0.0 && thisRow  < (double)(nrowM1)) &&
		 (thisCol  >= 0.0 && thisCol  < (double)(ncolM1)) 
		 )
		{/* OK to try this */
		}/* OK to try this */
#endif
	      /* October 21, 1998 */
	      if(
		 (thisSecn >= sec1 && thisSecn < (double)(sec2M1)) &&
		 (thisRow  >= row1 && thisRow  < (double)(row2M1)) &&
		 (thisCol  >= col1 && thisCol  < (double)(col2M1)) 
		 )
		{/* OK to try this */
		  
		  double Ic,Jc,Kc,If,Jf,Kf;
		  double i,j,k;
		  
		  /* Add 0.0001 to all to force the ceiling to be
		   * the next pixel for thetas of modulus 90.0 */
		  Ic= (double)ceil((thisCol));
		  Jc= (double)ceil((thisRow));
		  Kc= (double)ceil((thisSecn));
		  If = (double)floor(thisCol);
		  Jf = (double)floor(thisRow);
		  Kf = (double)floor(thisSecn);
		  i = (double)thisCol;
		  j = (double)thisRow;
		  k = (double)thisSecn;
		  if(Ic==If)
		    Ic = If + (double)1.000000000;
		  if(Jc==Jf)
		    Jc = Jf + (double)1.000000000;
		  if(Kc==Kf)
		    Kc = Kf + (double)1.000000000;
		  if(thetaY != 0.0    && thetaX != 0.0 )
		    {/* DO it this way */
		      dens1    = (double)
			(
			 (Ic-i)*(Jc-j)*(Kc-k) * (double)COREMAPREF((Ic-1.0),
								   (Jc-1.0),
								   (Kc-1.0)) +
			 (Ic-i)*(Jc-j)*(k-Kf) * (double)COREMAPREF((Ic-1.0),
								   (Jc-1.0),
								   Kc) +
			 (i-If)*(Jc-j)*(Kc-k) * (double)COREMAPREF(Ic,(Jc-1.0),
								   (Kc-1.0)) +
			 (i-If)*(Jc-j)*(k-Kf) * (double)COREMAPREF(Ic,(Jc-1.0),
								   Kc) +
			 (Ic-i)*(j-Jf)*(Kc-k) * (double)COREMAPREF((Ic-1.0),Jc,
								   (Kc-1.0)) +
			 (Ic-i)*(j-Jf)*(k-Kf) * (double)COREMAPREF((Ic-1.0),Jc,
								   Kc) +
			 (i-If)*(j-Jf)*(Kc-k) * (double)COREMAPREF(Ic,Jc,
								   (Kc-1.0)) +
			 (i-If)*(j-Jf)*(k-Kf) * (double)COREMAPREF(Ic,Jc,Kc) 
			 );
		    }/* DO it this way */
		  else if(thetaY == 0.0 && thetaX != 0.0)
		    {/* DO it this way */
		      dens1    = (double)
			(
			 (Jc-j)*(Kc-k) * (double)COREMAPREF(If,(Jc-1.0),
							    (Kc-1.0)) +
			 (Jc-j)*(k-Kf) * (double)COREMAPREF(If,(Jc-1.0),Kc) +
			 (j-Jf)*(Kc-k) * (double)COREMAPREF(If,Jc,(Kc-1.0)) +
			 (j-Jf)*(k-Kf) * (double)COREMAPREF(If,Jc,Kc) 
			 );
		    }/* DO it this way */
		  else if(thetaY != 0.0 && thetaX == 0.0)
		    {/* DO it this way */
		      dens1    = (double)
			(
			 (Ic-i)*(Kc-k) * (double)COREMAPREF((Ic-1.0),Jf,
							    (Kc-1.0)) +
			 (Ic-i)*(k-Kf) * (double)COREMAPREF((Ic-1.0),Jf,Kc) +
			 (i-If)*(Kc-k) * (double)COREMAPREF(Ic,Jf,(Kc-1.0)) +
			 (i-If)*(k-Kf) * (double)COREMAPREF(Ic,Jf,Kc) 
			 );
		    }/* DO it this way */
		  else
		    {/* No rotation */
		      dens1    = (double)coremap[(
						  (((int)thisSecn)*ncol*nrow)+
						  (((int)thisRow)*ncol)+
						  ((int)thisCol))];
		    }/* No rotation */
		  
		  dens1 *= thisIrev;
		  
		}/* OK to try this */
	      else
		{/* Just go to next pixel */
		  dens1 = (double)surf_dens - (double)1.0;
		}/* Just go to next pixel */
	      
	      if(dens1 >= surf_dens) 
		{/*then     ! reached threshold density level */
		  
#ifdef NEEDED_here_for_first_section
		  if((secn     >= (topSec-1))  ||
		     ((int)thisSecn <= 1) ||
		     ((int)thisRow  >= (nrow-1)) ||
		     ((int)thisCol  >= (ncol-1)) ||
		     ((int)thisRow  <= 1) ||
		     ((int)thisCol  <= 1))
		    {/*then	     ! skip top section/row/col */
		      
		      tmp1 = (topSec-secn);
		      
		      if(tmp1 > topSec)
			tmp1= (double)topSec;
		      
		      topSecMtmp1 = topSec - tmp1;

		      dphshd_map[((row*largestDim)+col)] = 
			shad_map[((row*largestDim)+col)] = 
			  (float)
			    (topSecMtmp1);
		      
		      if(useForMinMax)
			{/* If radial we may not want to include
			  * the depth-cued one here */
			  
			  if(topSecMtmp1 != 0.0)
			    {/* DOn't include bckgrnd in min/max */
			      if(topSecMtmp1 < map_min)
				map_min = topSecMtmp1; /*globmin */
			      if(topSecMtmp1 > map_max)
				map_max = topSecMtmp1; /*globmax */
			    }/* DOn't include bckgrnd in min/max */
			  
			}/* If radial we may not want to include
			  * the depth-cued one here */
		      
		      /* Found the surface we're looking for -
		       * goto next column */
		      goto nextColumn;
		      
		    }/*then	     ! skip top section/row/col */
		  else
#else
		    if(TRUE)
#endif
		      {/* Not top row/col/sec else secn */
			double Ic,Jc,Kc,If,Jf,Kf;
			double i,j,k;
			
			/* Init it */
			dens2= surf_dens - 1.0;
			
			/* This mimics what's done for dens1 above */
			/* Get dens2 here from rotation of thetaX
			 * and thetaY */
			/* First do rotation in Y */
			if(thetaY != 0.0)
			  {/* rotate */
			    thisSecnF = (double)(
						 ((double)(
							   (double)col-xCen)*
						  (double)(0.0-sthetaY))+
						 ((double)(
							   ((double)secn+1.0)-
							   zCen) * 
						  cthetaY)
						 );
			    thisRowF  = (double)row;
			    thisColF  = (double)(((double)(
							   (double)col-xCen)*
						  cthetaY)+
						 ((double)(
							   ((double)secn+1.0)-
							   zCen)*
						  sthetaY));
			    thisSecnF  += (double)zCen;
			    thisColF  += (double)xCen;
			  }/* rotate */
			else
			  {/* No rotate */
			    thisSecnF = (double)((double)secn+1.0);
			    thisRowF  = (double)row;
			    thisColF  = (double)col;
			  }/* No rotate */
			/* Now do it in X */
			if(thetaX != 0.0)
			  {/* rotate */
			    thisSecn = (double)(
						((thisRowF-
						  (double)yCen) * 
						 sthetaX)+
						((thisSecnF-
						  (double)zCen) * 
						 cthetaX)
						);
			    thisRow  = (double)(
						((thisRowF-
						  (double)yCen) * 
						 cthetaX)+
						((thisSecnF-
						  (double)zCen) * 
						 (double)(0.0-sthetaX))
						);
			    thisCol  = (double)thisColF;
			    thisRow  += (double)yCen;
			    thisSecn += (double)zCen;
			  }/* rotate */
			else
			  {/* Don't rotate */
			    thisSecn = (double)thisSecnF;
			    thisRow  = (double)thisRowF;
			    thisCol  = (double)thisColF;
			  }/* Don't rotate */
			
			/* Center this thing */
			thisCol -= colOffset;
			thisRow -= rowOffset;
			
			/* We need to be able to see pixels outside the cut zones so
			 * do a wide search here */
#define OLDWAY_WORKS_HERE_123_8742 1
#ifdef OLDWAY_WORKS_HERE_123_8742
			if(
			   (thisSecn >= 0.0 && 
			    thisSecn < (double)origNumSecsM1) &&
			   (thisRow  >= 0.0 && 
			    thisRow  < (double)nrowM1) &&
			   (thisCol  >= 0.0 && 
			    thisCol  < (double)ncolM1) 
			   )
#else
			  if(
			     (thisSecn >= sec1 && 
			      thisSecn < (double)sec2M1) &&
			     (thisRow  >= row1 && 
			      thisRow  < (double)row2M1) &&
			     (thisCol  >= col1 && 
			      thisCol  < (double)col2M1) 
			     )
#endif
			    {/* OK to proceed */
			      /* Add 0.0001 to all to force the ceiling 
			       * to be
			       * the next pixel for thetas of modulus 
			       * 90.0 */
#ifdef OLDWAY_trythis_99
			      Ic= (double)ceil((thisCol+0.000001));
			      Jc= (double)ceil((thisRow+0.000001));
			      Kc= (double)ceil((thisSecn+0.000001));
#else
			      Ic= (double)ceil((thisCol));
			      Jc= (double)ceil((thisRow));
			      Kc= (double)ceil((thisSecn));
#endif
			      If = (double)floor(thisCol);
			      Jf = (double)floor(thisRow);
			      Kf = (double)floor(thisSecn);
			      if(Ic==If)
				Ic = If + (double)1.000000000;
			      if(Jc==Jf)
				Jc = Jf + (double)1.000000000;
			      if(Kc==Kf)
				Kc = Kf + (double)1.000000000;
			      i = (double)thisCol;
			      j = (double)thisRow;
			      k = (double)thisSecn;
			      
			      if(thetaY != 0.0 && thetaX != 0.0)
				{/* DO it this way */
				  dens2    = (double)
				    (
		     (Ic-i)*(Jc-j)*(Kc-k) * (double)COREMAPREF((Ic-1.0),
							       (Jc-1.0),
							       (Kc-1.0)) +
		     (Ic-i)*(Jc-j)*(k-Kf) * (double)COREMAPREF((Ic-1.0),
							       (Jc-1.0),
							       Kc) +
		     (i-If)*(Jc-j)*(Kc-k) * (double)COREMAPREF(Ic,(Jc-1.0),
							       (Kc-1.0)) +
		     (i-If)*(Jc-j)*(k-Kf) * (double)COREMAPREF(Ic,
							       (Jc-1.0),
							       Kc) +
		     (Ic-i)*(j-Jf)*(Kc-k) * (double)COREMAPREF((Ic-1.0),Jc,
							       (Kc-1.0)) +
		     (Ic-i)*(j-Jf)*(k-Kf) * (double)COREMAPREF((Ic-1.0),
							       Jc,Kc) +
		     (i-If)*(j-Jf)*(Kc-k) * (double)COREMAPREF(Ic,Jc,
							       (Kc-1.0)) +
		     (i-If)*(j-Jf)*(k-Kf) * (double)COREMAPREF(Ic,Jc,Kc) 
		     );
				}/* DO it this way */
			      else if(thetaY == 0.0 && thetaX != 0.0)
				{/* DO it this way */
				  dens2    = (double)
				    (
		     (Jc-j)*(Kc-k) * (double)COREMAPREF(If,
							(Jc-1.0),
							(Kc-1.0)) +
		     (Jc-j)*(k-Kf) * (double)COREMAPREF(If,
							(Jc-1.0),
							Kc) +
		     (j-Jf)*(Kc-k) * (double)COREMAPREF(If,Jc,
							(Kc-1.0)) +
		     (j-Jf)*(k-Kf) * (double)COREMAPREF(If,Jc,Kc) 
				     );
				}/* DO it this way */
			      else if(thetaY != 0.0 && thetaX == 0.0)
				{/* DO it this way */
				  dens2    = (double)
				    (
		     (Ic-i)*(Kc-k) * (double)COREMAPREF((Ic-1.0),
							Jf,
							(Kc-1.0)) +
		     (Ic-i)*(k-Kf) * (double)COREMAPREF((Ic-1.0),
							Jf,Kc) +
		     (i-If)*(Kc-k) * (double)COREMAPREF(Ic,Jf,
							(Kc-1.0)) +
		     (i-If)*(k-Kf) * (double)COREMAPREF(Ic,Jf,Kc) 
				     );
				}/* DO it this way */
			      else
				{/* No rotation */
				  dens2    = (double)coremap[(
							      (((int)thisSecn)*
							       ncol*nrow)+
							      (((int)thisRow)*
							       ncol)+
							      ((int)thisCol))];
				}/* No rotation */
			      
			      dens2 *= thisIrev;
			      
			      /*******************************************
			       * This is it!!!!!  If you're going from the
			       * inside out then reverse the dens2 
			       * concept!!
			       ******************************************/
			      if(((secn <= zCen) && 
				  (mapRadiusSq <= minRadSqT1p1)) ||
				 (mapRadiusSq >= maxRadSqTp95))
				dens2 *= -1.0;
			      
			      dens_diff = (double)dens1 - (double)dens2;
			      
			      tmp1      = (double)(topSec-secn);
			      
			      /*
			       *****************************************
			       *    trap for dens_diff close to 0.0    *
			       *****************************************
			       */
			      if(dens_diff < 0.0)
				absDensDiff = 0.0 - dens_diff;
			      else absDensDiff = (double)dens_diff;
			      
			      if(absDensDiff < 0.001) 
				{/*then*/
				  /* Do nothing tmp1 = tmp1; */
				}/*then*/
			      else
				{/* else A*/
				  double numer,denom;
				  numer = (double)((double)surf_dens-
						   (double)dens2);
				  denom = (double)dens_diff;
				  
				  if(denom != 0.0)
				    tmp_fract = (double)(numer) / 
				      (double)(denom);
				  else tmp_fract = 0.0;
				  
				  if(dens1 <= dens2)
				    tmp_fract = 0.0;
				  
				  
				  if(thetaY != 0.0 || thetaX != 0.0)
				    {/* Then account for slight differences */
				      if(thisCol >= (double)(col2-2))
					{/* We're on a cutting edge */
					  /* Get the fractional difference */
					  tmp_fract = thisCol - 
					    (double)(col2-2);
					}/* We're on a cutting edge */
				      if(thisRow >= (double)(row2-2))
					{/* We're on a cutting edge */
					  /* Get the fractional difference */
					  tmp_fract = thisRow - 
					    (double)(row2-2);
					}/* We're on a cutting edge */
				      if(thisSecn >= (double)(sec2-2))
					{/* We're on a cutting edge */
					  /* Get the fractional difference */
					  tmp_fract = thisSecn - 
					    (double)(sec2-2);
					}/* We're on a cutting edge */
				    }/* Then account for slight differences */
				  
				  
				  if(tmp_fract > 0.0) 
				    tmp1 = tmp1 + (double)tmp_fract;
				  
				}/* else A*/
			      
			      if(radialMapShade)
				{/* COmpute the radius here */
				  if(!lowRes)
				    {/* Do for Highres */
				      double q, qSq;
				      
				      xSq = ((double)col - radialXcen)*
					((double)col - radialXcen);
				      ySq = ((double)row - radialYcen)*
					((double)row - radialYcen);
				      
				      qSq = (xSq+ySq);
				      
				      zSq = ((double)secn - radialZcen)*
					((double)secn - radialZcen);
				      mapRadius = sqrt((qSq+zSq));
				      
				      /* Why do we have to halve it ??? */
				      /* mapRadius *= 0.5;   */
				      
				      tmp1= (topRadialSec - mapRadius) ;
				      /* If the center pixel is zero then
				       * it looks just like background -
				       * let's push it up above backgrd */
				      if(tmp1>=topRadialSec)
					tmp1 = topRadialSec-1.0; 
				      
				      topSecMtmp1 = topRadialSec - tmp1;
				      topSecMtmp1 = mapRadius; 
				      
				    }/* Do for Highres */
				  else
				    {/* Normal */
				      
#ifdef NOT_DONE_ABOVE
				      double q, qSq;
				      xSq = ((double)col - xCen)*
					((double)col - xCen);
				      ySq = ((double)row - yCen)*
					((double)row - yCen);
				      
				      qSq = (xSq+ySq);
				      
				      zSq = ((double)secn - zCen)*
					((double)secn - zCen);
				      mapRadius = sqrt((qSq+zSq));
#else
				      mapRadius = sqrt(mapRadiusSq);
#endif				  
				      
				      tmp1= (topSec - mapRadius) ;
				      /* If the center pixel is zero then
				       * it looks just like background -
				       * let's push it up above backgrd */
				      if(tmp1==topSec)
					tmp1 = topSec-1.0; 
				      
				    }/* Normal */
				}/* COmpute the radius here */
			      
			      if(lowRes || !radialMapShade)
				{/* OK to do these */
				  if(tmp1 > topSec)
				    tmp1= (double)topSec;
				  
				  topSecMtmp1 = topSec - tmp1;
				}/* OK to do these */
			      else
				{/* We're high res and radialMapShade */
				  /* Just use map radius */
				}/* We're high res and radialMapShade */
			      
			      dphshd_map[((row*largestDim)+col)] = 
				shad_map[((row*largestDim)+col)] = 
				  (float)
				    (topSecMtmp1);
			      
			      if(useForMinMax)
				{/* If radial we may not want to include
				  * the depth-cued one here */
				  
				  if(topSecMtmp1 != 0.0)
				    {/* DOn't include bckgrnd in min/max */
				      if(topSecMtmp1 < map_min)
					map_min = topSecMtmp1; /*globmin */
				      if(topSecMtmp1 > map_max)
					map_max = topSecMtmp1; /*globmax */
				    }/* DOn't include bckgrnd in min/max */
				  
				}/* If radial we may not want to include
				  * the depth-cued one here */
			      
			      /* Found the surface we're looking for -
			       * goto next column */
			      goto nextColumn;
			      
			    }/* OK to proceed */
			
		      }/* Not top row/col/sec else secn */
		  
		}/*then     ! reached threshold density level */
	      
	      
	    nextSection:;
	    }/* for secn *//* ! step to next map section */
	  
	  
	nextColumn:;
	}/* for col ! step to next map column*/
      
      
    }/*! process data row by row*//*! step to next map row*/
  
}/* dph_nomem_tilt_fast */
  




/*********************************************************
 * newTranslateThetaPhiOmega() - Newer rtn for translating
 * the roations.  Might be superior???
 */
void newTranslateThetaPhiOmega(double *thisCol,
			       double *thisRow,
			       double *thisSecn,
			       double xCen,double yCen,double zCen,
			       int col,int row,int secn,
			       double t1,
			       double t2,
			       double t3,
			       double t4,
			       double t5,
			       double t6,
			       double t7,
			       double t8,
			       double t9
			       )
{/* NEWTRANSLATETHETAPHIOMEGA */
  double x,y,z;

  z   = (double)(secn) - zCen;
  y   = (double)(row) - yCen;
  x   = (double)(col) - xCen;
  
  *thisSecn   = t7*x + t8*y + t9*z + zCen;
  *thisCol    = t1*x + t2*y + t3*z + xCen;
  *thisRow    = t4*x + t5*y + t6*z + yCen;

}/* NEWTRANSLATETHETAPHIOMEGA */





/*************************************************************
 * translateThetaPhiOmega() - Orig way to do the rotation.
 * This rtn might be useful if we ever get the x/y/z rotation
 * implemented.
 */
void translateThetaPhiOmega(double *thisCol,
			    double *thisRow,
			    double *thisSecn,
			    double xCen,double yCen,double zCen,
			    int col,int row,int secn,
			    double theta, double phi, double omega,
			    double stheta, double ctheta,
			    double sphi, double cphi,
			    double somega, double comega
			    )
{/* TRANSLATETHETAPHIOMEGA */
  double thisColF,thisRowF,thisSecnF;
  float tmpX,tmpY,tmpZ,tx,ty,tz;
  float origX,origY,origZ;

#ifdef DO_OLD_XY_WAY
  /* First do rotation in theta */
  if(theta != 0.0)
    {/* rotate */
      thisSecnF = (double)(
			   ((double)(
				     (double)col-xCen)*
			    (double)(0.0-stheta))+
			   ((double)(
				     ((double)secn)-
				     zCen) * 
			    ctheta)
			   );
      thisRowF  = (double)row;
      thisColF  = (double)(((double)(
				     (double)col-xCen)*
			    ctheta)+
			   ((double)(
				     ((double)secn)-
				     zCen)*
			    stheta));
      thisSecnF  += (double)zCen;
      thisColF  += (double)xCen;
    }/* rotate */
  else
    {/* No rotate */
      thisSecnF = (double)((double)secn);
      thisRowF  = (double)row;
      thisColF  = (double)col;
    }/* No rotate */
  /* Now do it in Phi */
  if(phi != 0.0)
    {/* rotate */
      *thisSecn = (double)(
			  ((thisRowF-
			    (double)yCen) * 
			   sphi)+
			  ((thisSecnF-
			    (double)zCen) * 
			   cphi)
			  );
      *thisRow  = (double)(
			  ((thisRowF-
			    (double)yCen) * 
			   cphi)+
			  ((thisSecnF-
			    (double)zCen) * 
			   (double)(0.0-sphi))
			  );
      *thisCol  = (double)thisColF;
      *thisRow  += (double)yCen;
      *thisSecn += (double)zCen;
    }/* rotate */
  else
    {/* Don't rotate */
      *thisSecn = (double)thisSecnF;
      *thisRow  = (double)thisRowF;
      *thisCol  = (double)thisColF;
    }/* Don't rotate */
#else

  origX= tmpX = col - xCen;
  origY= tmpY = row - yCen;
  origZ= tmpZ = secn - zCen;

  if(theta != 0.0)
    {/* theta */
      tx=(tmpX*ctheta+tmpZ*stheta);
      tz=(tmpX*-stheta+tmpZ*ctheta);
      tmpX = tx;
      tmpZ = tz;
    }/* theta */

  if(phi != 0.0)
    {/* phi */
      ty=(tmpY*cphi+tmpZ*sphi);
      tz=(tmpY*-sphi+tmpZ*cphi);
      tmpY = ty;
      tmpZ = tz;
/* Normal way we did it -
      ty=(tmpY*cphi+tmpZ*sphi);
      tz=(tmpY*-sphi+tmpZ*cphi);
      tmpY = ty;
      tmpZ = tz;
*/
    }/* phi */

  if(omega != 0.0)
    {/* Omega */
      tx=(tmpX*comega+tmpY*somega);
      ty=(tmpX*-somega+tmpY*comega);
      tmpX = tx;
      tmpY = ty;
/* Normal -
      tx=(tmpX*comega+tmpY*somega);
      ty=(tmpX*-somega+tmpY*comega);
      tmpX = tx;
      tmpY = ty;
*/
    }/* Omega */

  tmpX += xCen;
  tmpY += yCen;
  tmpZ += zCen;

  *thisSecn = (double)tmpZ;
  *thisRow  = (double)tmpY;
  *thisCol  = (double)tmpX;
  
#endif

}/* TRANSLATETHETAPHIOMEGA */


  


/*******************************************************************
 * getVoxel() - Get the interpolated voxel at the desired location.
 * NOTE:  We need to add the omega interpolation here.
*/
double getVoxel(float *coremap,
		double thisCol, double thisRow, double thisSecn,
		double theta, double phi, double omega,
		int ncol, int nrow)
{/* GETVOXEL */
  double rtnVal;
  double i,j,k,Ic,Jc,Kc,If,Jf,Kf;
#define COREMAPREF(x,y,z) coremap[((((int)z)*ncol*nrow)+(((int)y)*ncol)+((int)x))]

  Ic= (double)ceil((thisCol));
  Jc= (double)ceil((thisRow));
  Kc= (double)ceil((thisSecn));
  If = (double)floor(thisCol);
  Jf = (double)floor(thisRow);
  Kf = (double)floor(thisSecn);
  if(Ic==If)
    Ic = If + (double)1.000000000;
  if(Jc==Jf)
    Jc = Jf + (double)1.000000000;
  if((Kc==Kf) && (Kf < (double)largestDim-1.0))
    Kc = Kf + (double)1.000000000;
  i = (double)thisCol;
  j = (double)thisRow;
  k = (double)thisSecn;

  if(theta != 0.0 || phi != 0.0 || omega != 0.0)
    {/* DO it this way */
      rtnVal    = (double)
	(
	 (Ic-i)*(Jc-j)*(Kc-k) * (double)COREMAPREF((Ic-1.0),
						   (Jc-1.0),
						   (Kc-1.0)) +
	 (Ic-i)*(Jc-j)*(k-Kf) * (double)COREMAPREF((Ic-1.0),
						   (Jc-1.0),
						   Kc) +
	 (i-If)*(Jc-j)*(Kc-k) * (double)COREMAPREF(Ic,(Jc-1.0),
						   (Kc-1.0)) +
	 (i-If)*(Jc-j)*(k-Kf) * (double)COREMAPREF(Ic,
						   (Jc-1.0),
						   Kc) +
	 (Ic-i)*(j-Jf)*(Kc-k) * (double)COREMAPREF((Ic-1.0),Jc,
						   (Kc-1.0)) +
	 (Ic-i)*(j-Jf)*(k-Kf) * (double)COREMAPREF((Ic-1.0),
						   Jc,Kc) +
	 (i-If)*(j-Jf)*(Kc-k) * (double)COREMAPREF(Ic,Jc,
						   (Kc-1.0)) +
	 (i-If)*(j-Jf)*(k-Kf) * (double)COREMAPREF(Ic,Jc,Kc) 
	 );

    }/* DO it this way */
  else
    {/* No rotation */

      rtnVal    = (double)coremap[(
				  (((int)thisSecn)*
				   ncol*nrow)+
				  (((int)thisRow)*
				   ncol)+
				  ((int)thisCol))];
    }/* No rotation */
  

  return(rtnVal);

}/* GETVOXEL */






/***********************************************************************
 * thetaPhi_dphshd_C() - This is the display of the "TRUE" phi rotation.
 * Use the emmap_rob_dphsgd_C rtn for the "normal" way we do it.
 * Not sure why they want 2 ways to do it but there seems to be 
 * controversy as to which way is correct.
 * We are still getting some striping on cut and rotated maps.
C  *                                                                   *
C  * Modified from DPH_NOMEM_TILT_FAST   
C  *********************************************************************
C
*/
void thetaPhi_dphshd_C (float *coremap,
			float *origMap,
			int ncol, int nrow, 
			float *dphshd_map, 
			BYTE shift, BYTE surf, 
			int sec1, int sec2, 
			int col1, int col2,
			int row1, int row2,
			int num_views,
			int thisIrev, float surf_dens,
			int useForMinMax,
			float *shad_map,
			double phi, double theta, double omega,
			double minRadius, double maxRadius,
			double xCen, double yCen, double zCen,
			float radialXcen, float radialYcen, float radialZcen,
			int lowRes, int topRadialSec, int step)
{/* THETAPHI_DPHSHD_C */
  int      col, row, r, s, printIts;
  int      startCol,endCol,startRow,endRow;
  double   thisSecn,thisRow,thisCol;
  double   diffit;
  int      secn, topSec, bottomSec;
  int      colOffset,rowOffset,secOffset;
  double   origNumSecsM1,nrowM1,ncolM1,sec2M1,row2M1,col2M1;
  double   topSecMtmp1;
  double   tmp1;
  double   tmp_fract,cutFract;
  double   dens1, dens2, dens_diff, absDensDiff;
  double   q, qSq, mapRadiusSq, minRadSq,maxRadSq,minRadSqT1p1,maxRadSqTp95;
  double   xSq,ySq,zSq,mapRadius;
  double   thisSecnF,thisRowF,thisColF;
  double   cphi,sphi,ctheta,stheta,comega,somega;
  double   NUsec1,NUsec2M1;
  double   t1,t2,t3,t4,t5,t6,t7,t8,t9;
  double   lastTopSecMtmp1,lastTmp1,lastFract;
  double   ySqp,xSqp,qSqp,zSqp,mapRadiusSqp,mRSp;
  double   ySqq,xSqq,qSqq,zSqq,mapRadiusSqq,mRSq;
#define COREMAPREF(x,y,z) coremap[((((int)z)*ncol*nrow)+(((int)y)*ncol)+((int)x))]
  
  printIts = 0;

  origNumSecsM1= (double)(origNumSecs - 1.0);
  nrowM1       = (double)(nrow - 1.0);
  ncolM1       = (double)(ncol - 1.0);
  sec2M1 = sec2 - 1.0;
  row2M1 = row2 - 1.0;
  col2M1 = col2 - 1.0;

  minRadSq= minRadius * minRadius;
  /* Compute 10% past the lowest radius to see if we need to 
   * flip the sign or not (See below). */
  minRadSqT1p1 = minRadSq * 1.1;
  maxRadSq= maxRadius * maxRadius;
  maxRadSqTp95 = maxRadSq * 0.95;

  /* We've got a bug whereby we've got to add 90 degress to
   * our omega values ... */
#ifdef ORIGWAY
  comega = (double)cos((double)deg2rad((double)(omega+90.0)));
  somega = (double)sin((double)deg2rad((double)(omega+90.0)));
  comega = (double)cos((double)deg2rad((double)omega));
  somega = (double)sin((double)deg2rad((double)omega));
  cphi = (double)cos((double)deg2rad((double)phi));
  sphi = (double)sin((double)deg2rad((double)phi));
  ctheta = (double)cos((double)deg2rad((double)theta));
  stheta = (double)sin((double)deg2rad((double)theta));
#else
  comega = (double)cos((double)deg2rad((double)0.0-phi));
  somega = (double)sin((double)deg2rad((double)0.0-phi));
  cphi = (double)cos((double)deg2rad((double)omega));
  sphi = (double)sin((double)deg2rad((double)omega));
  ctheta = (double)cos((double)deg2rad((double)0.0-theta));
  stheta = (double)sin((double)deg2rad((double)0.0-theta));
#endif

  /*********************************************
   *    COMPUTE ELEMENTS OF ROTATION MATRIX    *
   *********************************************/
  t1 = ctheta*cphi*comega + sphi*somega;
  t2 = ctheta*sphi*comega - cphi*somega;
  t3 = -stheta*comega;
  t4 = ctheta*cphi*somega - sphi*comega;
  t5 = ctheta*sphi*somega + cphi*comega;
  t6 = -stheta*somega;
  t7 = stheta*cphi;
  t8 = stheta*sphi;
  t9 = ctheta;


  /* How much do we have to offset for asymmetric maps */
  colOffset = (largestDim - ncol) / 2;
  rowOffset = (largestDim - nrow) / 2;
  secOffset = (largestDim - origNumSecs) / 2;
  xCen += colOffset;
  yCen += rowOffset;

  radialXcen += colOffset;
  radialYcen += rowOffset;

  /* Now try and optimize for speed */
  startCol= 0;
  endCol  = largestDim;
  startRow= 0;
  endRow  = largestDim;

  if(omega==0.0)
    {/* only do if omega is zero */
      if(phi==0.0)
	{/* We don't have to worry about rows */
	  startRow = row1;
	  endRow   = row2;
	  endRow   += rowOffset; 
	}/* We don't have to worry about rows */
      if(theta==0.0)
	{/* We don't have to worry about cols */
	  startCol = col1;
	  endCol   = col2;
	  endCol   += colOffset;
	}/* We don't have to worry about cols */
    }/* only do if omega is zero */


  topSec = origNumSecs;
  if(!lowRes)
    topSec = largestDim;  /* Does this give us a better rotation??? */
  else topSec = origNumSecs;

  /* Try this */
  topSec = largestDim;

  bottomSec = 0;
/********************************
 * Why do we reset these here???
  startCol= 0;
  endCol  = largestDim;
  startRow= 0;
  endRow  = largestDim;
*********************************/

  lastTmp1= lastFract= lastTopSecMtmp1= 0.0;

#define WE_EVER_WANT_TO_ROTATE_HIRES_IMGS_ON_ORIGINAL_AXES 1
#ifdef WE_EVER_WANT_TO_ROTATE_HIRES_IMGS_ON_ORIGINAL_AXES
  if(!lowRes && (radialMapShade || saveRadialMapShade))
    {/* Do for Highres */
      /* topSec *= 2.0; */     /* In case we have to rotate it */
      xCen = radialXcen;
      yCen = radialYcen;
      zCen = radialZcen; 
    }/* Do for Highres */
#endif /*#ifdef WE_EVER_WANT_TO_ROTATE_HIRES_IMGS_ON_ORIGINAL_AXES */


  for(row = startRow;row<endRow;row+=step)
    {/*! process data row by row*/
      ySq = (double)(((double)row - yCen)*
		     ((double)row - yCen));
      
      for(col = startCol;col <endCol;col+=step)
	{/* for col */
	  tmp1 = 0.0;  /*	! initialize tmp1 */
	  
	  xSq = (double)(((double)col - xCen)*
			 ((double)col - xCen));
	  
	  qSq = (xSq+ySq);
	  
	  for(secn = (topSec-1);secn>=(bottomSec);secn--)
	    {/* for secn */
	      
	      /* CHeck to see if we're in radius */
	      zSq = (double)(((double)secn - zCen)*
			     ((double)secn - zCen));
	      mapRadiusSq = (qSq+zSq);
	      cutFract = 0.0;
	      if(
		 (mapRadiusSq < minRadSq || 
		  mapRadiusSq > maxRadSq)
		 )
	      	{/* OK to use this */
		  double sqrtVal;
		  // October 2005 RWA
		  // I added this concept of cutFract to help alleviate
		  // some of the pixel striping we were seeing when we
		  // cut the MAP radially.  Not sure this is really kosher
		  // but it looks like it helps ???
		  sqrtVal = sqrt(mapRadiusSq);
		  diffit = sqrtVal - maxRadius;
		  if((diffit < 1.0)&&(diffit > 0.0))
		    {/* special */
		      cutFract = -99.0;
		    }/* special */
		  diffit = minRadius - sqrtVal;
		  if((diffit < 1.0)&&(diffit > 0.0))
		    {/* special MIN */
		      cutFract = 99.0;
		    }/* special MIN */
		  if(sqrtVal < minRadius && sqrtVal <=1.0)
		    cutFract = 99.00;
		  if(cutFract==0.0)
		    goto nextSection;
	      	}/* OK to use this */
	      
	      /* Init it */
	      dens1= surf_dens - 1.0;
	      
	      newTranslateThetaPhiOmega(&thisCol,&thisRow,&thisSecn,
					xCen,yCen,zCen,
					col,row,secn,
					t1,t2,t3,t4,t5,t6,t7,t8,t9);
	      
	      thisCol -= colOffset;
	      thisRow -= rowOffset;
	      
/* #ifdef OLDWAY_WORKS_here_983284
	      if(
		 (thisSecn >= 0.0 && thisSecn < (double)(origNumSecsM1)) &&
		 (thisRow  >= 0.0 && thisRow  < (double)(nrowM1)) &&
		 (thisCol  >= 0.0 && thisCol  < (double)(ncolM1)) 
		 )
		{/+ OK to try this +/
		  
 *** #else */
	      /* October 21, 1998 which worked OK but for first slice
	       * in a cutX,cutY,cutZ scenario 
	      if(
		 (thisSecn >= sec1 && thisSecn < (double)(sec2M1)) &&
		 (thisRow  >= row1 && thisRow  < (double)(row2M1)) &&
		 (thisCol  >= col1 && thisCol  < (double)(col2M1)) 
		 )
		***********************************/
	      if(
		 (thisSecn >= sec1 && thisSecn <= (double)(sec2M1)) &&
		 (thisRow  >= row1 && thisRow  <= (double)(row2M1)) &&
		 (thisCol  >= col1 && thisCol  <= (double)(col2M1)) 
		 )
		{/* OK to try this */

		  dens1 = getVoxel(coremap,thisCol,thisRow,thisSecn,
				   theta, phi, omega,
				   ncol, nrow);
		  
		  dens1 *= thisIrev;
		  
		}/* OK to try this */
	      else
		{/* Just go to next pixel */
		  dens1 = (double)surf_dens - (double)1.0;
		}/* Just go to next pixel */
	      
	      if(dens1 >= surf_dens) 
		{/*then     ! reached threshold density level */
		  
		  /* Init it */
		  dens2= surf_dens - 1.0;
		  
		  newTranslateThetaPhiOmega(&thisCol,&thisRow,&thisSecn,
					    xCen,yCen,zCen,
					    col,row,(secn+1),
					    t1,t2,t3,t4,t5,t6,t7,t8,t9);
		  
		  /* Center this thing */
		  thisCol -= colOffset;
		  thisRow -= rowOffset;
		  
		  /* We need to be able to see pixels outside the cut 
		   * zones so do a wide search here */
#define OLDWAY_WORKS_HERE_123_8742 1
#ifdef OLDWAY_WORKS_HERE_123_8742
		  if(
		     (thisSecn >= 0.0 && 
		      thisSecn < (double)origNumSecsM1) &&
		     (thisRow  >= 0.0 && 
		      thisRow  < (double)nrowM1) &&
		     (thisCol  >= 0.0 && 
		      thisCol  < (double)ncolM1) 
		     )
#else 
		    if(
		       (thisSecn >= sec1 && 
			thisSecn < (double)sec2M1) &&
		       (thisRow  >= row1 && 
			thisRow  < (double)row2M1) &&
		       (thisCol  >= col1 && 
			thisCol  < (double)col2M1) 
		       )
#endif
		      {/* OK to proceed */
			
			dens2 = getVoxel(coremap,thisCol,thisRow,thisSecn,
					 theta, phi, omega,
					 ncol, nrow);
			dens2 *= thisIrev;
			
			/*******************************************
			 * This is it!!!!!  If you're going from the
			 * inside out then reverse the dens2 
			 * concept!!
			 ******************************************/
#ifdef OLDWAY_20051121
			if(((secn <= zCen) && 
			    (mapRadiusSq <= minRadSqT1p1)) ||
			   (mapRadiusSq >= maxRadSqTp95))
			  dens2 *= -1.0;
#else
			if(((secn < zCen) && 
			    (mapRadiusSq <= minRadSqT1p1)) ||
			   (mapRadiusSq >= maxRadSqTp95))
			  dens2 *= -1.0;
#endif
			
			dens_diff = (double)dens1 - (double)dens2;
			
			tmp1      = (double)(topSec-secn);
			
			/*
			 *****************************************
			 *    trap for dens_diff close to 0.0    *
			 *****************************************
			 */
			if(dens_diff < 0.0)
			  absDensDiff = 0.0 - dens_diff;
			else absDensDiff = (double)dens_diff;
			
			if(absDensDiff < 0.0000001)  
			  {/*then*/
			    /* Do nothing tmp1 = tmp1; */
			  }/*then*/
			else
			  {/* else A*/
			    double numer,denom;
			    numer = (double)((double)surf_dens-
					     (double)dens2);
			    denom = (double)dens_diff;
			    // Why not this ?? denom = (double)absDensDiff;
			    
			    if(denom != 0.0)
			      tmp_fract = (double)(numer) / 
				(double)(denom);
			    else tmp_fract = 0.0;
		
			    // Why do we eliminate the fraction if
			    // the densities are switched ??
			    if(dens1 <= dens2)
			      tmp_fract = 0.0;

		    
#ifdef ORIG_WAY /* Kinda works Oct 3, 2002 - some striping though */
			    if(theta != 0.0 || phi != 0.0 || omega != 0.0)
			      {/* Then account for slight differences */
				if(thisCol >= (double)(col2-2))
				  {/* We're on a cutting edge */
				    /* Get the fractional difference */
				    tmp_fract = thisCol - (double)(col2-2);
				  }/* We're on a cutting edge */
				if(thisRow >= (double)(row2-2))
				  {/* We're on a cutting edge */
				    /* Get the fractional difference */
				    tmp_fract = thisRow - (double)(row2-2);
				  }/* We're on a cutting edge */
				if(thisSecn >= (double)(sec2-2))
				  {/* We're on a cutting edge */
				    /* Get the fractional difference */
				    tmp_fract = thisSecn - (double)(sec2-2);
				  }/* We're on a cutting edge */
			      }/* Then account for slight differences */

			    /*******************************/
			    /* This kinda works 2005/11/21 */
			    /*******************************/
			    if(theta != 0.0 || phi != 0.0 || omega != 0.0)
			      {/* Then account for slight differences */
				if(thisCol >= (double)(col2-1))
				  {/* We're on a cutting edge */
				    /* Get the fractional difference */
				    tmp_fract = thisCol - (double)(col2-1);
				  }/* We're on a cutting edge */
				if(thisRow >= (double)(row2-1))
				  {/* We're on a cutting edge */
				    /* Get the fractional difference */
				    tmp_fract = thisRow - (double)(row2-1);
				  }/* We're on a cutting edge */
				if(thisSecn >= (double)(sec2-1))
				  {/* We're on a cutting edge */
				    /* Get the fractional difference */
				    tmp_fract = thisSecn - (double)(sec2-1);
				  }/* We're on a cutting edge */
			      }/* Then account for slight differences */

#else
			    if(theta != 0.0 || phi != 0.0 || omega != 0.0)
			      {/* Then account for slight differences */

				if(thisCol <= (double)(col1))
				  {/* We're on a cutting edge */

				    /* Get the fractional difference */
				    tmp_fract = ((double)col1) - thisCol;

				  }/* We're on a cutting edge */


				if(thisCol >= (double)(col2-1))
				  {/* We're on a cutting edge */

				    /* Get the fractional difference */
				    tmp_fract = thisCol - (double)(col2-1);

				  }/* We're on a cutting edge */
				if(thisRow < (double)(row1))
				  {/* We're on a cutting edge */
				    /* Get the fractional difference */
				    tmp_fract = row1 - thisRow;
				  }/* We're on a cutting edge */

				if(thisRow >= (double)(row2-1))
				  {/* We're on a cutting edge */
				    /* Get the fractional difference */
				    tmp_fract = thisRow - (double)(row2-1);
				  }/* We're on a cutting edge */

				if(thisSecn < (double)(sec1))
				  {/* We're on a cutting edge */
				    /* Get the fractional difference */
				    tmp_fract = thisSecn - sec1;
				  }/* We're on a cutting edge */

				if(thisSecn >= (double)(sec2-1))
				  {/* We're on a cutting edge */
				    /* Try this */
				    tmp1      = (double)(topSec-thisSecn);
				    /* Get the fractional difference */
				    tmp_fract = thisSecn - (double)(sec2-1);
				  }/* We're on a cutting edge */

			      }/* Then account for slight differences */
#endif

			    if((cutFract == -99.0))
			      {/* Do the MAX cut thing */
				double newzSq,newTmp1;
				newzSq = maxRadSq - qSq;
				if(newzSq > 0.0)
				  newTmp1 = sqrt(newzSq)+zCen;
				else
				  newTmp1 = zCen - sqrt(fabs(newzSq));
				tmp1      = (double)(topSec-newTmp1);
			      }/* Do the MAX cut thing */
			    else if((cutFract == 99.0))
			      {/* Do the MIN cut thing */
				double newzSq,newTmp1;
				newzSq = minRadSq - qSq;
				if(newzSq > 0.0)
				  newTmp1 = sqrt(newzSq)+zCen;
				else
				  newTmp1 = zCen - sqrt(fabs(newzSq));
				tmp1      = (double)(newTmp1);
			      }/* Do the MIN cut thing */
			    else
			      {/* normal */
				if(tmp_fract > 0.0) 
				  tmp1 = tmp1 + (double)tmp_fract;
			      }/* normal */
			  }/* else A*/
			
			if(radialMapShade)
			  {/* COmpute the radius here */
			    if(!lowRes)
			      {/* Do for Highres */
				double q, qSq;
				
				xSq = ((double)col - radialXcen)*
				  ((double)col - radialXcen);
				ySq = ((double)row - radialYcen)*
				  ((double)row - radialYcen);
				
				qSq = (xSq+ySq);
				
				zSq = ((double)secn - radialZcen)*
				  ((double)secn - radialZcen);
				mapRadius = sqrt((qSq+zSq));
				
				/* Why do we have to halve it ??? */
				/* mapRadius *= 0.5;   */
				
				tmp1= (topRadialSec - mapRadius) ;
				/* If the center pixel is zero then
				 * it looks just like background -
				 * let's push it up above backgrd */
				if(tmp1>=topRadialSec)
				  tmp1 = topRadialSec-1.0; 
				
				topSecMtmp1 = topRadialSec - tmp1;
				topSecMtmp1 = mapRadius; 
				
			      }/* Do for Highres */
			    else
			      {/* Normal */
				
#ifdef NOT_DONE_ABOVE
				double q, qSq;
				    xSq = ((double)col - xCen)*
				      ((double)col - xCen);
				ySq = ((double)row - yCen)*
				  ((double)row - yCen);
				
				qSq = (xSq+ySq);
				
				zSq = ((double)secn - zCen)*
				  ((double)secn - zCen);
				mapRadius = sqrt((qSq+zSq));
#else
				mapRadius = sqrt(mapRadiusSq);
#endif				  
				
				tmp1= (topSec - mapRadius) ;
				/* If the center pixel is zero then
				 * it looks just like background -
				 * let's push it up above backgrd */
				if(tmp1==topSec)
				  tmp1 = topSec-1.0; 
				
			      }/* Normal */
			  }/* COmpute the radius here */
			
			if(lowRes || !radialMapShade)
			  {/* OK to do these */
			    if(tmp1 > topSec)
			      tmp1= (double)topSec;
			    
			    topSecMtmp1 = (float)topSec - tmp1;
			  }/* OK to do these */
			else
			  {/* We're high res and radialMapShade */
			    /* Just use map radius */
			  }/* We're high res and radialMapShade */
			

			dphshd_map[((row*largestDim)+col)] = 
			  shad_map[((row*largestDim)+col)] = 
			    (float)
			      (topSecMtmp1);
			
			if(useForMinMax)
			  {/* If radial we may not want to include
			    * the depth-cued one here */
			    
			    if(topSecMtmp1 != 0.0)
			      {/* DOn't include bckgrnd in min/max */
				if(topSecMtmp1 < map_min)
				  map_min = topSecMtmp1; /*globmin */
				if(topSecMtmp1 > map_max)
				  map_max = topSecMtmp1; /*globmax */
			      }/* DOn't include bckgrnd in min/max */
			    
			  }/* If radial we may not want to include
			    * the depth-cued one here */
			
			for(r=0;r<step;r++)
			  {/* propagate this for step */
			    for(s=0;s<step;s++)
			      {/* Do all rows */
				dphshd_map[(((row+s)*largestDim)+(col+r))] = 
				  shad_map[(((row+s)*largestDim)+(col+r))] = 
				    (float)
				      (topSecMtmp1);
			      }/* Do all rows */
			  }/* propagate this for step */

			/* Found the surface we're looking for -
			 * goto next column */
			goto nextColumn;
			
		      }/* OK to proceed */
		  
		  
		}/*then     ! reached threshold density level */
	      
	      
	    nextSection:;
	    }/* for secn *//* ! step to next map section */
	  
	  
	nextColumn:;
	}/* for col ! step to next map column*/
      
      
    }/*! process data row by row*//*! step to next map row*/

  
}/* THETAPHI_DPHSHD_C */




/***********************************************************************
C  *********************************************************************
C  *                                                                   *
C  *            *****     SUBROUTINE ICOSRADIALPROJECT       *****   *
C  *                                                                   *
C  *********************************************************************
* Do an icosahedral radial projection of the map.
*/
void icosRadialProject (float *coremap, 
			int ncol, int nrow, int ndepth, 
			float *buf,
			float x_org, float y_org, float z_org, 
			float radius, float smallestDens,
			float theta, float phi,float omega,
			int colOffset, int rowOffset,int largestDim,
			int thisIrev, int tag, float thresh, float slid,
			int destroy, float minRadius)
{/* ICOSRADIALPROJECT */
 int nsec, irec, cut_num, rad_sec, dummyl, m;
 int jchar, numchari4, col, row, sec, rec_num,secn;
 int Ici,Jci,Kci,Ifi,Jfi,Kfi;
 double rad, rad_sqr, min_rad_sqr, xx, yy, zz, sum;
 double fs, percent, sec_factor, dist,
       dens1, dens2, dens_diff1, dens_diff2, div,
       rad_max, delta_rad, dist_sum,
       thisSecnF,thisRowF,thisColF,thisSecn,thisRow,thisCol,
       xCen,yCen,zCen,Ic,Jc,Kc,If,Jf,Kf,
       i,j,k,thisRadius,xdyRow,xdyCol,xdySec;
 int intCol,intRow,intSec;
 double   cphi,sphi,ctheta,stheta,comega,somega;
 double t1,t2,t3,t4,t5,t6,t7,t8,t9;
 double calcRadius, thisZorg;;

 /* Stuff globals */
 threshold = thresh;
 slide = slid;

 /* Make sure we're displaying density correctly */
 thisIrev = 0.0 - thisIrev;

/*
 *  We are calling this every time for every radius - stepping done
 *  Out side of this routine.
 */

  /* We've got a bug whereby we've got to add 90 degress to
   * our omega values ... */
#ifdef ORIGWAY
  comega = (double)cos((double)deg2rad((double)(omega+90.0)));
  somega = (double)sin((double)deg2rad((double)(omega+90.0)));
  comega = (double)cos((double)deg2rad((double)omega));
  somega = (double)sin((double)deg2rad((double)omega));
  cphi = (double)cos((double)deg2rad((double)phi));
  sphi = (double)sin((double)deg2rad((double)phi));
  ctheta = (double)cos((double)deg2rad((double)theta));
  stheta = (double)sin((double)deg2rad((double)theta));
#else
  comega = (double)cos((double)deg2rad((double)0.0-phi));
  somega = (double)sin((double)deg2rad((double)0.0-phi));
  cphi = (double)cos((double)deg2rad((double)omega));
  sphi = (double)sin((double)deg2rad((double)omega));
  ctheta = (double)cos((double)deg2rad((double)0.0-theta));
  stheta = (double)sin((double)deg2rad((double)0.0-theta));
#endif

  /*********************************************
   *    COMPUTE ELEMENTS OF ROTATION MATRIX    *
   *********************************************/
  t1 = ctheta*cphi*comega + sphi*somega;
  t2 = ctheta*sphi*comega - cphi*somega;
  t3 = -stheta*comega;
  t4 = ctheta*cphi*somega - sphi*comega;
  t5 = ctheta*sphi*somega + cphi*comega;
  t6 = -stheta*somega;
  t7 = stheta*cphi;
  t8 = stheta*sphi;
  t9 = ctheta;

 xCen = (float)((ncol+1) / 2) - 1.0;
 yCen = (float)((nrow+1) / 2) - 1.0;
 zCen = (float)((ndepth+1) / 2)- 1.0;

 map_min = 0.0;
 map_max = 0.0;

 printf("radius[%f]  minRadius[%f]\n",radius,minRadius);

 rad = radius;
 rad_sqr = rad*rad;
 min_rad_sqr = minRadius*minRadius;

 if(destroy)
   thisZorg = ndepth;
 else thisZorg = z_org;

 for(row=0;row<nrow;row++)
   {/* DO ROW = 1,NROW  ! PROCESS DATA ROW BY ROW */
     xx   = ((float)row-x_org)*((float)row-x_org);
     for(col=0;col<ncol;col++)
       {/* DO COL = 1,NCOL */
	 intCol = col + colOffset;
	 intRow = row + rowOffset;
	 buf[(intCol+(intRow*ncol))] = 0.0;	/*! INITIALIZE BUF(COL,row)*/
	 yy       = ((float)col-y_org)*((float)col-y_org);
	 sum      = xx + yy;
	 /* I think we need to have sum < rad_sqr here to exclude 
	  * looking at anything inside the radius NO - we need to look 
	  * at all pixels!!!
	  * if (sum > rad_sqr) 
	  *    goto nextCol;
	  */
	 /* Now here  I'm thinking we need to iterate between z_org and 0 or
	  * z_org and numSecs - e.g. Just look at half of the pixels.
	  */
	 for(secn = 0; secn < (int)thisZorg; secn ++)
	   {/* Do these sections */
	     /*	 dist_sum = sqrt(rad_sqr - sum);
		 dist     = z_org + dist_sum;
		 rad_sec  = (int)dist;
		 */
	     zz = ((float)secn - zCen)*((float)secn - zCen);

	     /* Not sure how to setup sec_factor */
	     /*	     sec_factor = dist - ((float)rad_sec);
		     if (rad_sec > ndepth) 
		     rad_sec = ndepth;
		     if (rad_sec < 1) 
		     rad_sec = 1;
		     */
	     sec_factor = 0.0;

	     /*
	      *  For some reason the row and col got messed in coremap???
	      *  DENS1 = COREMAP(COL,ROW,RAD_SEC)
	      *  DENS1 = COREMAP(rad_sec,col,row)
	      */
	     dens1   = 0.0; 
	     rad_sec = secn; 
	     
	     /* Get dens1 based on the rotation of phi, omega,
	      * and theta */
	     newTranslateThetaPhiOmega(&thisCol,&thisRow,&thisSecn,
				       xCen,yCen,zCen,
				       col,row,rad_sec,
				       t1,t2,t3,t4,t5,t6,t7,t8,t9);
	     
	     /* Call XDY's rtn here with row,col,secn (or whatever we're 
	      * iterating on for z).
	      */
	     thisRadius = (double)radius;
	     xdyRow = thisCol - x_org;
	     xdyCol = (ndepth - thisSecn)  - z_org;
	     xdySec = (nrow - thisRow) - y_org;
	     if(icosshell(xdyCol,xdyRow,xdySec,thisRadius,tag))
	       {/* YES this is on the icosahedral surface */
		 dens1 = getVoxel(coremap,thisCol,thisRow,thisSecn,
				  theta, phi, omega,
				  ncol, nrow);
		 
		 dens1 *= thisIrev;
		 
		 if (rad_sec >= ndepth) 
		   {
		     intCol = col + colOffset;
		     intRow = row + rowOffset;
		     buf[(intCol+(intRow*ncol))] = dens1 - smallestDens;
		   }
		 else
		   {/* DO other */
		     dens2 = 0.0; 
		     rad_sec = secn + 1;
		     
		     /* Get dens2 based on the rotation of phi, omega,
		      * and theta */
		     newTranslateThetaPhiOmega(&thisCol,&thisRow,&thisSecn,
					       xCen,yCen,zCen,
					       col,row,rad_sec,
					       t1,t2,t3,t4,t5,t6,t7,t8,t9);
		     
		     dens2 = getVoxel(coremap,thisCol,thisRow,thisSecn,
				      theta, phi, omega,
				      ncol, nrow);
		     
		     dens2 *= thisIrev;
		     
		     dens_diff1 = dens1*(1.0-sec_factor) ;
		     dens_diff2 = dens2*sec_factor;
		     intCol = col + colOffset;
		     intRow = row + rowOffset;
		     buf[(intCol+(intRow*ncol))]   = (dens_diff1 + 
						      dens_diff2) - 
							smallestDens;
		   }/* DO other */
	     
		 intCol = col + colOffset;
		 intRow = row + rowOffset;
		 map_min = MIN(map_min,buf[(intCol+(intRow*ncol))]);  
		                                        /*GLOBAL MIN*/
		 map_max = MAX(map_max,buf[(intCol+(intRow*ncol))]);  
		                                        /*GLOBAL MAX*/
		 goto nextCol;  /* Go find another column */
	       }/* YES this is on the icosahedral surface */
	     else if(destroy)
	       {/* Candidate for destroying */
		 float temp;
		 temp = sqrt(sum);
		 calcRadius = zz * (temp*temp);
		   
		 if(calcRadius < min_rad_sqr ||
		    calcRadius > rad_sqr)
		   {/* kill it */
		     coremap[(int)((secn*nrow*ncol)+
			      (intRow*ncol)+intCol)] = 0.0;
		   }/* kill it */


	       }/* Candidate for destroying */

	   }/* Do these sections */
	 
       nextCol:
	 dummyl = 0;  /* Do nothing... */
       }/* DO COL = 1,NCOL */
     
     
   }/* DO ROW = 1,NROW  ! PROCESS DATA ROW BY ROW */


}/* ICOSRADIALPROJECT */





/***********************************************************************
C  *********************************************************************
C  *                                                                   *
C  *            *****     SUBROUTINE RAD_DENS_FAST_ROB_C       *****   *
C  *                                                                   *
C  *********************************************************************
* DO a spherical radial projection of the map.
*/
void rad_dens_fast_rob_C (float *coremap, 
			  int ncol, int nrow, int ndepth, 
			  float *buf,
			  float x_org, float y_org, float z_org, 
			  float radius, float smallestDens,
			  float theta, float phi,float omega,
			  int colOffset, int rowOffset,int largestDim,
			  int thisIrev)
{/* rad_dens_fast_rob_C */
  int p,q;
  int nsec, irec, cut_num, rad_sec, dummyl, m;
  int jchar, numchari4, col, row, sec, rec_num,secn;
  int Ici,Jci,Kci,Ifi,Jfi,Kfi;
  double rad, rad_sqr, xx, yy, zz, sum;
  double fs, percent, sec_factor, dist,
       dens1, dens2, dens_diff1, dens_diff2, div,
       rad_max, delta_rad, dist_sum,
       thisSecnF,thisRowF,thisColF,thisSecn,thisRow,thisCol,
       xCen,yCen,zCen,Ic,Jc,Kc,If,Jf,Kf,
       i,j,k;
  int intCol,intRow,intSec;
  double   cphi,sphi,ctheta,stheta,comega,somega;
  double t1,t2,t3,t4,t5,t6,t7,t8,t9;

  /* Make sure we're displaying density correctly */
  thisIrev = 0.0 - thisIrev;

/*
 *  We are calling this every time for every radius - stepping done
 *  Out side of this routine.
 */

  /* We've got a bug whereby we've got to add 90 degress to
   * our omega values ... */
#ifdef ORIGWAY
  comega = (double)cos((double)deg2rad((double)(omega+90.0)));
  somega = (double)sin((double)deg2rad((double)(omega+90.0)));
  comega = (double)cos((double)deg2rad((double)omega));
  somega = (double)sin((double)deg2rad((double)omega));
  cphi = (double)cos((double)deg2rad((double)phi));
  sphi = (double)sin((double)deg2rad((double)phi));
  ctheta = (double)cos((double)deg2rad((double)theta));
  stheta = (double)sin((double)deg2rad((double)theta));
#else
  comega = (double)cos((double)deg2rad((double)0.0-phi));
  somega = (double)sin((double)deg2rad((double)0.0-phi));
  cphi = (double)cos((double)deg2rad((double)omega));
  sphi = (double)sin((double)deg2rad((double)omega));
  ctheta = (double)cos((double)deg2rad((double)0.0-theta));
  stheta = (double)sin((double)deg2rad((double)0.0-theta));
#endif

  /*********************************************
   *    COMPUTE ELEMENTS OF ROTATION MATRIX    *
   *********************************************/
  t1 = ctheta*cphi*comega + sphi*somega;
  t2 = ctheta*sphi*comega - cphi*somega;
  t3 = -stheta*comega;
  t4 = ctheta*cphi*somega - sphi*comega;
  t5 = ctheta*sphi*somega + cphi*comega;
  t6 = -stheta*somega;
  t7 = stheta*cphi;
  t8 = stheta*sphi;
  t9 = ctheta;

 xCen = (float)((ncol+1) / 2) - 1.0;
 yCen = (float)((nrow+1) / 2) - 1.0;
 zCen = (float)((ndepth+1) / 2)- 1.0;

 map_min = 0.0;
 map_max = 0.0;

 rad = radius;
 rad_sqr = rad*rad;
 for(row=0;row<nrow;row++)
   {/* DO ROW = 1,NROW  ! PROCESS DATA ROW BY ROW */
     xx   = ((float)row-x_org)*((float)row-x_org);
     for(col=0;col<ncol;col++)
       {/* DO COL = 1,NCOL */
	 intCol = col + colOffset;
	 intRow = row + rowOffset;
	 buf[(intCol+(intRow*largestDim))] = 0.0;

	 yy       = ((float)col-y_org)*((float)col-y_org);
	 sum      = xx + yy;
	 if (sum > rad_sqr) 
	   goto nextCol;
	 dist_sum = sqrt(rad_sqr - sum);
	 dist     = z_org + dist_sum;
	 rad_sec  = (int)dist;
	 sec_factor = dist - ((float)rad_sec);
	 if (rad_sec >= ndepth) 
	   rad_sec = ndepth-1;
	 if (rad_sec < 1) 
	   rad_sec = 1;
	 /*
	  *  For some reason the row and col got messed in coremap???
	  *  DENS1 = COREMAP(COL,ROW,RAD_SEC)
	  *  DENS1 = COREMAP(rad_sec,col,row)
	  */
	 dens1 = 0.0; 

	 secn = rad_sec;

	 /* Get dens1 based on the rotation of phi, omega,
	  * and theta */
	 newTranslateThetaPhiOmega(&thisCol,&thisRow,&thisSecn,
				   xCen,yCen,zCen,
				   col,row,secn,
				   t1,t2,t3,t4,t5,t6,t7,t8,t9);

	 dens1 = getVoxel(coremap,thisCol,thisRow,thisSecn,
			  theta, phi, omega,
			  ncol, nrow);
	 
	 dens1 *= thisIrev;
		  
	 if (rad_sec >= ndepth-1) 
	   {
	     intCol = col + colOffset;
	     intRow = row + rowOffset;
	     buf[(intCol+(intRow*ncol))] = dens1 - smallestDens;
	   }
	 else
	   {/* DO other */
	     dens2 = 0.0; 

	     secn = rad_sec + 1;

	     /* Get dens2 based on the rotation of phi, omega,
	      * and theta */
	     newTranslateThetaPhiOmega(&thisCol,&thisRow,&thisSecn,
				       xCen,yCen,zCen,
				       col,row,secn,
				       t1,t2,t3,t4,t5,t6,t7,t8,t9);

	     dens2 = getVoxel(coremap,thisCol,thisRow,thisSecn,
			      theta, phi, omega,
			      ncol, nrow);
	 
	     dens2 *= thisIrev;
		  
	     dens_diff1 = dens1*(1.0-sec_factor) ;
	     dens_diff2 = dens2*sec_factor;
	     intCol = col + colOffset;
	     intRow = row + rowOffset;
	     buf[(intCol+(intRow*largestDim))]   = (dens_diff1 + dens_diff2) - 
	       smallestDens;
	   }/* DO other */

	 intCol = col + colOffset;
	 intRow = row + rowOffset;
	 map_min = MIN(map_min,buf[(intCol+(intRow*largestDim))]);  /*GLOBAL MIN*/
	 map_max = MAX(map_max,buf[(intCol+(intRow*largestDim))]);  /*GLOBAL MAX*/

       nextCol:
	 dummyl = 0;  /* Do nothing... */
       }/* DO COL = 1,NCOL */
     
     
   }/* DO ROW = 1,NROW  ! PROCESS DATA ROW BY ROW */


}/* rad_dens_fast_rob_C */





/****************************************************************************
C  **************************************************************************
C  *                                                                        *
C  *                *****     SUBROUTINE DPH_RESCALE_rob *****              *
C  *                                                                        *
C  *    RESCALE THE DATA STORED IN CORE MEMORY (DPHSHD_MAP) OR ON THE       *
C  *     SCRATCH DISK.                                                      *
C  *                                                                        *
C  *    APRIL 1991  PURDUE UNIVERSITY VAX 8550, T. BAKER.                   *
C  *    LAST MODIFIED  1-MAY-1991 BY T. BAKER.                              *
C  **************************************************************************
C
* Ported to C 1996 RWA
* 
* Rescale all frames to consistent levels.
* 
*/
void dph_rescale_rob (int ncol, int nrow, int num_views, 
		      cue_t *cuedMap, int tryColors, 
		      int *radialStepSize, int hiColor)
{/* DPH_RESCALE */
  int col, row, view;
  int thisRow,tvPtr,i;
  float *dphshd_map;
  float thisMin, thisMax;
  float scaleFactor;

  scaleFactor = ((float)tryColors-7) / (map_max-map_min);

  if(radialMapShade)
    {/* Compute the stepSize */

      /* Compute the stepsize */
      *radialStepSize = (int)(map_max - map_min);
#ifdef WE_DONT_GET_FROM_INTERFACE
      expandStep = ((tryColors-7)/4)/ *radialStepSize;
      expandStep = MAX(1,expandStep);
      expandStep = 8;
#endif
      *radialStepSize = (int)map_max;
      /* Minus 7 because we want to make sure we see everything
       * in the colormap */
      scaleFactor = ((float)tryColors-7) / (map_max-map_min);
      scaleFactor /= (float)expandStep;

      /* Now let's compute what radius will be what colormap entry */
      for(i=0;i<256;i++)
	radialValue[i]= 0.0;
      for(i=0;i<(tryColors-5);i++)
	{/* Do all possibles */
	  if(hiColor)
	    radialValue[(7+i)]=  (float)
	      (
	       (float)((float)i/(float)expandStep)  /
	       ((float)scaleFactor)
	       + (float)map_min);
	  else
	    radialValue[(pixels_return[0]+7+i)]=  (float)
	      (
	       (float)((float)i/(float)expandStep)  /
	       ((float)scaleFactor)
	       + (float)map_min);
	}/* Do all possibles */
    }/* Compute the stepSize */

  thisMin=999999.0; thisMax= -999999.0;  
  thisMin = HUGE;   /* dgm HUGE instead of 999999.0 */
  thisMax = -HUGE;  /* dgm -HUGE instead of -999999.0 */  
  for(view = 0;view<num_views;view++)
    {/* view=1 to numviews */

      /* Always scale the shaded one - if this is unshaded the .shad
       * buffer contains the cued data and it's the one being displayed
       */
      if(radialMapShade && secondMapShade)
	dphshd_map = cuedMap[view].radial;
      else dphshd_map = cuedMap[view].shad;

      for(row = 0;row < nrow; row++)
	{/* row */
	  thisRow= (row*ncol);
	  tvPtr= thisRow;

	  for(col = 0; col < ncol;col++)
	    {/* col */
	      
	      if(dphshd_map[(tvPtr+col)] != 0.0)
		{/* Then let's scale it */
		  if(radialMapShade)
		    {/* Do it in INT to make "holes" in colormap that
		      * we can exploit to put the shading into */
		      dphshd_map[(tvPtr+col)] = 
			(float)(
				((int)
				 ((((float)dphshd_map[(tvPtr+col)])-
				   map_min) 
				  * 
				  scaleFactor)
				 * (int)expandStep)
				) ;

		      *radialStepSize = (int)(MIN(*radialStepSize,
						  (int)dphshd_map[(tvPtr+col)]
						  )); 
		      /* We just need this to be the expandStep don't
			 * we??? */
		      *radialStepSize = (int)expandStep;
		    }/* Do it in INT to make "holes" in colromap that
		      * we can exploit to put the shading into */
		  else
		    {/* depth cued */
		      dphshd_map[(tvPtr+col)] = 
			(float)(
				(
				 ((float)dphshd_map[(tvPtr+col)])-
				 map_min
				 ) * 
				scaleFactor
				) ;
		    }/* depth cued */
		}/* Then let's scale it */
	      
	      if(dphshd_map[(tvPtr+col)]  > thisMax)
		thisMax = (float)dphshd_map[(tvPtr+col)];
	      if(dphshd_map[(tvPtr+col)]  < thisMin)
		thisMin = (float)dphshd_map[(tvPtr+col)];
	      
	    }/* col */
	  
	}/* row */
      
    }/* view=1 to numviews */

  map_max = thisMax;
  map_min = thisMin;

}/* DPH_RESCALE */






/************************************************************************
C  **********************************************************************
C  *                                                                    *
C  *                  *****  SUBROUTINE MAP_1D_rob *****                *
C  *                                                                    *
C  *    COMPUTE SPHERICALLY-AVERAGED DENSITY	                        *
C  *                                                                    *
C  *    SEPTEMBER 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    MODIFIED 29-MAR-1991 BY T. BAKER.                               *
C  *	Converted to Xwindows, November 1993, by Eric Eltzroth		*
C  *	Modified 22 June 1995, by D. Belnap				*
C  **********************************************************************
C
*/
void map_1d_rob (float *map, int ncol, int nrow, int nsec, 
		 float xcen, float ycen, float zcen,
		 int interp, int nterp, float max_r, 
		 float *rint, int *irnum, float *sum_rint,
		 float *sumsq_rint, float *stdev)
{/* map_1d_rob */
#ifdef ORIGINAL_WAY
  int    ncol, nrow, nsec, inum, ir, interp, ipuck;
  int    nterp, i, j, j1, j2, k, ixtab, iytab, irnum(nterp);
  float       xcen, ycen, zcen, rint(nterp), rint_min, rint_max;
  float       map(ncol,nrow,nsec), x, y, z, ysq, zsq, max_r;
  float       test, radius, rsq_lim, fraction;
  float       sum_rint(nterp), sumsq_rint(nterp), stdev(nterp);
  float       sum_sq_over_n, numerator, denominator;
  float thisFloat;
  BYTE      outfile, plot;
  char out_filename[64];
#else
  int    inum, ir, ipuck, kTncolTnrow, jTncol;
  int    i, j, j1, j2, k, ixtab, iytab;
  float  rint_min, rint_max;
  float  x, y, z, ysq, zsq;
  float  test, radius, rsq_lim, fraction;
  float  sum_sq_over_n, numerator, denominator;
  float  thisFloat;
  BYTE   outfile, plot;
#endif

  inum    = nterp;
  rsq_lim = max_r*max_r;
  
  for(i=0;i<inum;i++)
    {/* do i = 1,inum ! initialize 1d interpolation array */
      irnum[i]      = 0;
      rint[i]       = 0.0;
      sum_rint[i]   = 0.0;
      sumsq_rint[i] = 0.0;
    }/* do i = 1,inum ! initialize 1d interpolation array */

  /*
    c  **************************************************************
    c  *    step through image and reduce to 1d circular average    *
    c  **************************************************************
    */

  for(k=0;k<nsec;k++)
    {/* do k = 1,nsec */
      z   = (float)(k) - zcen;
      zsq = z*z;
      kTncolTnrow= k*ncol*nrow;
      for(j=0;j<nrow;j++)
	{/* do j = 1,nrow */
	  y   = (float)(j) - ycen;
	  ysq = y*y;
	  jTncol = j * ncol;
	  for(i=0;i<ncol;i++)
	    {/* do i = 1,ncol */
	      x    = (float)(i) - xcen;
	      test = x*x + ysq + zsq;
	      if(test <= rsq_lim) 
		{/* then */
		  ir             = (int)(interp*sqrt(test)) ;
		  irnum[ir]      = irnum[ir] + 1;
		  thisFloat = map[((kTncolTnrow)+(jTncol)+i)];
		  rint[ir]       = rint[ir] + thisFloat;
		  sumsq_rint[ir] = sumsq_rint[ir] + (thisFloat * thisFloat);
		}/* then */
	    }/* do i = 1,ncol */
	}/* do j = 1,nrow */
    }/* do k = 1,nsec */

  /*
    c  ********************************
    c  *    normalize the 1d array    *
    c  ********************************
    */
  rint_min = r4_max;
  rint_max = r4_min;


  for(i=0;i<inum;i++)
    {/* do i = 1,inum */
      if(irnum[i] > 1) 
	{/* then */
	  sum_rint[i] = rint[i];	      /*! for st. dev. calculation*/
	  rint[i]     = rint[i]/(float)(irnum[i]);
	}/* then */
      
      rint_min = MIN(rint_min,rint[i]);
      rint_max = MAX(rint_max,rint[i]);
    }/* do i = 1,inum */

  /*
    c  ***************************************
    c  *    now try to fill in the blanks    *
    c  ***************************************
    */
  for(i=1;i<(inum-2);i++)
    {/* do i = 2,(inum-2) */
      /* I'm thinking we just want to go less than zero - e.g. the corners
       * are at zero so don't iunterpolate those ... ???
       if(irnum[i]<=0) 
      */
      if(irnum[i]<0) 
	{/* then */
	  j1 = i;
	  while(irnum[j1]==0 && j1 > 0)	/* ! find last point */
	    j1 = j1 - 1;
	  j2 = i;
	  while(irnum[j2]==0 && j2<inum)	/* ! find next point */
	    j2 = j2 + 1;
	  fraction = (float)(j2-i)/(float)(j2-j1);
	  rint[i]  = rint[j1]*fraction + rint[j2]*(1.0-fraction);
	}/* then */
    }/* do i = 2,(inum-2) */
  
  /*
    c  ***************************************************************
    c  *  compute standard deviation at each radius where irnum > 1  *
    c  ***************************************************************
    */
  for(i=0;i<inum;i++)
    {/* do i = 1,inum */
      if (irnum[i] > 1) 
	{/* then */
	  sum_sq_over_n = (sum_rint[i]*sum_rint[i])/(float)(irnum[i]);
	  numerator     = sumsq_rint[i] - sum_sq_over_n;
	  denominator   = (float)(irnum[i] - 1);
	  stdev[i]      = sqrt(numerator/denominator);
	}/* then */
      else
	stdev[i] = 0.0;
    }/* do i = 1,inum */
  


}/* MAP_1D_ROB */



/***************************************************************************
C  **********************************************************************
C  *                                                                    *
C  *                  *****  SUBROUTINE IMG_1D_rob  *****               *
C  *                                                                    *
C  *    COMPUTE CIRCULARLY-AVERAGED DENSITY				*
C  *                                                                    *
C  *    DECEMBER 1986  PURDUE UNIVERSITY VAX 11/750, T. BAKER           *
C  *    MODIFIED 15-JUL-1990 BY D. BELNAP.                              *
C  *    MODIFIED 23-Nov-1993 By Eric Eltzroth				*
C  *	Modified 12 Jan 1995 by D. Belnap				*
C  **********************************************************************
C
*/
void img_1d_rob(float *img,
		int ncol, int nrow,
		float xcen, float ycen,
		int interp, int nterp,
		float *rint, int *irnum,
		float img_bkg)
{/* img_1d_rob */
  int i, j, k;
  int inum, ir;
  float rint_min, rint_max;
  float    x, y, ysq, test, radius;

  inum = nterp*2;

  for(i=0;i<inum+2;i++)
    {/* do i = 1,inum  ! initialize 1d interpolation array */
      rint[i]  = 0.0;
      irnum[i] = 0;
    }/* do i = 1,inum  ! initialize 1d interpolation array */
  
  /* c  **************************************************************
     c  *    step through image and reduce to 1d circular average    *
     c  **************************************************************/
  for(j=0;j<nrow;j++)
    {/* do j = 1,nrow */
      y = ((float)j) - ycen;
      ysq = y*y;
      for(i=0;i<ncol;i++)
	{/* do i = 1,ncol */
	  x         = ((float)i) - xcen;
	  ir        = (int)(interp*sqrt(x*x+ysq)) ;
	  if(TRUE)
	    {/* then */
	      irnum[ir] = irnum[ir] + 1;
	      rint[ir]  = rint[ir] + (float)(img[((i*ncol)+j)] - 
					     (float)img_bkg);
	    }/* then */
	}/* do i = 1,ncol */
    }/* do j = 1,nrow */
  
  rint_min = 
    rint_max = rint[0];

  /* c
     c  ********************************
     c  *    normalize the 1d array    *
     c  ********************************/
  for(i=0;i<inum;i++)
    {/* do i = 1,inum */
      if (irnum[i] > 1)  
	rint[i] = rint[i]/((float)irnum[i]);
    }/* do i = 1,inum */

  /* c  ********************************************************************
     c  *   interpolate density where irnum = 0, subtract background, output
     c  ********************************************************************/
  for(i=0;i<inum;i++)
    {/* do i = 1,inum */
      if (irnum[i]==0 && i==0) 
	rint[0] = rint[1];
      if (irnum[i]==0 && (i!=0 && i<inum)) 
	  rint[i] = (rint[i-1] + rint[i+1])/2;
    }/* do i = 1,inum */


}/* img_1d_rob */



/******************************************************************
* ricardo_routine() -                                             *
*                    Program to impose plane group symmetry on    *
*                    an image motif to create a whole unit cell.  *
*                                                                 *
* April 30,1999, Ricardo Bernal                                   *
*******************************************************************/
void ricardo_routine(int phaseGroupSymmetry,float **float_image_buffer,
                     int *cdNumCols, int *cdNumRows,
                     float origX, float origY)
{
  float *temp_image_buffer,*newimage;
  int newCols;
  int newRows;
  int i,j;                           /* loop control variables */
  float a11,a12,a13,a21,a22,a23;
  float b11,b12,b13,b21,b22,b23;
  float c11,c12,c13,c21,c22,c23;
  float d11,d12,d13,d21,d22,d23;
  float e11,e12,e13,e21,e22,e23;
  float f11,f12,f13,f21,f22,f23;   /* a-k are variables for       */
  float g11,g12,g13,g21,g22,g23;   /* matrix symmetry operations  */
  float h11,h12,h13,h21,h22,h23;   /* 13 & 23 are the translation */
  float i11,i12,i13,i21,i22,i23;
  float j11,j12,j13,j21,j22,j23;
  float k11,k12,k13,k21,k22,k23;

  int oldx,oldy,newx,newy;   /* coordinate variables           */
  int bgx;                   /* variables for background value */
  int bgy;
  float background;
  float *thisBuffer;

 if(origX == 0.0)
   origX = 1.0;
 if(origY == 0.0)
   origY = 1.0;

 if(phaseGroupSymmetry != 1) /** If its P1 then just fall out of routine **/
 {
  newRows = *cdNumRows * 2;
  newCols = *cdNumCols * 2;
  if(phaseGroupSymmetry == 3)
  {
    newCols = *cdNumCols;
  }

  newimage=(float *)malloc(sizeof(float)*(newCols)*(newRows));

  /*********  Initialize the new array to make even background **********/

  bgx = *cdNumRows;
  bgy = *cdNumCols;
  thisBuffer = *float_image_buffer;

  for(i=*cdNumRows;i > (*cdNumRows-(*cdNumRows/20));i--)
  {
    for(j=*cdNumCols;j > (*cdNumCols-(*cdNumCols/20));j--)
    {
      if(thisBuffer[((i*(*cdNumCols))+j)] > 
	 thisBuffer[((bgx*(*cdNumCols))+bgy)])
      {
        bgx = i;
        bgy = j;
      }
    }
  }
  for(i=0;i < newRows;i++)
  {
    for(j=0;j < newCols;j++)
    {
      newimage[((i*(newCols))+j)]=thisBuffer[((bgx*(*cdNumCols))+bgy)];
    }
  }

  /***************** Fill the new array with the old image **********/

  for(i=0;i < (newRows);i++)
  {
    for(j=0;j < (newCols);j++)
    {
      if(i < *cdNumRows && j < *cdNumCols)
        newimage[((i*(newCols))+j)]=thisBuffer[((i*(*cdNumCols))+j)];
    }
  }

  /*************** Generate symmetry related motifs ***********/

  if(phaseGroupSymmetry==2)   /* p2 */
    {/* P2 selected */
      a11 = -1*origX;   /* the convention is positive x down (with rows) */
      a12 =  0*origX;   /* and positive y to the right (with columns)    */
      a21 =  0*origY;   /*  a11  a12  */
      a22 = -1*origY;   /*  a21  a22  */
      
      for(oldx=1;oldx <= (*cdNumRows);oldx++)
	{
	  for(oldy=1;oldy <= (*cdNumCols);oldy++)
	    {
	      newx = ((oldx*a11) + (oldy*a21));
	      newy = ((oldx*a12) + (oldy*a22));
	      if(newx < 0)
		newx = (int)(newx + newRows*origX);
	      if(newy < 0)
		newy = (int)(newy + newCols*origY);
	      
	      if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
		 thisBuffer[((bgx*(*cdNumCols))+bgy)])
		newimage[((newx-1)*(newCols))+(newy-1)] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
	      if(newimage[((newx-1)*(newCols))+(newy-1)] > 255)
		newimage[((newx-1)*(newCols))+(newy-1)] = 255;
	      if(newimage[((newx-1)*(newCols))+(newy-1)] < 0)
		newimage[((newx-1)*(newCols))+(newy-1)] = 0;
	      else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		      thisBuffer[((bgx*(*cdNumCols))+bgy)])
		newimage[(((newx-1)*(newCols))+(newy-1))] +=  
		  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
		  thisBuffer[((bgx*(*cdNumCols))+bgy)];
	      if(newimage[((newx-1)*(newCols))+(newy-1)] > 255)
		newimage[((newx-1)*(newCols))+(newy-1)] = 255;
	      if(newimage[((newx-1)*(newCols))+(newy-1)] < 0)
		newimage[((newx-1)*(newCols))+(newy-1)] = 0;
	    }
	}
    }/* P2 selected */
  else if(phaseGroupSymmetry==3)   /* pm */
  {
    a11 = -1;      
    a12 =  0;      
    a21 =  0;
    a22 =  1;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21));
        newy = ((oldx*a12) + (oldy*a22));
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;

        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==4)  /* pg */
  {
    a11 = -1;      
    a12 =  0;      
    a13 =  0;
    a21 =  0;
    a22 =  1;
    a23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = ((oldx*a12) + (oldy*a22)) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;

        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==5)   /* cm */
  {
    a11 = -1;      
    a12 =  0;      
    a13 =  0;
    a21 =  0;
    a22 =  1;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = ((oldx*a12) + (oldy*a22)) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;

        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 =  1;
    b12 =  0;
    b13 =  *cdNumRows;
    b21 =  0;
    b22 =  1;
    b23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*b11) + (oldy*b21)) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 = -1;
    c12 =  0;
    c13 =  *cdNumRows;
    c21 =  0;
    c22 =  1;
    c23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;

        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==6)  /* p2mm */
  {
    a11 = -1;      
    a12 =  0;      
    a13 =  0;
    a21 =  0;
    a22 = -1;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = ((oldx*a12) + (oldy*a22)) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;

        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 = -1;
    b12 =  0;
    b13 =  0;
    b21 =  0;
    b22 =  1;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*b11) + (oldy*b21)) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 =  1;
    c12 =  0;
    c13 =  0;
    c21 =  0;
    c22 = -1;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==7)  /* p2mg */
  {
    a11 = -1;      
    a12 =  0;      
    a13 =  0;
    a21 =  0;
    a22 = -1;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = ((oldx*a12) + (oldy*a22)) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 = -1;
    b12 =  0;
    b13 =  *cdNumRows;
    b21 =  0;
    b22 =  1;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*b11) + (oldy*b21)) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 =  1;
    c12 =  0;
    c13 =  *cdNumRows;
    c21 =  0;
    c22 = -1;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==8)  /* p2gg */
  {
    a11 = -1;      
    a12 =  0;      
    a13 =  0;
    a21 =  0;
    a22 = -1;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = ((oldx*a12) + (oldy*a22)) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 = -1;
    b12 =  0;
    b13 =  *cdNumRows;
    b21 =  0;
    b22 =  1;
    b23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*b11) + (oldy*b21)) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 =  1;
    c12 =  0;
    c13 =  *cdNumRows;
    c21 =  0;
    c22 = -1;
    c23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==9)  /* c2mm */
  {
    a11 = -1;      
    a12 =  0;      
    a13 =  0;
    a21 =  0;
    a22 = -1;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = ((oldx*a12) + (oldy*a22)) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 = -1;
    b12 =  0;
    b13 =  0;
    b21 =  0;
    b22 =  1;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*b11) + (oldy*b21)) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 =  1;
    c12 =  0;
    c13 =  0;
    c21 =  0;
    c22 = -1;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    d11 =  1;      
    d12 =  0;      
    d13 =  *cdNumRows;
    d21 =  0;
    d22 =  1;
    d23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*d11) + (oldy*d21)) + d13;
        newy = ((oldx*d12) + (oldy*d22)) + d23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    e11 = -1;
    e12 =  0;
    e13 =  *cdNumRows;
    e21 =  0;
    e22 = -1;
    e23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*e11) + (oldy*e21)) + e13;
        newy = ((oldx*e12) + (oldy*e22)) + e23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    f11 = -1;
    f12 =  0;
    f13 =  *cdNumRows;
    f21 =  0;
    f22 =  1;
    f23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*f11) + (oldy*f21)) + f13;
        newy = ((oldx*f12) + (oldy*f22)) + f23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    g11 =  1;
    g12 =  0;
    g13 =  *cdNumRows;
    g21 =  0;
    g22 = -1;
    g23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*g11) + (oldy*g21)) + g13;
        newy = ((oldx*g12) + (oldy*g22)) + g23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==10)  /* p4 */
  {
    a11 = -1;      
    a12 =  0;      
    a13 =  0;
    a21 =  0;
    a22 = -1;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = ((oldx*a12) + (oldy*a22)) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 =  0;
    b12 =  1;
    b13 =  0;
    b21 = -1;
    b22 =  0;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*b11) + (oldy*b21)) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 =  0;
    c12 = -1;
    c13 =  0;
    c21 =  1;
    c22 =  0;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==11)  /* p4mm */
  {
    a11 = -1;     
    a12 =  0;      
    a13 =  0;
    a21 =  0;
    a22 = -1;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = ((oldx*a12) + (oldy*a22)) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 =  0;
    b12 =  1;
    b13 =  0;
    b21 = -1;
    b22 =  0;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*b11) + (oldy*b21)) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 =  0;
    c12 = -1;
    c13 =  0;
    c21 =  1;
    c22 =  0;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    d11 = -1;      
    d12 =  0;      
    d13 =  0;
    d21 =  0;
    d22 =  1;
    d23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*d11) + (oldy*d21)) + d13;
        newy = ((oldx*d12) + (oldy*d22)) + d23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    e11 =  1;
    e12 =  0;
    e13 =  0;
    e21 =  0;
    e22 = -1;
    e23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*e11) + (oldy*e21)) + e13;
        newy = ((oldx*e12) + (oldy*e22)) + e23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    f11 =  0;
    f12 =  1;
    f13 =  0;
    f21 =  1;
    f22 =  0;
    f23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*f11) + (oldy*f21)) + f13;
        newy = ((oldx*f12) + (oldy*f22)) + f23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    g11 =  0;
    g12 = -1;
    g13 =  0;
    g21 = -1;
    g22 =  0;
    g23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*g11) + (oldy*g21)) + g13;
        newy = ((oldx*g12) + (oldy*g22)) + g23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==12)  /* p4gm */
  {
    a11 = -1;      
    a12 =  0;      
    a13 =  0;
    a21 =  0;
    a22 = -1;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = ((oldx*a12) + (oldy*a22)) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 =  0;
    b12 =  1;
    b13 =  0;
    b21 = -1;
    b22 =  0;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*b11) + (oldy*b21)) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 =  0;
    c12 = -1;
    c13 =  0;
    c21 =  1;
    c22 =  0;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    d11 = -1;      
    d12 =  0;      
    d13 =  *cdNumRows;
    d21 =  0;
    d22 =  1;
    d23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*d11) + (oldy*d21)) + d13;
        newy = ((oldx*d12) + (oldy*d22)) + d23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    e11 =  1;
    e12 =  0;
    e13 =  *cdNumRows;
    e21 =  0;
    e22 = -1;
    e23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*e11) + (oldy*e21)) + e13;
        newy = ((oldx*e12) + (oldy*e22)) + e23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    f11 =  0;
    f12 =  1;
    f13 =  *cdNumRows;
    f21 =  1;
    f22 =  0;
    f23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*f11) + (oldy*f21)) + f13;
        newy = ((oldx*f12) + (oldy*f22)) + f23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    g11 =  0;
    g12 = -1;
    g13 =  *cdNumRows;
    g21 = -1;
    g22 =  0;
    g23 =  *cdNumCols;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*g11) + (oldy*g21)) + g13;
        newy = ((oldx*g12) + (oldy*g22)) + g23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==13)  /* p3 */
  {
    a11 =  0;      
    a12 =  1;      
    a13 =  0;
    a21 = -1;
    a22 =  0;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = (oldx - oldy) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 =  0;
    b12 = -1;
    b13 =  0;
    b21 = -1;
    b22 =  0;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (-oldx + oldy) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==14)  /* p3m1 */
  {
    a11 =  0;      
    a12 =  1;      
    a13 =  0;
    a21 = -1;
    a22 =  0;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = (oldx - oldy) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 =  0;
    b12 = -1;
    b13 =  0;
    b21 = -1;
    b22 =  0;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (-oldx + oldy) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 =  0;
    c12 = -1;
    c13 =  0;
    c21 = -1;
    c22 =  0;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    d11 =  1;
    d12 =  0;
    d13 =  0;
    d21 =  0;
    d22 =  1;
    d23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (-oldx + oldy) + d13;
        newy = ((oldx*d12) + (oldy*d22)) + d23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    e11 =  1;
    e12 =  0;
    e13 =  0;
    e21 =  0;
    e22 =  1;
    e23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*e11) + (oldy*e21)) + e13;
        newy = (oldx-oldy) + e23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==15)  /* p31m */
  {
    a11 =  0;      
    a12 =  1;      
    a13 =  0;
    a21 = -1;
    a22 =  0;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = (oldx - oldy) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 =  0;
    b12 = -1;
    b13 =  0;
    b21 = -1;
    b22 =  0;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (-oldx + oldy) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 =  0;
    c12 =  1;
    c13 =  0;
    c21 =  1;
    c22 =  0;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    d11 =  1;
    d12 =  0;
    d13 =  0;
    d21 =  0;
    d22 = -1;
    d23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (oldx - oldy) + d13;
        newy = ((oldx*d12) + (oldy*d22)) + d23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    e11 = -1;
    e12 =  0;
    e13 =  0;
    e21 =  0;
    e22 =  1;
    e23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*e11) + (oldy*e21)) + e13;
        newy = (-oldx + oldy) + e23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==16)  /* p6 */
  {
    a11 =  0;      
    a12 =  1;      
    a13 =  0;
    a21 = -1;
    a22 =  0;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = (oldx - oldy) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 =  0;
    b12 = -1;
    b13 =  0;
    b21 = -1;
    b22 =  0;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (-oldx + oldy) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 = -1;
    c12 =  0;
    c13 =  0;
    c21 =  0;
    c22 = -1;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    d11 =  0;
    d12 =  1;
    d13 =  0;
    d21 =  1;
    d22 =  0;
    d23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*d11) + (oldy*d21)) + d13;
        newy = (-oldx + oldy) + d23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    e11 =  0;
    e12 =  1;
    e13 =  0;
    e21 =  1;
    e22 =  0;
    e23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (oldx - oldy) + e13;
        newy = ((oldx*e12) + (oldy*e22)) + e23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }
  else if(phaseGroupSymmetry==17)  /* p6mm */
  {
    a11 =  0;      
    a12 =  1;      
    a13 =  0;
    a21 = -1;
    a22 =  0;
    a23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*a11) + (oldy*a21)) + a13;
        newy = (oldx - oldy) + a23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    b11 =  0;
    b12 = -1;
    b13 =  0;
    b21 = -1;
    b22 =  0;
    b23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (-oldx + oldy) + b13;
        newy = ((oldx*b12) + (oldy*b22)) + b23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    c11 = -1;
    c12 =  0;
    c13 =  0;
    c21 =  0;
    c22 = -1;
    c23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*c11) + (oldy*c21)) + c13;
        newy = ((oldx*c12) + (oldy*c22)) + c23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    d11 =  0;
    d12 =  1;
    d13 =  0;
    d21 =  1;
    d22 =  0;
    d23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*d11) + (oldy*d21)) + d13;
        newy = (-oldx + oldy) + d23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    e11 =  0;
    e12 =  1;
    e13 =  0;
    e21 =  1;
    e22 =  0;
    e23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (oldx - oldy) + e13;
        newy = ((oldx*e12) + (oldy*e22)) + e23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    f11 =  0;
    f12 = -1;
    f13 =  0;
    f21 = -1;
    f22 =  0;
    f23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*f11) + (oldy*f21)) + f13;
        newy = ((oldx*f12) + (oldy*f22)) + f23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    g11 =  1;
    g12 =  0;
    g13 =  0;
    g21 =  0;
    g22 =  1;
    g23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (-oldx + oldy) + g13;
        newy = ((oldx*g12) + (oldy*g22)) + g23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    h11 =  1;
    h12 =  0;
    h13 =  0;
    h21 =  0;
    h22 =  1;
    h23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*h11) + (oldy*h21)) + h13;
        newy = (oldx-oldy) + h23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    i11 =  0;
    i12 =  1;
    i13 =  0;
    i21 =  1;
    i22 =  0;
    i23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*i11) + (oldy*i21)) + i13;
        newy = ((oldx*i12) + (oldy*i22)) + i23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    j11 =  1;
    j12 =  0;
    j13 =  0;
    j21 =  0;
    j22 = -1;
    j23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = (oldx - oldy) + j13;
        newy = ((oldx*j12) + (oldy*j22)) + j23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
    k11 = -1;
    k12 =  0;
    k13 =  0;
    k21 =  0;
    k22 =  1;
    k23 =  0;

    for(oldx=1;oldx <= (*cdNumRows);oldx++)
    {
      for(oldy=1;oldy <= (*cdNumCols);oldy++)
      {
        newx = ((oldx*k11) + (oldy*k21)) + k13;
        newy = (-oldx + oldy) + k23;
        if(newx < 0)
          newx = newx + newRows;
        if(newy < 0)
          newy = newy + newCols;
        if(newimage[(((newx-1)*(newCols))+(newy-1))] > 
	   thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] += 
                  (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))]);
        else if(newimage[(((newx-1)*(newCols))+(newy-1))] == 
		thisBuffer[((bgx*(*cdNumCols))+bgy)])
          newimage[(((newx-1)*(newCols))+(newy-1))] +=  
           (thisBuffer[(((oldx-1)*(*cdNumCols))+(oldy-1))])-
	    thisBuffer[((bgx*(*cdNumCols))+bgy)];
      }
    }
  }

  /********* Transfer new values back **********/
  free(*float_image_buffer);
  *float_image_buffer=(float *)malloc(sizeof(float)*(newCols)*(newRows));

  temp_image_buffer=*float_image_buffer;
  for(i=0;i < newRows;i++)
    for(j=0;j < newRows;j++)
    {
      temp_image_buffer[((i*(newCols))+j)]=newimage[((i*newCols)+j)];
    }
  free(newimage);
  *cdNumRows = newRows;
  *cdNumCols = newCols;
  *float_image_buffer = temp_image_buffer;

  /********* Transfer new values back **********/

    /* END RICARDO_ROUTINE */
 }
} 


/* End of emUnix.c     */
